<?php

namespace WPEP;

use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

/**
 * Class API
 * @package WPEP
 * @author Robert Rusu
 */
class API {

  /**
   * @var API
   */
  protected static $instance;

  /**
   * @return API
   */
  public static function instance() {
    if (!isset(self::$instance))
      self::$instance = new self();

    return self::$instance;
  }

  protected $_prefix      = 'wpep';
  protected $_request_map = [];

  public function setup() {
    foreach( $this->get_default_request_map() as $namespace => $current_namespace_data )
      $this->register( $namespace, $current_namespace_data[ 'method_map' ], $current_namespace_data[ 'args' ] );
  }

  public function get_request_list() {
    $response = [];

    foreach( $this->_request_map as $namespace => $method_map ) {
      foreach( $method_map as $method_name => $method_data ) {
        if( !isset( $response[ $namespace ] ) )
          $response[$namespace ] = [];

        $response[ $namespace ][ $namespace . '/' . $method_name ] = $namespace . ' - ' . $method_name;
      }
    }

    arsort( $response );

    return $response;
  }

  public function get_request_map() {
    return $this->_request_map;
  }

  public function register_content_type( $post_type, $method_map, $args = [] ) {
    $this->register( 'content-type/' . $post_type, $method_map, $args );
  }

  public function register( $namespace, $method_map, $args = [] ) {
    $args = apply_filters( 'wpep_api_register_args', $args, $namespace );

    $args = wp_parse_args( $args, [
      'type' => WPEP_API_METHOD_TYPE_PRIVATE
    ] );

    if( !isset( $this->_request_map[ $namespace ] ) )
      $this->_request_map[ $namespace ] = [];

    foreach( $method_map as $method_name => $method_handler ) {
      if( isset( $this->_request_map[ $namespace ][ $method_name ] ) )
        continue;

      if( is_array( $method_handler ) ) {
        $this->_request_map[ $namespace ][ $method_name ] = wp_parse_args( $method_handler, $args );
      } else {
        $this->_request_map[ $namespace ][ $method_name ] = $args;
        $this->_request_map[ $namespace ][ $method_name ][ 'callback' ] = $method_handler;
      }

      if( $this->_request_map[ $namespace ][ $method_name ][ 'type' ] === WPEP_API_METHOD_TYPE_PUBLIC )
        add_action( 'wp_ajax_nopriv_' . $this->_prefix . '/' . $namespace . '/' . $method_name, [ $this, '_wp_ajax_handle_request' ] );

      if( $this->_request_map[ $namespace ][ $method_name ][ 'type' ] === WPEP_API_METHOD_TYPE_ADMIN )
        add_action( 'wp_ajax_' . $this->_prefix . '/' . $namespace . '/' . $method_name, [ $this, '_wp_ajax_handle_admin_request'] );
      else
        add_action( 'wp_ajax_' . $this->_prefix . '/' . $namespace . '/' . $method_name, [ $this, '_wp_ajax_handle_request' ] );
    }
  }

  public function request_exists_by_key( $key_identifier ) {
    $action    = substr( $key_identifier, strrpos( $key_identifier, '/' ) + 1 );
    $namespace = trim( substr( $key_identifier, 0, strlen( $key_identifier ) - strlen( $action ) ), '/' );

    if( !isset( $this->_request_map[ $namespace ] ) || !isset( $this->_request_map[ $namespace ][ $action ] ) )
      return false;

    return true;
  }

  public function get_request_response_by_key( $key_identifier, $request_data ) {
    $action    = substr( $key_identifier, strrpos( $key_identifier, '/' ) + 1 );
    $namespace = trim( substr( $key_identifier, 0, strlen( $key_identifier ) - strlen( $action ) ), '/' );

    return $this->get_request_response( $namespace, $action, $request_data );
  }

  public function get_request_response( $namespace, $action, $request_data ) {
    if( !isset( $this->_request_map[ $namespace ] ) || !isset( $this->_request_map[ $namespace ][ $action ] ) )
      return [
        'status'  => 'error',
        'error'   => __( "Invalid Request", "wpep" )
      ];

    $current_request_data = $this->_request_map[ $namespace ][ $action ];

    if( !isset( $current_request_data[ 'callback' ] )
      || ( $current_request_data[ 'type' ] === WPEP_API_METHOD_TYPE_PRIVATE && !is_user_logged_in() ) )
      return [
        'status'  => 'error',
        'error'   => __( "Invalid Request", "wpep" )
      ];

    if( isset( $current_request_data[ 'user_can' ] ) ) {
      if( is_bool( $current_request_data[ 'user_can' ] ) ) {
        $has_access = $current_request_data[ 'user_can' ];
      } else if( is_callable( $current_request_data[ 'user_can' ] ) ) {
        $has_access = $current_request_data[ 'user_can' ]( $request_data );
      } else {
        $has_access = current_user_can( $current_request_data[ 'user_can' ] );
      }

      if( !$has_access )
        return [
          'status'  => 'error',
          'error'   => __( "Invalid Request", "wpep" )
        ];
    }

    if( wpep_get_setting_flag( 'data-system-api-tracking' ) ) {
      $system_log = WPEP_Entity_SystemLog::instance( 'api', $namespace . '/' . $action, time() );
      $system_log->started_arguments = $request_data;
      $system_log->save();
    }

    $response = ( is_callable( $current_request_data[ 'callback' ] ) ? apply_filters(
      'wpep_api_request_response_' . $namespace,
      call_user_func( $current_request_data[ 'callback' ], $request_data ),
      $action,
      $request_data
    ) : false );

    if( false === $response ) {
      $response = [
        'status'  => 'error',
        'error'   => __( "Invalid Request", "wpep" )
      ];
    } else if( true === $response ) {
      $response = [ 'status' => 'ok' ];
    }

    if( wpep_get_setting_flag( 'data-system-api-tracking' ) && isset( $system_log ) ) {
      $system_log->completed_arguments = $response;
      $system_log->completed_at = time();
      $system_log->save();
    }

    return $response;
  }

  public function get_default_request_map() {
    $response = [
      'authentication' => [
        'method_map' => [
          'express'        => '\WPEP\API\Authentication::express',
          'login'          => '\WPEP\API\Authentication::login',
          'register'       => '\WPEP\API\Authentication::register',
          'reset_password' => '\WPEP\API\Authentication::reset_password'
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PUBLIC
        ]
      ],
      'misc' => [
        'method_map' => [
          'allowed-mime-types'            => '\WPEP\API\Misc::allowed_mime_types',
          'format-timestamps-to-datetime' => '\WPEP\API\Misc::format_timestamps_to_datetime',
          'icons'                         => '\WPEP\API\Misc::icons',
          'date-modify-drip-options'      => '\WPEP\API\Misc::date_modify_drip_options',
          'content-type-search'           => '\WPEP\API\Misc::content_type_search_results'
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'user' => [
        'method_map' => [
          'account-update-profile'          => '\WPEP\API\User\Account::update_profile',
          'account-update-profile-picture'  => '\WPEP\API\User\Account::update_profile_picture',
          'account-update-profile-cover'    => '\WPEP\API\User\Account::update_profile_cover',
          'account-change-password'         => '\WPEP\API\User\Account::change_password',

          'upload-file' => '\WPEP\API\User::upload_file',

          'template-change-password' => '\WPEP\API\User::template_change_password',
          'template-notification-management'  => '\WPEP\API\User::template_notification_management',
          'notification-status-flag'          => '\WPEP\API\User::notification_status_flag',
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'user/auto-login' => [
        'method_map' => [
          'token-entries' => '\WPEP\API\User\AutoLogin::token_entries',
          'token-reset'   => '\WPEP\API\User\AutoLogin::token_reset',
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can' => '\WPEP\ACL\API::manage_settings'
        ]
      ],
      'discussion' => [
        'method_map' => [
          'user-notifications'            => [
            'callback' => '\WPEP\API\Discussion::notifications',
            'type'     => WPEP_API_METHOD_TYPE_PUBLIC
          ],
          'mention-user-search'           => '\WPEP\API\Discussion::mention_user_search',
          'template-popup'                => '\WPEP\API\Discussion::template_popup',
          'template-user-list-navigation' => '\WPEP\API\Discussion::template_user_list_navigation'
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'discussion/comments' => [
        'method_map' => [
          'entries' => '\WPEP\API\Discussion\Comments::entries',
          'replies' => '\WPEP\API\Discussion\Comments::replies'
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'discussion/comment' => [
        'method_map' => [
          'post'      => '\WPEP\API\Discussion\Comment::post',
          'pin'       => '\WPEP\API\Discussion\Comment::pin',
          'delete'    => '\WPEP\API\Discussion\Comment::delete',
          'react'     => '\WPEP\API\Discussion\Comment::react',
          'reactions' => '\WPEP\API\Discussion\Comment::reactions',
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'system/discussion' => [
        'method_map' => [
          'activity-report' => '\WPEP\API\System\Discussion::activity_report',
        ],
        'args' => [
          'type'    => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can'=> '\WPEP\ACL\API::manage_members'
        ]
      ],
      'system/email' => [
        'method_map' => [
          'entries'           => '\WPEP\API\System\Email::entries',
          'entries-analytics' => '\WPEP\API\System\Email::entries_analytics',
          'activity-report'   => '\WPEP\API\System\Email::activity_report'
        ],
        'args' => [
          'type'    => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can'=> '\WPEP\ACL\API::manage_members'
        ]
      ],
      'system/log' => [
        'method_map' => [
          'entries'              => '\WPEP\API\System\Log::entries',
          'activity-report'      => '\WPEP\API\System\Log::activity_report',
          'configuration-report' => '\WPEP\API\System\Log::configuration_report',
        ],
        'args' => [
          'type'    => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can'=> '\WPEP\ACL\API::manage_settings'
        ]
      ],
      'system/misc' => [
        'method_map' => [
          'template-authentication' => [
            'callback'  => '\WPEP\API\System\Misc::template_authentication',
            'type'      => WPEP_API_METHOD_TYPE_PUBLIC
          ],
          'menu-notifications' => [
            'callback'  => '\WPEP\API\System\Misc::menu_notifications',
            'user_can'  => '\WPEP\ACL\API::manage_settings'
          ],
          'update-course-user-data' => [
            'callback'  => '\WPEP\API\System\Misc::update_course_user_data',
            'user_can'  => '\WPEP\ACL\API::manage_members'
          ],
          'reset-course-user-progress' => [
            'callback'  => '\WPEP\API\System\Misc::reset_course_user_progress',
            'user_can'  => '\WPEP\ACL\API::manage_members'
          ],
          'reset-assessment-user-progress' => [
            'callback'  => '\WPEP\API\System\Misc::reset_assessment_user_progress',
            'user_can'  => '\WPEP\ACL\API::manage_members'
          ],
          'submit-assessment-review' => [
            'callback'  => '\WPEP\API\System\Misc::submit_assessment_review',
            'user_can'  => '\WPEP\ACL\API::manage_members'
          ],
        ],
        'args' => [
          'type'  => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'system/platform-health' => [
        'method_map' => [
          'export-plain'         => '\WPEP\API\System\PlatformHealth::export_plain',
          'export-sections-html' => '\WPEP\API\System\PlatformHealth::export_sections_html'
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can' => '\WPEP\ACL\API::manage_settings'
        ]
      ],
      'system/plugin' => [
        'method_map' => [
          'list-from-store'    => '\WPEP\API\System\Plugin::list_from_store',
          'install-from-store' => [
            'callback'  => '\WPEP\API\System\Plugin::install_from_store',
            'user_can'  => 'install_plugins'
          ],
          'activate'           => [
            'callback'  => '\WPEP\API\System\Plugin::activate',
            'user_can'  => 'activate_plugins'
          ]
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can' => '\WPEP\ACL\API::manage_settings'
        ]
      ],
      'system/post' => [
        'method_map' => [
          'content-activity-report' => [
            'callback'  => '\WPEP\API\System\Post::content_activity_report',
            'user_can'  => WPEP_CAP_MANAGE_MEMBERS
          ],
          'search-map'                  => '\WPEP\API\System\Post::search_map',
          'search-learning-journey-map' => '\WPEP\API\System\Post::search_learning_journey_map',
          'save'                        => '\WPEP\API\System\Post::save'
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'system/setting' => [
        'method_map' => [
          'search'          => '\WPEP\API\System\Setting::search',
          'search-by-index' => '\WPEP\API\System\Setting::search_by_index',
          'save'            => '\WPEP\API\System\Setting::save',
          'disable-legacy-comments' => '\WPEP\API\System\Setting::disable_legacy_comments'
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can' => '\WPEP\ACL\API::manage_settings' 
        ]
      ],
      'system/statistics' => [
        'method_map' => [
          'configuration-report' => [
            'callback'  => '\WPEP\API\System\Statistics::configuration_report',
            'user_can'  => '\WPEP\ACL\API::manage_members'
          ],
          'user-id-configuration-report' => [
            'callback'  => '\WPEP\API\System\Statistics::user_id_configuration_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_user_id'
          ],
          'post-id-configuration-report' => [
            'callback'  => '\WPEP\API\System\Statistics::post_id_configuration_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_post_id'
          ],
          'user-id-post-id-configuration-report' => [
            'callback'  => '\WPEP\API\System\Statistics::user_id_post_id_configuration_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_user_id_post_id'
          ],
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'system/statistics-current-user' => [
        'method_map' => [
          'configuration-report' => [
            'callback'  => '\WPEP\API\System\StatisticsCurrentUser::configuration_report',
            'user_can'  => '__return_true'
          ]
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE
        ]
      ],
      'system/user' => [
        'method_map' => [
          'search'                         => '\WPEP\API\System\User::search',
          'entries'                        => [
            'callback'  => '\WPEP\API\System\User::entries',
            'user_can'  => '\WPEP\ACL\API::analytics_view_content_type'
          ],
          'post-type-activity-report'      => [
            'callback'  => '\WPEP\API\System\User::post_type_activity_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_content_type'
          ],
          'post-type-activity-list'        => [
            'callback'  => '\WPEP\API\System\User::post_type_activity_list',
            'user_can'  => '\WPEP\ACL\API::analytics_view_content_type'
          ],

          'post-id-activity-report'        => [
            'callback'  => '\WPEP\API\System\User::post_id_activity_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_post_id'
          ],
          'post-id-activity-list'          => [
            'callback'  => '\WPEP\API\System\User::post_id_activity_list',
            'user_can'  => '\WPEP\ACL\API::analytics_view_post_id'
          ],

          'user-id-post-id-activity-report' => [
            'callback'  => '\WPEP\API\System\User::user_id_post_id_activity_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_user_id_post_id'
          ],
          'user-id-post-id-action-tracking-list' => [
            'callback'  => '\WPEP\API\System\User::user_id_post_id_action_tracking_list',
            'user_can'  => '\WPEP\ACL\API::analytics_view_user_id_post_id'
          ],

          'activity-report'                => [
            'callback'  => '\WPEP\API\System\User::activity_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_content_type'
          ],
          'authentication-activity-report' => [
            'callback'  => '\WPEP\API\System\User::authentication_activity_report',
            'user_can'  => '\WPEP\ACL\API::analytics_view_content_type'
          ],

          'template-statistics-current-user' => [
            'callback'  => '\WPEP\API\User::template_statistics_current_user',
            'user_can'  => '__return_true'
          ]
        ],
        'args' => [
          'type'     => WPEP_API_METHOD_TYPE_PRIVATE,
          'user_can' => '\WPEP\ACL\API::manage_members'
        ]
      ]
    ];

    if( wpep_get_setting_flag( 'user-account-delete-enabled' ) )
      $response[ 'user' ][ 'method_map' ][ 'delete' ] = '\WPEP\API\User::delete';

    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) == true || wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) == true )
      $response[ 'user' ][ 'method_map' ][ 'privacy-confirm-agreement' ] = '\WPEP\API\User::privacy_confirm_agreement';

    if( wpep_get_setting_flag( 'user-profile-tab-my-content-enabled' ) )
      $response[ 'user' ][ 'method_map' ][ 'template-content-purchased' ] = '\WPEP\API\User::template_content_purchased';

    if( wpep_get_setting_flag( 'user-profile-tab-author-content-enabled' ) )
      $response[ 'user' ][ 'method_map' ][ 'template-content-created' ] = '\WPEP\API\User::template_content_created';

    if( wpep_discussion_direct_url_enabled() ) {
      $response[ 'discussion' ][ 'method_map' ][ 'template-direct-url-my-discussions' ] = '\WPEP\API\Discussion::template_direct_url_my_discussions';
      $response[ 'discussion' ][ 'method_map' ][ 'template-direct-url-discussion' ]     = '\WPEP\API\Discussion::template_direct_url_discussion';
    }

    return $response;
  }

  public function _wp_ajax_handle_admin_request() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [ 'status' => 'error' ] ) );

    $key_identifier = substr( current_filter(), strlen( 'wp_ajax_' . $this->_prefix . '/' ) );

    $response = $this->get_request_response_by_key( $key_identifier, $_POST );

    echo ( is_array( $response ) ? json_encode( $response ) : $response );
    exit;
  }

  public function _wp_ajax_handle_request() {
    $key_identifier = substr( current_filter(), strlen( is_user_logged_in() ? 'wp_ajax_' . $this->_prefix . '/' : 'wp_ajax_nopriv_' . $this->_prefix . '/' ) );

    $response = $this->get_request_response_by_key( $key_identifier, $_POST );

    echo ( is_array( $response ) ? json_encode( $response ) : $response );
    exit;
  }

}
