<?php

namespace WPEP\API;

use WPEP\Entity\Discussion as Entity_Discussion;

class Discussion {

  public static function notifications() {
    if( !is_user_logged_in() )
      return [
        'status'        => 'error',
        'error'         => __( "User is not logged in", "wpep" ),
        'notifications' => []
      ];

    $user_notifications = wpep_controller()->discussion->data->get_user_notifications( get_current_user_id() );

    return [
      'status'        => 'ok',
      'notifications' => $user_notifications
    ];
  }

  public static function mention_user_search( $request_data ) {
    if( !isset( $request_data[ 'search_term' ] ) || !isset( $request_data[ 'discussion_id' ] ) )
      return false;

    $discussion = Entity_Discussion::instance()->get_discussion( [ 'id' => intval( $request_data[ 'discussion_id' ] ) ] );

    if( $discussion === null )
      return false;

    $integration = wpep_controller()->discussion->get( $discussion->namespace );

    if( $integration === false )
      return false;

    if( !$integration->current_user_can_view( $discussion->id ) )
      return false;

    $discussion_user_ids_data = Entity_Discussion::instance()->get_discussion_user_ids( $discussion->id );

    if( empty( $discussion_user_ids_data[ 'ids' ] ) )
      return [
        'status'  => 'ok',
        'results' => []
      ];

    $users = wpep_search_users_by_args( [
      'search'    => $request_data[ 'search_term' ],
      'match_ids' => $discussion_user_ids_data[ 'ids' ],
      'limit'     => 10
    ] );

    if( empty( $users ) )
      return [
        'status'  => 'ok',
        'results' => []
      ];

    $results = [];

    foreach( $users as $user ) {
      $results[] = [
        'id'    => $user->ID,
        'name'  => wpep_content_user_profile_format_name( $user ),
        'image' => wpep_user_profile_picture_url( $user->ID, 32 )
      ];
    }

    return [
      'status'  => 'ok',
      'results' => $results
    ];
  }

  public static function template_popup( $request_data ) {
    if( !isset( $request_data[ 'discussion_id' ] ) )
      return false;

    $discussion  = Entity_Discussion::instance()->get_discussion( [ 'id' => absint( $request_data[ 'discussion_id' ] ) ] );
    $integration = wpep_controller()->discussion->get( $discussion->namespace );

    if( $integration === false )
      return false;

    if( !$integration->current_user_can_view( $discussion->id ) )
      return false;

    return [
      'status'      => 'ok',
      'content'     => wpep_controller()->template->get_template( 'discussion/template-popup.php', [
        'post_id'              => $discussion->post_id,
        'discussion_id'        => $discussion->id,
        'discussion'           => $discussion,
        'discussion_namespace' => $integration->get_namespace()
      ] ),
    ];
  }

  public static function template_user_list_navigation( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_id' ] ) )
      return false;

    $namespace     = $request_data[ 'namespace' ];
    $discussion_id = intval( $request_data[ 'discussion_id' ] );
    $page          = isset( $request_data[ 'page' ] ) ? absint( $request_data[ 'page' ] ) : 1;

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return false;

    if( !$integration->current_user_can_view( $discussion_id ) )
      return false;

    $discussion_users = $integration->get_discussion_users( $discussion_id, [
      'limit'   => WPEP_DISCUSSION_USER_LIST_PER_PAGE,
      'offset'  => ( $page == 1 ? 0 : ( $page - 1 ) * WPEP_DISCUSSION_USER_LIST_PER_PAGE )
    ] );

    return [
      'status'      => 'ok',
      'timestamp'   => time(),
      'content'     => wpep_template_get( 'discussion/user-list-navigation.php', [
        'discussion_id'     => $discussion_id,
        'discussion_users'  => isset( $discussion_users['entries'] ) ? $discussion_users['entries'] : [],
        'pagination_page'   => $page,
        'total_users'       => $discussion_users['total'],
        'entries_per_page'  => WPEP_DISCUSSION_USER_LIST_PER_PAGE
      ] )
    ];
  }

  public static function template_direct_url_my_discussions( $request_data ) {
    $args = [
      'limit'  => WPEP_DISCUSSION_DIRECT_URL_MY_DISCUSSIONS_PER_PAGE,
      'offset' => ( isset( $request_data[ 'offset' ] ) ? intval( $request_data[ 'offset' ] ) : 0 )
    ];

    if( isset( $request_data[ 'last_activity' ] ) )
      $args[ 'last_activity' ] = [ '>=' => date( "Y-m-d H:i:s", intval( $request_data[ 'last_activity' ] ) ) ];

    $my_discussions = wpep_controller()->discussion->user_get_discussions_information( get_current_user_id(), $args );

    return [
      'status'      => 'ok',
      'timestamp'   => time(),
      'content'     => wpep_template_get( 'discussion/direct-url/my-discussions.php', [
        'current_discussion_id' => false,
        'discussions_list'      => $my_discussions[ 'results' ],
        'discussions_total'     => $my_discussions[ 'total' ],
        'discussions_has_more'  => ( $args[ 'offset' ] + count( $my_discussions[ 'results' ] ) < $my_discussions[ 'total' ] )
      ] )
    ];
  }

  public static function template_direct_url_discussion( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_id' ] ) )
      return false;

    $namespace        = $request_data[ 'namespace' ];
    $discussion_id    = absint( $request_data[ 'discussion_id' ] );

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return false;

    $discussion = Entity_Discussion::instance()->get_discussion( [ 'id' => $discussion_id ] );

    if( !$integration->current_user_can_view( $discussion ) )
      return false;

    ob_start();

    wpep_template_load( 'discussion/direct-url/current-discussion.php', [
      'current_discussion' => $discussion,
    ] );

    $content = ob_get_contents();

    ob_end_clean();

    return [
      'status'    => 'ok',
      'timestamp' => time(),
      'content'   => $content
    ];
  }

}
