<?php

namespace WPEP\API\Discussion;

use WPEP\Entity\Discussion as WPEP_Discussion;
use WPEP\Container\FilterHacks as WPEP_FilterHacks;

class Comment {

  use WPEP_FilterHacks;

  public static function post( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_id' ] ) || !isset( $request_data[ 'comment_parent' ] ) || !isset( $request_data[ 'content' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace         = $request_data[ 'namespace' ];
    $discussion_id     = intval( $request_data[ 'discussion_id' ] );
    $comment_parent_id = intval( $request_data[ 'comment_parent' ] );
    $content           = $request_data[ 'content' ];

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ];

    if( !$integration->current_user_can_post( $discussion_id ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Current user does not have access to post.", "wpep" )
      ];

    $content = wpep_utilities_sanitize_input( $content );
    $comment_type = isset( $request_data['comment_type'] ) ? $request_data['comment_type'] : '';
    $comment_type = $integration->has_comment_type( $comment_type, $discussion_id ) ? $comment_type : '';
    $current_date_time = date( "Y-m-d H:i:s", time() );
    $can_reply_comment = true;

    if( $comment_parent_id != 0 ) {
      $discussion_comment_parent = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $comment_parent_id ] );

      if( $discussion_comment_parent->discussion_id != $discussion_id )
        return [
          'status'  =>  'error',
          'error'   =>  __( "Parent Comment Discussion ID doesn't match the current discussion.", "wpep" )
        ];

      if( $discussion_comment_parent->parent_id != 0 )
        $can_reply_comment = false;
    }

    $discussion_comment_information = [
      'parent_id'     => $comment_parent_id,
      'discussion_id' => $discussion_id,
      'user_id'       => get_current_user_id(),
      'content'       => $content,
      'created_at'    => $current_date_time,
      'last_activity' => $current_date_time,
      'type'          => $comment_type
    ];
    $discussion_comment_id = WPEP_Discussion::instance()->add_discussion_comment( $discussion_comment_information );

    if( $discussion_comment_id === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Comment could not be saved.", "wpep" )
      ];

    if( wpep_get_setting_flag( 'comments-mentions-status' ) )
      self::_handle_comment_mentions( $discussion_comment_id, $discussion_comment_information, $integration );

    do_action( "wpep_discussion_event_added_comment", $discussion_comment_id, $discussion_comment_information, $integration );

    $discussion_comment = WPEP_Discussion::instance()->get_discussion_comment( [
      'id' => $discussion_comment_id
    ] );

    if( isset( $discussion_comment_parent ) ) {
      $sql =  'UPDATE `' . wpep_wpdb()->prefix . WPEP_DB_TABLE_DISCUSSION_COMMENT . '` SET last_activity = "' . $current_date_time . '"';
      $sql .= ' WHERE id = ' . $discussion_comment_parent->id;

      if( $discussion_comment_parent->parent_id != 0 )
        $sql .= ' OR id = ' . $discussion_comment_parent->parent_id;

      wpep_wpdb()->query( $sql );
    }

    self::_static_filter_hack_deque( 'embed_oembed_html' );

    $response = [
      'status'          => 'ok',
      'comment_data'    => wpep_controller()->discussion->data->format_comment_data( $discussion_comment, $integration, $can_reply_comment )
    ];

    self::_static_filter_hack_reque( 'embed_oembed_html' );

    return $response;
  }

  /**
   * @param int $discussion_comment_id
   * @param array $discussion_comment_information
   * @param \WPEP_Discussion_Integration $integration
   */
  private static function _handle_comment_mentions( $discussion_comment_id, $discussion_comment_information, $integration ) {
    $mentioned_user_ids = wpep_document_dom_extract_element_attribute(
      $discussion_comment_information[ 'content' ],
      'span',
      'data-wpep-mention-user-id'
    );

    if( empty( $mentioned_user_ids ) )
      return;

    $discussion = WPEP_Discussion::instance()->get_discussion( [
      'id' => $discussion_comment_information[ 'discussion_id' ]
    ] );

    if( $discussion === null )
      return;

    $mentioned_user_ids = array_map( 'intval', $mentioned_user_ids );
    $notified_user_ids = [];

    $current_user_id = get_current_user_id();

    $base_email_title   = wpep_get_setting( 'comments-mentions-notification-title' );
    $base_email_content = wpep_get_setting( 'comments-mentions-notification-content' );

    foreach( [
               'mention_source_name'          => wpep_content_user_profile_format_name( $current_user_id ),
               'mention_source_picture'       => '<img src="' . wpep_user_profile_picture_url( $current_user_id, 40 ) . '" style="margin: 0;padding: 0;width: 40px;height: 40px;overflow: hidden;background: #bcbcbc;-webkit-border-radius: 20px;-moz-border-radius: 20px;border-radius: 20px;"/>',
               'discussion_comment_content'   => $discussion_comment_information[ 'content' ],
               'discussion_name'              => $integration->get_discussion_name( $discussion ),
               'discussion_comment_date_time' => wpep_format_date_with_time( time() )
             ] as $replace_key => $replace_value ) {
      $base_email_title   = str_replace( '[' . $replace_key . ']', $replace_value, $base_email_title );
      $base_email_content = str_replace( '[' . $replace_key . ']', $replace_value, $base_email_content );
    }

    foreach( $mentioned_user_ids as $mentioned_user_id ) {
      $user = get_user_by( 'id', $mentioned_user_id );

      if( $user === null )
        continue;

      if( in_array( $mentioned_user_id, $notified_user_ids ) )
        continue;

      wp_set_current_user( $mentioned_user_id );

      if( !$integration->current_user_can_view( $discussion_comment_information[ 'discussion_id' ] ) )
        continue;

      $discussion_link = $integration->get_discussion_comment_highlight_link( $discussion, $discussion_comment_id, $mentioned_user_id );

      $current_email_title   = str_replace( '[discussion_link]', $discussion_link, $base_email_title );
      $current_email_content = str_replace( '[discussion_link]', $discussion_link, $base_email_content );

      if( strpos( $current_email_content, '[mention_activity_preview]') !== false ) {
        $discussions_map = [ 0 => $discussion ] ;

        $discussions_map[ 0 ]->comments = [ (object) $discussion_comment_information ];

        $current_email_content = str_replace( '[mention_activity_preview]', wpep_template_get( 'email-template/discussion-notifications.php', [
          'discussions_map' => $discussions_map,
          'user_id'         => $mentioned_user_id
        ] ), $current_email_content );
      }

      WPEP_Discussion::instance()->add_discussion_comment_user_data(
        WPEP_DISCUSSION_COMMENT_USER_DATA_MENTION,
        $mentioned_user_id,
        $discussion_comment_id,
        $mentioned_user_id
      );

      wpep_controller()->userEngagement->send_email( $user, $current_email_title, $current_email_content, true );
    }

    wp_set_current_user( $current_user_id, wpep_content_user_profile_format_name( $current_user_id ) );
  }

  public static function pin( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_id' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace             = $request_data[ 'namespace' ];
    $discussion_comment_id = intval( $request_data[ 'discussion_comment_id' ] );
    $pin_status            = isset( $request_data[ 'pin_status' ] ) && intval( $request_data[ 'pin_status' ] ) ? 1 : 0;

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ];

    if( !$integration->current_user_can_pin_comment( $discussion_comment_id ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Your user is not allowed to pin to the top this comment.", "wpep" )
      ];

    WPEP_Discussion::instance()->pin_discussion_comment_by_id( $discussion_comment_id, $pin_status );

    return [
      'status'                => 'ok',
      'discussion_comment_id' => $discussion_comment_id,
      'pin_status'            => $pin_status
    ];
  }

  public static function delete( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_comment_id' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace = $request_data[ 'namespace' ];
    $discussion_comment_id = intval( $request_data[ 'discussion_comment_id' ] );

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ];

    if( !$integration->current_user_can_delete_comment( $discussion_comment_id ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Your user is not allowed to delete this comment.", "wpep" )
      ];

    WPEP_Discussion::instance()->delete_discussion_comment_by_id( $discussion_comment_id );

    return [
      'status'      => 'ok',
      'timestamp'   => time()
    ];
  }

  public static function react( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_comment_id' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace             = $request_data[ 'namespace' ];
    $discussion_comment_id = intval( $request_data[ 'discussion_comment_id' ] );
    $reaction_code         = $request_data[ 'reaction_code' ];

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ];

    $discussion_comment = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $discussion_comment_id ] );

    if( !$integration->current_user_can_view( $discussion_comment->discussion_id ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Your user is not allowed to react to this comment.", "wpep" )
      ];

    if( $reaction_code === '' )
      WPEP_Discussion::instance()->remove_discussion_comment_user_data( WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION, null, $discussion_comment_id, get_current_user_id() );
    else
      WPEP_Discussion::instance()->set_discussion_comment_user_data( WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION, $reaction_code, $discussion_comment_id, get_current_user_id() );

    wpep_controller()->discussion->data->update_comment_reaction_data( $discussion_comment_id );

    $discussion_comment = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $discussion_comment_id ] );
    $discussion_comment->user_reaction = $reaction_code;

    return [
      'status'          => 'ok',
      'timestamp'       => time(),
      'comment_data'    => wpep_controller()->discussion->data->format_comment_data( $discussion_comment, $integration, false )
    ];
  }

  public static function reactions( $request_data ) {
    if( !isset( $request_data[ 'namespace' ] ) || !isset( $request_data[ 'discussion_comment_id' ] ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Not all parameters have been provided", "wpep" )
      ];

    $namespace             = $request_data[ 'namespace' ];
    $discussion_comment_id = intval( $request_data[ 'discussion_comment_id' ] );

    $integration = wpep_controller()->discussion->get( $namespace );

    if( $integration === false )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Invalid Discussion namespace.", "wpep" )
      ];

    $discussion_comment = WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => $discussion_comment_id ] );

    if( !$integration->current_user_can_view( $discussion_comment->discussion_id ) )
      return [
        'status'  =>  'error',
        'error'   =>  __( "Your user is not allowed to view reactions for this comment.", "wpep" )
      ];

    return [
      'status'          => 'ok',
      'timestamp'       => time(),
      'reactions_data'  => wpep_controller()->discussion->data->get_comment_reactions( $discussion_comment_id, [
        'include_overview'         => ( isset( $request_data[ 'include_overview' ] ) ? intval( $request_data[ 'include_overview' ] ) : 0 ),
        'reaction_code'            => ( isset( $request_data[ 'list_code_filter' ] ) && $request_data[ 'list_code_filter' ] != false && in_array( $request_data[ 'list_code_filter' ], wpep_get_setting( 'comments-reactions-available-emoji-list' ) ) ? $request_data[ 'list_code_filter' ] : '' ),
        'pagination_current_page'  => ( isset( $request_data[ 'pagination_current_page' ] ) ? intval( $request_data[ 'pagination_current_page' ] ) : 1 ),
      ] )
    ];
  }

}
