<?php

namespace WPEP\API\System;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Plugin {

  public static function list_from_store( $request_data ) {
    $entries = get_transient( WPEP_STORE_PRODUCTS_RESPONSE_TRANSIENT );

    if ( empty( $entries ) ) {
      $response         = wp_remote_post( WPEP_STORE_PRODUCTS_LINK, [ 'timeout' => 15, 'sslverify' => true ] );
      $response_entries = json_decode(wp_remote_retrieve_body($response));

      if( !isset( $response_entries->products ) )
        return [];

      $entries = [];

      foreach( $response_entries->products as $response_entry ) {
        if( !is_array( $response_entry->info->category  ) )
          continue;

        $allowed = false;
        $category_title = "Extensions";

        foreach( $response_entry->info->category as $category_information ) {
          if( $category_information->slug == 'addons' ) {
            $allowed = true;
            continue;
          }

          $category_title = $category_information->name;
        }

        if( !$allowed )
          continue;

        if( !isset( $response_entry->info->category_name ) )
          $response_entry->info->category_name = "Extensions";

        $current_entry = [
          'id'              => $response_entry->info->id,
          'slug'            => $response_entry->info->slug,
          'title'           => $response_entry->info->title,
          'link'            => $response_entry->info->link,
          'thumbnail'       => ( isset( $response_entry->info->thumbnail ) && $response_entry->info->thumbnail != '' ? $response_entry->info->thumbnail : WPEP_PLACEHOLDER_IMAGE ),
          'excerpt'         => $response_entry->info->excerpt,
          'category_title'  => $category_title,

          'plugin_status'       => '',
          'plugin_status_label' => '',
          'plugin_name'         => trim(esc_html( str_ireplace( [ 'WPEP', WPEP_PLUGIN_NAME ], '', $response_entry->info->title ) ) ),
          'plugin_alias'        => str_replace( [ "gravity-forms" ], [ "gravityforms" ], str_ireplace( [ 'wpep-', strtolower( WPEP_PLUGIN_NAME ) . '-' ], '', $response_entry->info->slug ) )
        ];

        $current_entry[ 'plugin_directory' ] = 'wpep-' . $current_entry[ 'plugin_alias' ];
        $current_entry[ 'plugin_file_path' ] = $current_entry[ 'plugin_directory' ] . '/wpep-' . $current_entry[ 'plugin_alias' ] . '.php';

        $entries[] = $current_entry;
      }

      set_transient( WPEP_STORE_PRODUCTS_RESPONSE_TRANSIENT, $entries, DAY_IN_SECONDS );
    }

    if ( ! function_exists( 'is_plugin_active' ) )
      require_once( ABSPATH . '/wp-admin/includes/plugin.php' );

    foreach( $entries as $entry_key => $current_entry ) {
      if( !file_exists( WP_PLUGIN_DIR . '/' . $current_entry[ 'plugin_file_path' ] ) )
        continue;

      $entries[ $entry_key ][ 'plugin_status' ] = ( is_plugin_active( $current_entry[ 'plugin_file_path' ] ) ? 'active' : 'inactive' );
      $entries[ $entry_key ][ 'plugin_status_label' ] = ( $entries[ $entry_key ][ 'plugin_status' ] === 'active' ? __( "Active", "wpep" ) : __( "Inactive", "wpep" ) );
    }

    return [
      'status'  => 'ok',
      'entries' => $entries,
      'lang'    => [
        'install'  => __( "Install", "wpep" ),
        'activate' => __( "Activate", "wpep" ),
        'details'  => __( "Details", "wpep" )
      ]
    ];
  }

  public static function install_from_store( $request_data ) {
    if( !isset( $request_data['store_product_id'] ) || !isset( $request_data['store_product_title'] ) )
      return false;

    $store_product_id    = intval( $request_data['store_product_id'] );
    $store_product_title = $request_data['store_product_title'];

    if( !in_array( $store_product_id, wpep_controller()->adminController->license->get_license_bundled_products() ) )
      return [
        'status'  => 'error',
        'error'   => __( "The Bundle License cannot be validated, please download the plugin from the My Account section.", "wpep" )
      ];

    set_current_screen();

    $license_and_updates = new \WPEP_Integration_License_And_Updates(
      $store_product_title,
      $store_product_id,
      false,
      '0.0.0',
      false
    );

    $license_data = $license_and_updates->api->activate_license();

    if( !isset( $license_data->license ) || $license_data->license != 'valid' )
      return [
        'status'  => 'error',
        'error'   => __( "The Bundle License cannot be validated, please download the plugin from the My Account section.", "wpep" )
      ];

    $request_params = $license_and_updates->api->get_remote_request_params();
    $request_params['body'] = $license_and_updates->api->get_request_params( 'get_version' );

    $request = wp_remote_post( WPEP_STORE_LINK, $request_params );

    if ( is_wp_error( $request ) )
      return [
        'status'  => 'error',
        'error'   => __( "Plugin Information cannot be fetched from server, please download the plugin from the My Account section.", "wpep" )
      ];

    $request = json_decode( wp_remote_retrieve_body( $request ) );

    if( !isset( $request->package ) )
      return [
        'status'  => 'error',
        'error'   => __( "Plugin Package cannot be fetched from server, please download the plugin from the My Account section.", "wpep" )
      ];

    if( !class_exists( '\Plugin_Upgrader' ) )
      require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

    if( !class_exists( '\WPEP_Silent_Upgrader_Skin') )
      require_once WPEP_BASE_PATH . '/lib/_internal/Silent_Upgrader.php';

    try {
      ob_start();
      $installer = new \Plugin_Upgrader( new \WPEP_Silent_Upgrader_Skin() );
      $installation_response = $installer->install( $request->package );
      ob_end_clean();
      if( is_wp_error( $installation_response ) )
        return [
          'status'  => 'error',
          'error'   => $installation_response->get_error_message()
        ];

      wp_cache_flush();
    } catch ( \Exception $exception ) {
      return [
        'status'  => 'error',
        'error'   => $exception->getMessage()
      ];
    }

    if ( !$installer->plugin_info() )
      return [
        'status'  => 'error',
        'error'   => __( "The installation has failed, please install the plugin manually as we cannot do it for you at the moment.", "wpep" )
      ];

    return true;
  }

  public static function activate( $request_data ) {
    if( !isset( $request_data['plugin_path'] ) )
      return false;

    $activate = activate_plugin( $request_data['plugin_path'] );

    if ( is_wp_error( $activate ) )
      return [
        'status'  => 'error',
        'error'   => $activate->get_error_message()
      ];

    return true;
  }
}