<?php

namespace WPEP\API\System;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\API\Utils\Report as API_Utils_Report;
use WPEP\API\Utils\User as API_Utils_User;
use WPEP\API\Utils\Post as API_Utils_Post;
use WPEP\Entity\Discussion as Entity_Discussion;

class User {

  public static function search( $request_data ) {
    if( !isset( $request_data[ 'search' ] ) )
      return [
        'status'  => 'error',
        'error'   => __( "Missing search term.", "wpep" )
      ];

    $user_object_list = wpep_search_users_by_args( [
      'search'  => $request_data[ 'search' ],
      'limit'   => 15
    ] );

    if( empty( $user_object_list ) )
      return [
        'status'  => 'ok',
        'results' => []
      ];

    $search_results = [];

    foreach( $user_object_list as $user_object )
      $search_results[] = [
        'text'  => wpep_content_user_profile_format_name( $user_object ) . ' #' . $user_object->ID,
        'id'    => $user_object->ID
      ];

    return [
      'status'  => 'ok',
      'results' => $search_results
    ];
  }

  public static function entries( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $pagination_limit = 20;
    $current_page     = ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 );

    $get_entries_where = [
      'type'  => [
        'IN'  => [
          WPEP_ACTION_TRACKING_TYPE_REGISTRATION,
          WPEP_ACTION_TRACKING_TYPE_LOGIN,
          WPEP_ACTION_TRACKING_TYPE_LOGOUT,
          WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD,
          WPEP_ACTION_TRACKING_TYPE_CHANGE_PASSWORD,
          WPEP_ACTION_TRACKING_TYPE_EXPRESS
        ]
      ]
    ];

    if( isset( $request_data[ 'report_time_from' ] ) && isset( $request_data[ 'report_time_to' ] ) )
      $get_entries_where[ 'created_at' ] = [
        '>=' => date( "Y-m-d H:i:s", $request_data[ 'report_time_from' ] ),
        '<=' => date( "Y-m-d H:i:s", $request_data[ 'report_time_to' ] )
      ];

    if( isset( $request_data[ 'user_id' ] ) )
      $get_entries_where[ 'user_id' ] = intval( $request_data[ 'user_id' ] );
    elseif( isset( $request_data[ 'user_ids' ] ) )
      $get_entries_where[ 'user_id' ] = [ 'IN' => ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) ) ];


    $get_entries_response = WPEP_Entity_ActionTracking::instance()->get_tracked_user_ids( $get_entries_where, [
      'count_total' => true,
      'limit'       => $pagination_limit,
      'offset'      => ( $current_page - 1 ) * $pagination_limit
    ] );

    $entries = [];
    $entries_user_objects = [];

    if( !empty( $get_entries_response[ 'results' ] ) ) {

      $entries_user_objects = wpep_utilities_map_object( get_users( [ 'include' => $get_entries_response[ 'results' ] ] ), 'ID' );

      foreach( $entries_user_objects as $userObject ) {
        $last_login  = WPEP_Entity_ActionTracking::instance()->get_last_tracked_event( [
          'type'    => WPEP_ACTION_TRACKING_TYPE_LOGIN,
          'user_id' => $userObject->ID
        ] );
        $last_logout = WPEP_Entity_ActionTracking::instance()->get_last_tracked_event( [
          'type'    => WPEP_ACTION_TRACKING_TYPE_LOGOUT,
          'user_id' => $userObject->ID
        ] );

        $entries[] = [
          'user_id'                 => $userObject->ID,
          'last_login_formatted'    => ( $last_login !== null ? wpep_format_date_with_time( $last_login->created_at ) : '-' ),
          'last_logout_formatted'   => ( $last_logout !== null ? wpep_format_date_with_time( $last_logout->created_at ) : '-' ),
          'registered_at'           => $userObject->user_registered,
          'registered_at_formatted' => wpep_format_date_with_time( $userObject->user_registered ),
        ];
      }
    }

    $response = [
      'status'          =>  'ok',
      'entries_header'  => [
        'last_login_formatted'    => __( "Logged In", "wpep" ),
        'last_logout_formatted'   => __( "Logged Out", "wpep" ),
        'registered_at_formatted' => __( "Account Created", "wpep" ),
      ],
      'entries'         => $entries,
      'pagination'      => [
        'current'       => $current_page,
        'total'         => ceil( $get_entries_response[ 'total' ] / $pagination_limit ),
        'total_entries' => $get_entries_response[ 'total' ]
      ]
    ];

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];
    $contentTypes = wpep_content_type()->contentTypes;

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    $response = [];
    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $base_args = [
        'time_from' => $time_slot_from,
        'time_to'   => $time_slot_to
      ];

      if( isset( $request_data[ 'user_ids' ] ) )
        $base_args[ 'user_ids' ] = $request_data[ 'user_ids' ];

      $user_registration_count  = wpep_data_system()->get_registered_users_count( $base_args );
      $discussion_comment_count = absint( Entity_Discussion::instance()->get_comment_count_by_args( $base_args + [
          'parent'    => 0
        ] ) );
      $discussion_reply_count = absint( Entity_Discussion::instance()->get_comment_reply_count_by_args( $base_args ) );

      $current_time_slot_data = [];

      if( $user_registration_count !== 0 )
        $current_time_slot_data[ __( 'User Registration', "wpep" ) ] = $user_registration_count;

      $group_by_type_tracked_actions = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $base_args + [
        'group_by'  => [ 'type' ],
        'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION, WPEP_ACTION_TRACKING_TYPE_CHANGE_PASSWORD, WPEP_ACTION_TRACKING_TYPE_LOGOUT ]
      ] );

      foreach( $group_by_type_tracked_actions as $group_by_type_tracked_action_data ) {
        if( intval( $group_by_type_tracked_action_data->entries ) === 0 )
          continue;

        $current_time_slot_data[ 'Authentication - ' . wpep_controller()->actionTracking->get_event_name( $group_by_type_tracked_action_data->type ) ] = $group_by_type_tracked_action_data->entries;
      }

      if( $discussion_comment_count !== 0 )
        $current_time_slot_data[ __( 'Discussion - Comment', "wpep" ) ] = $discussion_comment_count;

      if( $discussion_reply_count !== 0 )
        $current_time_slot_data[ __( 'Discussion - Comment Reply', "wpep" ) ] = $discussion_reply_count;

      if( wpep_get_setting_flag( 'comments-reactions-status' ) ) {
        $discussion_reaction_count = absint( Entity_Discussion::instance()->get_comment_data_count_by_args( $base_args + [
            'key'       => WPEP_DISCUSSION_COMMENT_USER_DATA_REACTION
          ] ) );

        if( $discussion_reaction_count !== 0 )
          $current_time_slot_data[ __( 'Discussion - Comment Reaction', "wpep" ) ] = $discussion_reaction_count;
      }

      if( wpep_get_setting_flag( 'comments-mentions-status' ) ) {
        $discussion_mention_count = absint( Entity_Discussion::instance()->get_comment_data_count_by_args( $base_args + [
            'key'       => WPEP_DISCUSSION_COMMENT_USER_DATA_MENTION
          ] ) );

        if( $discussion_mention_count !== 0 )
          $current_time_slot_data[ __( 'Discussion - Comment Mention', "wpep" ) ] = $discussion_mention_count;
      }

      $group_by_type_post_type_tracked_actions = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $base_args + [
        'group_by'  => [ 'type', 'post_type' ],
        'type'      => [
          WPEP_ACTION_TRACKING_TYPE_OPEN_LINK,
          WPEP_ACTION_TRACKING_TYPE_ACCESS_CONTENT,
          WPEP_ACTION_TRACKING_TYPE_DOWNLOAD,
          WPEP_ACTION_TRACKING_TYPE_DOWNLOAD_DELIVERABLE,
          WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_STARTED,
          WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_COMPLETED,
          WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_FAILED,
          WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_REVIEW_REQUIRED,
          WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_RESET
        ]
      ] );

      foreach( $group_by_type_post_type_tracked_actions as $group_by_type_post_type_tracked_action_data ) {
        if( !isset( $contentTypes[ $group_by_type_post_type_tracked_action_data->post_type ] ) )
          continue;

        if( intval( $group_by_type_post_type_tracked_action_data->entries ) === 0 )
          continue;

        $current_time_slot_data[
          $contentTypes[ $group_by_type_post_type_tracked_action_data->post_type ]->get_singular_name() . ' - ' . wpep_controller()->actionTracking->get_event_name( $group_by_type_post_type_tracked_action_data->type )
        ] = $group_by_type_post_type_tracked_action_data->entries;
      }

      foreach( $contentTypes as $contentType ) {
        $content_type_data = $contentType->data_user_activity_report( $base_args );

        if( empty( $content_type_data ) )
          continue;

        $current_time_slot_data += $content_type_data;
      }

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

  public static function authentication_activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $base_args = [
        'time_from' => $time_slot_from,
        'time_to'   => $time_slot_to
      ];

      if( isset( $request_data[ 'user_ids' ] ) )
        $base_args[ 'user_ids' ] = $request_data[ 'user_ids' ];

      $user_registration_count = wpep_data_system()->get_registered_users_count( $base_args );

      $current_time_slot_data = [];

      if( $user_registration_count !== 0 )
        $current_time_slot_data[ __( 'User Registration', "wpep" ) ] = $user_registration_count;

      $group_by_type_tracked_actions = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $base_args + [
        'group_by'  => [ 'type' ],
        'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION, WPEP_ACTION_TRACKING_TYPE_CHANGE_PASSWORD, WPEP_ACTION_TRACKING_TYPE_LOGOUT ],
      ] );

      foreach( $group_by_type_tracked_actions as $group_by_type_tracked_action_data )
        $current_time_slot_data[ 'Authentication - ' . wpep_controller()->actionTracking->get_event_name( $group_by_type_tracked_action_data->type ) ] = $group_by_type_tracked_action_data->entries;

      $group_by_type_segment_tracked_actions = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $base_args + [
        'group_by'  => [ 'type', 'segment' ],
        'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ]
      ] );

      foreach( $group_by_type_segment_tracked_actions as $group_by_type_segment_tracked_action_data )
        $current_time_slot_data[ 'Authentication - ' . wpep_controller()->actionTracking->get_event_name( $group_by_type_segment_tracked_action_data->type ). ' - ' . $group_by_type_segment_tracked_action_data->segment ] = $group_by_type_segment_tracked_action_data->entries;

      $group_by_type_post_id_tracked_actions = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $base_args + [
        'group_by'   => [ 'type', 'post_id' ],
        'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ]
      ] );

      foreach( $group_by_type_post_id_tracked_actions as $group_by_type_post_id_tracked_action )
          $current_time_slot_data[ 'Authentication - ' . wpep_controller()->actionTracking->get_event_name( $group_by_type_post_id_tracked_action->type ). ' - ' . $group_by_type_post_id_tracked_action->post_title ] = $group_by_type_post_id_tracked_action->entries;

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

  public static function post_type_activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_type'] ) || !wpep_content_type()->is_enabled( $request_data[ 'post_type'] ) )
      return false;

    $time_slots = $request_data[ 'report_time_slots' ];

    if( isset( $request_data[ 'user_id' ] ) ) {
      $request_data[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $request_data[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $request_data[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $request_data[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    $response = [];

    foreach( $time_slots as $time_slot_from => $time_slot_to ) {
      $current_time_slot_data = [];

      $base_args = [
        'time_from'  => $time_slot_from,
        'time_to'    => $time_slot_to
      ];

      if( isset( $request_data[ 'user_ids' ] ) && !empty( $request_data[ 'user_ids' ] ) )
        $base_args[ 'user_ids' ] = $request_data[ 'user_ids' ];

      if( isset( $request_data[ 'post_ids' ] ) && !empty( $request_data[ 'post_ids' ] ) )
        $base_args[ 'post_ids' ] = $request_data[ 'post_ids' ];

      $data = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $base_args + [
          'group_by'   => [ 'type', 'post_id' ],
          'type'       => [
            WPEP_ACTION_TRACKING_TYPE_OPEN_LINK,
            WPEP_ACTION_TRACKING_TYPE_DOWNLOAD,
            WPEP_ACTION_TRACKING_TYPE_DOWNLOAD_DELIVERABLE,
            WPEP_ACTION_TRACKING_TYPE_ACCESS_CONTENT,
            WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_STARTED,
            WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_COMPLETED,
            WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_FAILED,
            WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_REVIEW_REQUIRED,
            WPEP_ACTION_TRACKING_TYPE_CONTENT_STATUS_RESET
          ],
          'post_type'  => $request_data[ 'post_type']
        ] );

      foreach( $data as $information ) {
        $current_time_slot_data[ wpep_controller()->actionTracking->get_event_name( $information->type ) . ' - ' . $information->post_title ] = $information->entries;
      }

      $current_time_slot_data += wpep_content_type()->contentTypes[ $request_data[ 'post_type'] ]->data_user_post_activity_report( $base_args );

      $response[ $time_slot_from ] = $current_time_slot_data;
    }

    return [ 'status' => 'ok', 'map' => $response ] ;
  }

  public static function post_type_activity_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_type'] ) || !wpep_content_type()->is_enabled( $request_data[ 'post_type'] ) )
      return false;

    $contentTypeInstance = wpep_content_type()->contentTypes[ $request_data[ 'post_type' ] ];

    $content_type_function_args = [
      'current_page'      => ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 ),
      'pagination_limit'  => 20,
      'user_ids'          => [],
      'post_ids'          => []
    ];

    if( isset( $request_data[ 'user_id' ] ) ) {
      $content_type_function_args[ 'user_ids' ] = intval( $request_data[ 'user_id' ] );
    } elseif( isset( $request_data[ 'user_ids' ] ) )
      $content_type_function_args[ 'user_ids' ] = ( is_array( $request_data[ 'user_ids' ] ) ? array_map( 'intval', $request_data[ 'user_ids' ] ) : intval( $request_data[ 'user_ids' ] ) );

    if( isset( $request_data[ 'post_id' ] ) ) {
      $content_type_function_args[ 'post_ids' ] = intval( $request_data[ 'post_id' ] );
    } elseif( isset( $request_data[ 'post_ids' ] ) )
      $content_type_function_args[ 'post_ids' ] = ( is_array( $request_data[ 'post_ids' ] ) ? array_map( 'intval', $request_data[ 'post_ids' ] ) : intval( $request_data[ 'post_ids' ] ) );

    if( isset( $request_data[ 'report_time_from' ] ) )
      $content_type_function_args[ 'time_from' ] = $request_data[ 'report_time_from' ];
    if( isset( $request_data[ 'report_time_to' ] ) )
      $content_type_function_args[ 'time_to' ] = $request_data[ 'report_time_to' ];

    $response = $contentTypeInstance->data_user_post_activity_list( $content_type_function_args );

    if( !is_array( $response ) ) {
      $response = [
        'entries_header'  => [
          'event_name'            => __( "Event", "wpep" ),
          'event_segment'         => __( "Segment", "wpep" ),
          'created_at_formatted'  => __( "Tracked At", "wpep" )
        ],
        'entries'         => [],
        'pagination'      => [
          'current'       => $content_type_function_args[ 'current_page' ],
          'total'         => 0,
          'total_entries' => 0
        ]
      ];

      $get_entries_where = [
        'post_type' => $contentTypeInstance->get_post_type()
      ];

      if( isset( $request_data[ 'report_time_from' ] ) && isset( $request_data[ 'report_time_to' ] ) )
        $get_entries_where[ 'created_at' ] = [
          '>=' => date( "Y-m-d H:i:s", $request_data[ 'report_time_from' ] ),
          '<=' => date( "Y-m-d H:i:s", $request_data[ 'report_time_to' ] )
        ];

      if( !empty( $content_type_function_args[ 'user_ids' ] ) )
        $get_entries_where[ 'user_id' ] = is_array( $content_type_function_args[ 'user_ids' ] ) ? [ 'IN' => $content_type_function_args[ 'user_ids' ] ] : $content_type_function_args[ 'user_ids' ];

      if( !empty( $content_type_function_args[ 'post_ids' ] ) )
        $get_entries_where[ 'post_id' ] = is_array( $content_type_function_args[ 'post_ids' ] ) ? [ 'IN' => $content_type_function_args[ 'post_ids' ] ] : $content_type_function_args[ 'post_ids' ];

      $tracked_events = WPEP_Entity_ActionTracking::instance()->get_tracked_events( $get_entries_where, [
        'limit'       => $content_type_function_args[ 'pagination_limit' ],
        'offset'      => ( $content_type_function_args[ 'current_page' ] - 1 ) * $content_type_function_args[ 'pagination_limit' ],
        'count_total' => true
      ] );

      $response[ 'pagination' ][ 'total' ] = ceil( $tracked_events[ 'total' ] / $content_type_function_args[ 'pagination_limit' ] );
      $response[ 'pagination' ][ 'total_entries' ] = $tracked_events[ 'total' ];

      foreach( $tracked_events[ 'results' ] as $tracked_event ) {
        $response[ 'entries' ][] = [
          'post_id'              => $tracked_event->post_id,
          'user_id'              => $tracked_event->user_id,
          'event_name'           => wpep_controller()->actionTracking->get_event_name( $tracked_event->type ),
          'event_type'           => $tracked_event->type,
          'event_segment'        => $tracked_event->segment,
          'created_at'           => $tracked_event->created_at,
          'created_at_formatted' => wpep_format_date_with_time( $tracked_event->created_at )
        ];
      }
    }

    $response[ 'status' ] = 'ok';

    $response = API_Utils_Post::entries( $response, $request_data );

    if( isset( $content_type_function_args[ 'user_ids' ] ) && is_numeric( $content_type_function_args[ 'user_ids' ] ) )
      return $response;

    $user_ids = wpep_utility_map_to_array_assoc(
      $response[ 'entries' ],
      'user_id',
      'user_id'
    );

    if( empty( $user_ids ) )
      return $response;

    $entries_user_objects = ( isset( $response[ 'entries' ] ) ? wpep_utilities_map_object(
      get_users( [
        'include' => $user_ids
      ] ),
      'ID'
    ) : [] );

    return API_Utils_User::entries( $response, $request_data, $entries_user_objects );
  }

  public static function post_id_activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_id'] ) )
      return false;

    $post = get_post( intval( $request_data[ 'post_id'] ) );

    if( !wpep_content_type()->is_enabled( $post->post_type ) || !in_array( $post->post_status, wpep_post_monitored_post_status_list() ) )
      return false;

    $request_data[ 'post_type' ] = $post->post_type;
    $request_data[ 'post_ids' ] = $request_data[ 'post_id'];

    unset( $request_data[ 'post_id'] );

    return self::post_type_activity_report( $request_data );
  }

  public static function post_id_activity_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_id'] ) )
      return false;

    $post = get_post( intval( $request_data[ 'post_id'] ) );

    if( !wpep_content_type()->is_enabled( $post->post_type ) || !in_array( $post->post_status, wpep_post_monitored_post_status_list() ) )
      return false;

    $request_data[ 'post_type' ] = $post->post_type;
    $request_data[ 'post_ids' ] = $request_data[ 'post_id'];

    unset( $request_data[ 'post_id'] );

    return self::post_type_activity_list( $request_data );
  }

  public static function user_id_post_id_activity_report( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_id'] ) )
      return false;

    $post = get_post( intval( $request_data[ 'post_id'] ) );

    if( !wpep_content_type()->is_enabled( $post->post_type ) || !in_array( $post->post_status, wpep_post_monitored_post_status_list() ) )
      return false;

    $request_data[ 'post_type' ] = $post->post_type;
    $request_data[ 'post_ids' ] = $request_data[ 'post_id'];
    $request_data[ 'user_ids' ] = $request_data[ 'user_id'];

    unset( $request_data[ 'post_id'] );
    unset( $request_data[ 'user_id'] );

    return self::post_type_activity_report( $request_data );
  }

  public static function user_id_post_id_action_tracking_list( $request_data ) {
    $request_data = API_Utils_Report::parse_activity_report( $request_data );

    if( $request_data === false )
      return false;

    if( !isset( $request_data[ 'post_id'] ) )
      return false;

    $post = get_post( intval( $request_data[ 'post_id'] ) );

    if( !wpep_content_type()->is_enabled( $post->post_type ) || !in_array( $post->post_status, wpep_post_monitored_post_status_list() ) )
      return false;

    $response = [
      'status'          => 'ok',
      'entries_header'  => [
        'event_name'            => __( "Event", "wpep" ),
        'event_segment'         => __( "Segment", "wpep" ),
        'created_at_formatted'  => __( "Tracked At", "wpep" )
      ],
      'entries'         => [],
      'pagination'      => [
        'current'       => ( isset( $request_data[ 'pagination' ] ) ? intval( $request_data[ 'pagination' ] ) : 1 ),
        'total'         => 0,
        'total_entries' => 0
      ]
    ];

    $get_entries_where = [
      'user_id'   => intval( $request_data[ 'user_id'] ),
      'post_id'   => intval( $request_data[ 'post_id'] )
    ];

    if( isset( $request_data[ 'report_time_from' ] ) && isset( $request_data[ 'report_time_to' ] ) )
      $get_entries_where[ 'created_at' ] = [
        '>=' => date( "Y-m-d H:i:s", $request_data[ 'report_time_from' ] ),
        '<=' => date( "Y-m-d H:i:s", $request_data[ 'report_time_to' ] )
      ];

    $tracked_events = WPEP_Entity_ActionTracking::instance()->get_tracked_events( $get_entries_where, [
      'limit'       => 20,
      'offset'      => ( $response['pagination'][ 'current' ] - 1 ) * 20,
      'count_total' => true
    ] );

    $response[ 'pagination' ][ 'total' ] = ceil( $tracked_events[ 'total' ] / 20 );
    $response[ 'pagination' ][ 'total_entries' ] = $tracked_events[ 'total' ];

    foreach( $tracked_events[ 'results' ] as $tracked_event ) {
      $response[ 'entries' ][] = [
        'post_id'              => $tracked_event->post_id,
        'user_id'              => $tracked_event->user_id,
        'event_name'           => wpep_controller()->actionTracking->get_event_name( $tracked_event->type ),
        'event_type'           => $tracked_event->type,
        'event_segment'        => $tracked_event->segment,
        'created_at'           => $tracked_event->created_at,
        'created_at_formatted' => wpep_format_date_with_time( $tracked_event->created_at )
      ];
    }

    return $response;
  }

}