<?php

namespace WPEP\API;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\Entity\Discussion as Entity_Discussion;

class User {

  public static function template_change_password() {
    return [
      'status'  => 'ok',
      'content' => wpep_template_get( 'user/profile/form-change-password.php' )
    ];
  }

  public static function template_content_created( $request_information ) {
    if( !isset( $request_information[ 'user_id' ] ) )
      return false;

    $user_id = intval( $request_information[ 'user_id' ] );

    return wpep_content_primary_list_by_id_map( wpep_content_type()->get_user_created_content_ids( $user_id ), [
      'courses_per_row' => WPEP_ITEMS_PER_ROW_DYNAMIC,
      'navigation_xhr'  => [
        'action' => 'wpep/user/template-content-created',
        'args'   => [
          'user_id' => $user_id
        ]
      ]
    ] );
  }

  public static function template_content_purchased( $request_information ) {
    if( !isset( $request_information[ 'user_id' ] ) )
      return false;

    $user_id = intval( $request_information[ 'user_id' ] );

    if( !wpep_controller()->contentLibrary->is_integrated() || !\WPEP\ACL\User::profile( $user_id ) )
      return false;

    return do_shortcode( '[' . WPEP_SHORT_CODE_PREFIX . 'index display_header="0" categories_per_row="1" category_navigation="0" display_restriction="access" pagination_per_page="0" navigation_xhr="' . esc_attr( json_encode( [
        'action' => 'wpep/user/template-content-purchased',
        'args'   => [
          'user_id' => $user_id
        ]
      ] ) ) . '"]' );
  }

  public static function delete( $request_data ) {
    if( !isset( $request_data['user_id'] ) || !isset( $request_data['nonce'] ) )
      return false;

    if( wp_verify_nonce( $request_data['nonce'], 'wpep-account-delete-' . $request_data['user_id'] ) === false )
      return false;

    if( $request_data['user_id'] != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return false;

    if( user_can( $request_data['user_id'], WPEP_CAP_MANAGE_SETTINGS ) )
      return [
        'status' => 'error',
        'error'  => __( 'Admin Users cannot delete their own accounts from the User Profile, to prevent mistakes.', "wpep" )
      ];

    wp_delete_user( $request_data[ 'user_id' ] );

    return [
      'status'       => 'ok',
      'redirect_url' => wpep_get_setting( 'user-account-delete-success-page', get_site_url() )
    ];
  }

  public static function privacy_confirm_agreement( $request_data ) {
    $form_information = $request_data['form_information'];

    if( !isset( $form_information['nonce'] ) )
      return [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ];

    if( wp_verify_nonce( $form_information['nonce'], 'wpep-agreement-' . get_current_user_id() ) === false )
      return [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ];

    if( wpep_get_setting_flag( 'user-service-agreement-enabled' )
        && ( !isset( $form_information['service_agreement'] ) || intval( $form_information['service_agreement'] ) != 1 ) )
      return [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-service-agreement-error' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ];

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' )
        && ( !isset( $form_information['privacy_agreement'] ) || intval( $form_information['privacy_agreement'] ) != 1 ) )
      return [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-privacy-agreement-error' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ];

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) )
      update_user_meta( get_current_user_id(), WPEP_USER_META_AGREEMENT_PRIVACY_DATE, date( "Y-m-d" ) );
    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) )
      update_user_meta( get_current_user_id(), WPEP_USER_META_AGREEMENT_TOS_DATE, date( "Y-m-d" ) );

    return [
      'status'              => 'success',
      'notification'        => wpep_get_setting_translated( 'user-agreement-notification-success' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'refresh'             => 1
    ];
  }

  public static function template_notification_management( $request_data ) {
    $user_id     = ( isset( $request_data['user_id'] ) ? absint( $request_data['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return false;

    return [
      'status'  => 'ok',
      'content' => wpep_template_get( 'user/notification-management.php', [
        'discussions'              => wpep_controller()->discussion->user_get_discussions_information( $user_id, [
          'limit'       => false,
          'count_total' => false
        ] ),
        'user_id' => $user_id,
        'has_email_notifications' => !intval( get_user_meta( $user_id, 'wpep_disable_email_notifications', true ) ),
        'has_weekly_recap'        => !intval( get_user_meta( $user_id, 'wpep_disable_email_weekly_recap', true ) )
      ] )
    ];
  }

  public static function notification_status_flag( $request_data ) {
    if( !isset( $request_data[ 'notification_identifier' ] ) )
      return false;

    $notification_identifier = $request_data[ 'notification_identifier' ];

    $user_id     = ( isset( $request_data['user_id'] ) ? absint( $request_data['user_id'] ) : get_current_user_id() );
    $notification_status = ( isset( $request_data[ 'notification_status' ] ) ? absint( $request_data[ 'notification_status' ] ) : 0 );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return false;

    if( $notification_identifier === 'platform_wide' ) {
      $input_disable_email_notifications = $notification_status ? 0 : 1;
      $current_disable_email_notifications = intval( get_user_meta( $user_id, 'wpep_disable_email_notifications', true ) );

      if( $input_disable_email_notifications !== $current_disable_email_notifications )
        update_user_meta( $user_id, 'wpep_disable_email_notifications', $input_disable_email_notifications );
    } elseif( $notification_identifier === 'weekly_recap' ) {
      $input_disable_email_notifications = $notification_status ? 0 : 1;
      $current_disable_email_notifications = intval( get_user_meta( $user_id, 'wpep_disable_email_weekly_recap', true ) );

      if( $input_disable_email_notifications !== $current_disable_email_notifications )
        update_user_meta( $user_id, 'wpep_disable_email_weekly_recap', $input_disable_email_notifications );
    } else {

      $discussion_id = intval( str_replace( 'discussion:', '', $notification_identifier ) );

      $discussion  = Entity_Discussion::instance()->get_discussion( [ 'id' => $discussion_id ] );

      $integration = wpep_controller()->discussion->get( $discussion->namespace );

      if( $integration === false )
        return false;

      if( !$integration->current_user_can_view( $discussion->id ) )
        return false;

      Entity_Discussion::instance()->set_discussion_user_data( WPEP_DISCUSSION_USER_DATA_EMAIL_NOTIFICATION_STATUS, $notification_status, $discussion->id, $user_id );
    }

    return [
      'status'              => 'ok',
      'timestamp'           => time(),
      'notification_status' => $notification_status
    ];
  }

  public static function upload_file( $request_information ) {
    if( !is_user_logged_in() )
      return false;

    if( !isset( $request_information['upload_name'] )
        || !isset( $request_information['upload_type'] )
        || !isset( $request_information['upload_content'] ) )
      return false;

    $post_id = ( isset( $request_information['post_id'] ) ? intval( $request_information['post_id'] ) : 0 );

    if( $post_id === 0 ) {
      if( !isset( $request_information[ 'discussion_id' ] ) )
        return false;

      $discussion = Entity_Discussion::instance()->get_discussion( [ 'id' => intval( $request_information[ 'discussion_id' ] ) ] );

      if( $discussion === null )
        return false;

      $post_id = $discussion->post_id;

      if( $post_id === 0 )
        return false;
    }

    if( !wpep_controller()->contentLibrary->has_access_to_post( $post_id ) )
      return false;

    $upload_file_name = $request_information['upload_name'];
    $upload_file_extension = substr( $upload_file_name, strrpos( $upload_file_name, '.') + 1 );

    $upload_file_name = substr( $upload_file_name, 0, strlen( $upload_file_name ) - strlen( $upload_file_extension ) - 1 );

    $file_name    = get_current_user_id() . '-' . time() . '-' . wpep_label_to_alias( $upload_file_name ) . '.' . $upload_file_extension;
    $file_type    = $request_information['upload_type'];
    $file_content = base64_decode( $request_information['upload_content'] );

    if( $file_content === false )
      return false;

    $allowed_mime_types = get_allowed_mime_types();

    if( !in_array( $file_type, $allowed_mime_types ) )
      return false;

    $allowed_file_extensions = [];

    foreach( $allowed_mime_types as $k => $v )
      $allowed_file_extensions = array_merge( $allowed_file_extensions, explode( "|", $k ) );

    if ( !in_array( $upload_file_extension, $allowed_file_extensions ) )
      return false;

    $destination_directory = wpep_get_upload_dir() . DIRECTORY_SEPARATOR . 'user-file-uploads';
    $destination_url       = wpep_get_upload_dir( "baseurl" ) . '/' . 'user-file-uploads';

    if( !is_dir( $destination_directory ) )
      mkdir($destination_directory, 0755, true);

    file_put_contents($destination_directory . DIRECTORY_SEPARATOR . $file_name, $file_content );

    $attachment = [
      'post_mime_type' => @mime_content_type( $destination_directory . DIRECTORY_SEPARATOR . $file_name ),
      'guid'           => $destination_url . '/' . $file_name,
      'post_parent'    => 0,
      'post_title'     => $file_name,
    ];

    $new_attachment_id = wp_insert_attachment( $attachment, $file_name, 0 );

    if( is_wp_error( $new_attachment_id ) )
      return [
        'status'  => 'error',
        'error'   => $new_attachment_id->get_error_message()
      ];

    update_post_meta( $new_attachment_id, '_wp_attached_file', $destination_directory . '/' . $file_name );
    update_post_meta( $new_attachment_id, 'wpep_uploaded_file_user_id', get_current_user_id() );

    if( !function_exists( 'wp_generate_attachment_metadata' ) )
      require ( ABSPATH . 'wp-admin/includes/image.php' );

    $attach_data = wp_generate_attachment_metadata( $new_attachment_id, get_attached_file( $new_attachment_id ) );

    wp_update_attachment_metadata( $new_attachment_id,  $attach_data );

    return [
      'status' => 'ok',
      'link'   => wp_get_attachment_url( $new_attachment_id )
    ];
  }

  public function template_statistics_current_user( $request_args ) {
    $user_id = ( isset( $request_args[ 'user_id' ] ) && current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ? $request_args[ 'user_id' ] : get_current_user_id() );

    return '<div data-ec-component="interval-navigation-panel" 
                 data-ec-component-action="wpep/system/statistics-current-user/configuration-report"
                 data-ec-component-args="' . esc_attr( json_encode( [ 'scu_user_id' => $user_id ] ) ) . '"></div>';
  }

}