<?php

namespace WPEP\API\User;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\Entity\Discussion as Entity_Discussion;

class Account {

  public static function update_profile( $request_information ) {
    if( !isset( $request_information['information'] ) )
      return false;

    $information = $request_information['information'];

    $user_id = ( isset( $information['wpep_user_profile_id'] ) ? intval( $information['wpep_user_profile_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return false;

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      return false;

    $wp_user_update = [];

    if( isset( $information['wpep_user_url'] ) && $information['wpep_user_url'] != $user->user_url ) {
      $user->user_url = sanitize_text_field( $information['wpep_user_url'] );

      $wp_user_update['user_url'] = $user->user_url;
    }

    if( isset( $information['wpep_user_first_name'] ) && $information['wpep_user_first_name'] != $user->first_name ) {
      $user->first_name = sanitize_text_field( $information['wpep_user_first_name'] );

      $wp_user_update['first_name'] = $user->first_name;
    }

    if( isset( $information['wpep_user_last_name'] ) && $information['wpep_user_last_name'] != $user->last_name ) {
      $user->last_name = sanitize_text_field( $information['wpep_user_last_name'] );

      $wp_user_update['last_name'] = $user->last_name;
    }

    if( isset( $information['wpep_user_description'] ) && $information['wpep_user_description'] != $user->description ) {
      $user->description = wpep_utilities_sanitize_input( $information['wpep_user_description'] );

      $wp_user_update['description'] = $user->description;
    }

    if( isset( $information['wpep_user_student_identification'] ) && $information['wpep_user_student_identification'] != get_user_meta( $user_id, WPEP_USER_META_STUDENT_IDENTIFICATION, true ) ) {
      update_user_meta( $user_id, WPEP_USER_META_STUDENT_IDENTIFICATION, sanitize_text_field( $information['wpep_user_student_identification'] ) );
    }

    if( isset( $information['wpep_user_location'] ) && $information['wpep_user_location'] != get_user_meta( $user_id, WPEP_USER_LOCATION_META, true ) ) {
      update_user_meta( $user_id, WPEP_USER_LOCATION_META, sanitize_text_field( $information['wpep_user_location'] ) );
    }

    if( isset( $information['wpep_user_privacy_type'] ) && wpep_public_profiles_enabled() ) {
      if( in_array( $information['wpep_user_privacy_type'], [ WPEP_USER_PROFILE_PRIVACY_TYPE_PUBLIC, WPEP_USER_PROFILE_PRIVACY_TYPE_PRIVATE ] )
        && $information['wpep_user_privacy_type'] != get_user_meta( $user_id, WPEP_USER_META_PROFILE_PRIVACY_TYPE, true ) )
        update_user_meta( $user_id, WPEP_USER_META_PROFILE_PRIVACY_TYPE, sanitize_text_field( $information['wpep_user_privacy_type'] ) );
    }

    if( isset( $information['wpep_student_supervisor'] ) && wpep_get_setting_flag( 'enable-student-supervisor' ) )
      wpep_user_update_student_supervisor( $user_id, $information['wpep_student_supervisor'] );

    $response = [
      'status'               => 'ok',
      'display_name'         => wpep_content_user_profile_format_name( $user ),
      'description'          => wpautop( make_clickable( $user->description ) )
    ];

    if( isset( $information['wpep_user_social_network'] ) ) {
      $current_social_networks = get_user_meta( $user_id, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, true );

      if( !is_array( $current_social_networks ) )
        $current_social_networks = [];

      foreach( wpep_user_profile_social_network_platform_list_active() as $social_network_key => $social_network_information ) {
        if( !isset( $information['wpep_user_social_network'][$social_network_key] ) )
          continue;

        $current_val = trim( $information['wpep_user_social_network'][$social_network_key] );

        if( $current_val === '' ) {
          unset( $current_social_networks[$social_network_key] );
          continue;
        }

        $current_social_networks[$social_network_key] = (
        isset( $social_network_information['type'] ) && $social_network_information['type'] == 'link'
          ? esc_url( $current_val )
          : sanitize_text_field( $current_val )
        );
      }

      update_user_meta( $user_id, WPEP_USER_META_SOCIAL_NETWORK_PROFILES, $current_social_networks );

      $response['social_networks_html'] = wpep_template_get( 'partial/user-profile-information-section-social-networks-open.php', [
        'user_social_networks'  => $current_social_networks
      ] );
    }

    do_action( 'wpep_user_profile_information_save', $information, $user );

    if( !empty( $wp_user_update ) )
      wp_update_user( [ 'ID' => $user_id ] + $wp_user_update );

    return $response;
  }

  /**
   * This method is not registered in the API, it's intended to be used by self::update_profile_picture & self::update_profile_cover at the moment
   * @param $request_information
   * @return array|bool
   */
  public static function update_photo_by_reference( $request_information ) {
    if( !isset( $request_information[ 'photo' ] )
        || !isset( $request_information[ 'reference' ] ) )
      return false;

    if ( !preg_match('/^data:image\/(\w+);base64,/', $request_information[ 'photo' ], $type) )
      return [
        'status'  => 'error',
        'error'   => __( "Image File is corrupted", "wpep" )
      ];

    $image_data = substr( $request_information[ 'photo' ], strpos( $request_information[ 'photo' ], ',' ) + 1 );
    $type = strtolower( $type[1] ); // jpg, png, gif

    if ( !in_array($type, [ 'jpg', 'jpeg', 'gif', 'png' ] ) )
      return [
        'status'  => 'error',
        'error'   => sprintf( __( "Allowed image files are : %s", "wpep" ), [ 'jpg', 'jpeg', 'gif', 'png' ] )
      ];

    $image_data = base64_decode($image_data);

    if ($image_data === false)
      return [
        'status'  => 'error',
        'error'   => __( "Image File is corrupted", "wpep" )
      ];

    $destination_directory = wpep_get_upload_dir() . DIRECTORY_SEPARATOR . 'user-profile-uploads';
    $destination_url       = wpep_get_upload_dir( "baseurl" ) . '/' . 'user-profile-uploads';
    $file_name             = $request_information[ 'reference' ] . '-' . time() . '.' . $type;

    if( !is_dir( $destination_directory ) )
      mkdir($destination_directory, 0755, true);

    file_put_contents($destination_directory . DIRECTORY_SEPARATOR . $file_name, $image_data );

    $attachment = [
      'post_mime_type' => @mime_content_type( $destination_directory . DIRECTORY_SEPARATOR . $file_name ),
      'guid'           => $destination_url . '/' . $file_name,
      'post_parent'    => 0,
      'post_title'     => $request_information[ 'reference' ],
    ];

    $new_attachment_id = wp_insert_attachment( $attachment, $file_name, 0 );
    update_post_meta( $new_attachment_id, '_wp_attached_file', $destination_directory . '/' . $file_name );
    update_post_meta( $new_attachment_id, 'wpep_uploaded_file_user_id', get_current_user_id() );

    if( !function_exists( 'wp_generate_attachment_metadata' ) )
      require ( ABSPATH . 'wp-admin/includes/image.php' );

    $attach_data = wp_generate_attachment_metadata( $new_attachment_id, get_attached_file( $new_attachment_id ) );

    wp_update_attachment_metadata( $new_attachment_id,  $attach_data );

    $attachment_image_src = wp_get_attachment_image_src( $new_attachment_id, 'full' );

    if( !isset( $attachment_image_src[ 0 ] ) )
      return [
        'status'  => 'error',
        'error'   => __( "Image upload has failed", "wpep" )
      ];

    return [
      'status'  =>  'ok',
      'url'     => $attachment_image_src[ 0 ]
    ];
  }

  public static function update_profile_picture( $request_information ) {
    if( !isset( $request_information['information'] ) )
      return false;

    $information = $request_information['information'];

    $user_id = ( isset( $information['wpep_user_id'] ) ? intval( $information['wpep_user_id'] ) : 0 );

    if( $user_id == false )
      $user_id = ( isset( $information['user_id'] ) ? intval( $information['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return false;

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      return false;

    $upload_response = self::update_photo_by_reference( [
      'photo'     => $information['picture'],
      'reference' => 'profile-picture-' . $user_id
    ] );

    if( $upload_response[ 'status' ] !== 'ok' )
      return $upload_response;

    $information['picture'] = $upload_response[ 'url' ];

    update_user_meta( $user_id, WPEP_USER_META_PROFILE_PICTURE, $information['picture'] );

    return [
      'status'        => 'ok',
      'picture'       => $information['picture']
    ];
  }

  public static function update_profile_cover( $request_information ) {
    if( !isset( $request_information['information'] ) )
      return false;

    $information = $request_information['information'];

    $user_id = ( isset( $information['wpep_user_id'] ) ? intval( $information['wpep_user_id'] ) : 0 );

    if( $user_id == false )
      $user_id = ( isset( $information['user_id'] ) ? intval( $information['user_id'] ) : get_current_user_id() );

    if( $user_id != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return false;

    $user = get_user_by( 'ID', $user_id );

    if( $user === false )
      return false;

    $upload_response = self::update_photo_by_reference( [
      'photo'     => $information['picture'],
      'reference' => 'cover-picture-' . $user_id
    ] );

    if( $upload_response[ 'status' ] !== 'ok' )
      return $upload_response;

    $information['picture'] = $upload_response[ 'url' ];

    update_user_meta( $user_id, WPEP_USER_META_COVER_PICTURE, $information['picture'] );

    return [
      'status'        => 'ok',
      'picture'       => $information['picture']
    ];
  }

  public static function change_password( $request_information ) {
    if( !isset( $request_information['information'] ) )
      return false;

    $user_id = get_current_user_id();
    $password_update_old = isset( $request_information[ 'information']['wpep_password_update_old'] ) ? $request_information[ 'information']['wpep_password_update_old'] : '';
    $password_update_new = isset( $request_information[ 'information']['wpep_password_update_new'] ) ? $request_information[ 'information']['wpep_password_update_new'] : '';
    $password_update_new_confirm = isset( $request_information[ 'information']['wpep_password_update_new_confirm'] ) ? $request_information[ 'information']['wpep_password_update_new_confirm'] : '';

    $user = get_user_by( 'id', $user_id );

    if ( ! wp_check_password( $password_update_old, $user->user_pass, $user->ID ) )
      return [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-profile-change-password-notification-wrong-existent' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ];

    if( $password_update_new !== $password_update_new_confirm )
      return [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-profile-change-password-notification-not-match' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ];

    wp_set_password( $password_update_new, $user_id );

    wp_set_auth_cookie( $user->ID );
    wp_set_current_user( $user->ID );
    do_action('wp_login', $user->user_login, $user);

    WPEP_Entity_ActionTracking::instance()->track_event( 0, $user->ID, WPEP_ACTION_TRACKING_TYPE_CHANGE_PASSWORD, '' );

    return [
      'status'              => 'ok',
      'notification'        => wpep_get_setting_translated( 'user-profile-change-password-notification-success' ),
      'notification_class'  => 'wpep-notification wpep-notification-success'
    ];
  }

}