<?php

namespace WPEP\API\Utils;

class Report {

  public static function configuration_for_statistics( $request_data ) {
    $system_report_start_time = strtotime( wpep_report_start_date() . ' 00:00:00' );

    $report_start_time  = ( isset( $request_data[ 'start_time'] ) ? intval($request_data[ 'start_time'] ) : $system_report_start_time );

    if( $report_start_time < $system_report_start_time )
      $report_start_time = $system_report_start_time;

    unset( $system_report_start_time );

    $current_time       = time();
    $today_noon_time    = strtotime( 'today 00:00:00', $current_time );
    $tomorrow_noon_time = strtotime('+1 day', $today_noon_time );

    $report_total_days = ceil( ( $tomorrow_noon_time - $report_start_time ) / DAY_IN_SECONDS );

    $response = [
      'navigation' => [],
      'navigation_start_time' => $report_start_time,
      'navigation_end_time'   => $tomorrow_noon_time
    ];

    if( !isset( $request_data[ 'restrict_navigation' ] ) || !in_array( 'week', $request_data[ 'restrict_navigation' ] ) ) {
      $response[ 'navigation' ][ 'week' ] = [
        'label'          => __( "Last Week", "wpep" ),
        'key'            => 'week',
        'time_from'      => strtotime('-1 week', $today_noon_time ),
        'time_to'        => $tomorrow_noon_time,
        'time_increment' => '+1 day'
      ];
    }

    if( $report_total_days > 7 && ( !isset( $request_data[ 'restrict_navigation' ] ) || !in_array( 'month_daily', $request_data[ 'restrict_navigation' ] ) ) ) {
      $response[ 'navigation' ][ 'month_daily' ] = [
        'label'          => __( "Last Month", "wpep" ),
        'key'            => 'month_daily',
        'time_from'      => strtotime('-1 month', $today_noon_time ),
        'time_to'        => $tomorrow_noon_time,
        'time_increment' => '+1 day',
      ];
    }

    if( $report_total_days > 30 ) {
      $response[ 'navigation' ][ 'yearly' ] = [
        'label'          => __( "Last Year", "wpep" ),
        'key'            => 'yearly',
        'time_from'      => strtotime('first day of this month -1 year', $today_noon_time ),
        'time_to'        => $tomorrow_noon_time,
        'time_increment' => '+1 month'
      ];
    }

    $response[ 'navigation' ][ 'all_time' ] = [
      'label'          => __( "All time", "wpep" ),
      'key'            => 'yearly',
      'time_from'      => $report_start_time,
      'time_to'        => $tomorrow_noon_time,
      'time_increment' => ( ( $tomorrow_noon_time - $report_start_time ) > YEAR_IN_SECONDS ? '+4 months' : '+1 month' )
    ];

    if( !isset( $request_data[ 'restrict_navigation' ] ) || !in_array( 'advanced', $request_data[ 'restrict_navigation' ] ) ) {
      $response[ 'navigation' ][ 'advanced' ] = [
        'key'    => 'advanced',
        'label'  => 'Advanced Filter',
        'type'   => 'custom',
        'field_group' => 'advanced',
        'fields' => [
          'from' => [
            'label'   => __( "From", "wpep" ),
            'type'    => 'datetime',
            'default' => wpep_format_date_with_time( $today_noon_time - WEEK_IN_SECONDS, 'datetimepicker' )
          ],
          'to' => [
            'label'   => __( "To", "wpep" ),
            'type'    => 'datetime',
            'default' => wpep_format_date_with_time( $tomorrow_noon_time, 'datetimepicker' )
          ],
          'increment' => [
            'label'   => __( "Increment", "wpep" ),
            'type'    => 'select',
            'options' => [
              '+1 day'    => __( "Daily", "wpep" ),
              '+1 week'   => __( "Weekly", "wpep" ),
              '+1 month'  => __( "Monthly", "wpep" ),
              '+1 hour'   => __( "By the Hour", "wpep" ),
            ]
          ]
        ],
        'submit' => __( "Filter", "wpep" )
      ];
    }

    return $response;
  }

  /**
   * @param $request_data
   * @return bool
   */
  public static function parse_activity_report( $request_data ) {
    $activity_report_from = (
      isset( $request_data['advanced'] ) && isset( $request_data['advanced'][ 'from' ] )
        ? ( is_numeric( $request_data['advanced'][ 'from' ] ) ? intval( $request_data['advanced'][ 'from' ] ) : strtotime( $request_data['advanced'][ 'from' ] ) )
        : (
          isset( $request_data[ 'report_time_from' ] )
            ? ( is_numeric( $request_data[ 'report_time_from' ] ) ? intval( $request_data[ 'report_time_from' ] ) : strtotime( $request_data[ 'report_time_from' ] ) )
            : false
      )
    );

    $activity_report_to = (
      isset( $request_data['advanced'] ) && isset( $request_data['advanced'][ 'to' ] )
        ? ( is_numeric( $request_data['advanced'][ 'to' ] ) ? intval( $request_data['advanced'][ 'to' ] ) : strtotime( $request_data['advanced'][ 'to' ] ) )
        : (
          isset( $request_data[ 'report_time_to' ] )
            ? ( is_numeric( $request_data[ 'report_time_to' ] ) ? intval( $request_data[ 'report_time_to' ] ) : strtotime( $request_data[ 'report_time_to' ] ) )
            : false
      )
    );

    $activity_report_increment = (
      isset( $request_data['advanced'] ) && isset( $request_data['advanced'][ 'increment' ] )
        ? $request_data['advanced'][ 'increment' ]
        : ( isset( $request_data[ 'report_time_increment' ] ) ? $request_data[ 'report_time_increment' ] : DAY_IN_SECONDS )
    );

    if( $activity_report_from === false || $activity_report_to === false )
      return false;

    if( is_numeric( $activity_report_increment ) ) {
      $activity_report_increment = intval( $activity_report_increment );
    } else {
      // Split this, without using another variable.
      $activity_report_increment = explode( " ", $activity_report_increment );

      $activity_report_increment[1] = strtolower( $activity_report_increment[1] );

      if( !in_array( $activity_report_increment[1], [ 'hour', 'day', 'week', 'month', 'year', 'days', 'weeks', 'months', 'years' ] ) )
        return false;

      if( substr( $activity_report_increment[0], 0, 1 ) !== '+'
            && substr( $activity_report_increment[0], 0, 1 ) !== '-' )
        return false;

      // Okay, we'll ensure we have a number, or a lot of junk with +/-
      if( absint( str_replace( [ '+', '-' ], '', $activity_report_increment[0] ) ) === 0 )
        return false;

      // PHP Time patterns we allow : +/-NUMERIC DAY|WEEK|MONTH|YEAR|DAYS|WEEKS|MONTHS|YEARS
      if( count( $activity_report_increment ) !== 2 )
        return false;

      // Glue Back
      $activity_report_increment = implode( " ", $activity_report_increment );
    }

    $activity_report_limit = $activity_report_to;

    if( is_numeric( $activity_report_increment ) )
      $activity_report_limit += $activity_report_increment;
    else
      $activity_report_limit = strtotime( $activity_report_increment, $activity_report_limit );

    $request_data[ 'report_time_slots' ] = [];

    $previous_i = false;

    for( $i = $activity_report_from; $i < $activity_report_limit; ) {
      if( $previous_i !== false )
        $request_data['report_time_slots'][ $previous_i ] = $i;

      $previous_i = $i;

      if( is_numeric( $activity_report_increment ) ) {
        $i += $activity_report_increment;

        continue;
      }

      $i = @strtotime( $activity_report_increment, $i );

      if( $i === -1 )
        break;
    }

    $request_data[ 'report_time_from' ] = $activity_report_from;
    $request_data[ 'report_time_to' ]   = $activity_report_to;

    return $request_data;
  }

}