<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Container\FormFields as WPEP_FormFields;
use WPEP\Container\DataWalker as WPEP_DataWalker;

class Import {

  use WPEP_FormFields,
    WPEP_DataWalker;

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_import_process', [ $this, 'import_process' ] );
  }

  public function import_process() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )  )
      exit( '<div class="wpep-notification wpep-notification-danger">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    if( isset( $_POST[ 'import_file' ] ) ) {
      $response = $this->_prepare_temporary_directory( $_POST[ 'import_file' ] );

      if( $response !== true ) {
        echo '<div class="wpep-notification wpep-notification-danger">' . $response . '</div>';

        $this->_select_file();

        exit();
      } else {
        echo '<div class="wpep-notification wpep-notification-info">' . __( "Checking Import Contents", "wpep" ) . '</div>';

        exit();
      }
    } else if( is_dir( $this->_get_temporary_directory_path() ) ) {

      if( file_exists( $this->_get_temporary_directory_path() . '/attachments.json' ) ) {
        $this->_parse_temporary_directory_attachments();
      } else if( is_dir( $this->_get_temporary_directory_path() ) ) {
        $content = @json_decode( file_get_contents( $this->_get_temporary_directory_path() . '/data.json' ), true );

        if( is_array( $content ) && isset( $content['settings'] ) ) {
          wpep_data_system()->data_import_array( [ 'settings' => $content['settings'] ] );
          echo '<div class="wpep-notification wpep-notification-success">' . __( "Settings have been imported", "wpep" ) . '</div>';

          unset( $content['settings'] );

          file_put_contents( $this->_get_temporary_directory_path() . '/data.json', json_encode( $content ) );
          exit;
        } elseif( is_array( $content ) ) {
          wpep_data_system()->data_import_array( $content );
          echo '<div class="wpep-notification wpep-notification-success">' . __( "Content has been imported", "wpep" ) . '</div>';
        } else {
          echo '<div class="wpep-notification wpep-notification-danger">' . __( "Import Content is invalid, skipping.", "wpep" ) . '</div>';
        }

        wpep_file_system_remove_dir( $this->_get_temporary_directory_path() );

        $this->_select_file();
      } else {
        echo '<div class="wpep-notification wpep-notification-danger">' . __( "Unexpected Error, please try to import your content again.", "wpep" ) . '</div>';

        wpep_file_system_remove_dir( $this->_get_temporary_directory_path() );

        $this->_select_file();
      }

      exit;
    }

    $this->_select_file();

    exit;
  }

  private function _select_file() {

    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Import File", "wpep" ) . '</h3>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="dynamic"
                    value="' . __( "Start Import", "wpep" ) . '"/>';
    echo '</div>';

    echo '<div class="wpep-import-file-container">';
    $this->display_file_field("import_file", "import_file" );
    echo    '<p class="wpep-description">' . sprintf( __( "Currently works only with %s Backup Files", "wpep" ), WPEP_PLUGIN_NAME ) . '</p>';
    echo '</div>';


  }

  private function _prepare_temporary_directory( $file_path ) {
    $upload_dir   = wp_upload_dir();

    $zip_location = str_replace( $upload_dir['baseurl'], $upload_dir['basedir'], $file_path );

    $current_import_destination = $this->_get_temporary_directory_path();

    _wpep_http_locked_directory_check( $current_import_destination );

    $zip = new \ZipArchive();
    if( $zip->open( $zip_location ) === TRUE ) {
      $zip->extractTo( $current_import_destination );
      $zip->close();
    } else {
      return __(  "Could not open the ZIP File you have provided", "wpep" );
    }

    if( !is_file( $current_import_destination . '/' . 'data.json' ) ) {
      wpep_file_system_remove_dir( $current_import_destination );

      return "data.json is missing from the uploaded file, please verify if you've uploaded the right thing.";
    }

    return true;
  }

  private function _get_temporary_directory_path() {
    return wpep_get_upload_dir() . '/backup-temporary';
  }

  private function _parse_temporary_directory_attachments() {
    $attachments_data = @json_decode( file_get_contents( $this->_get_temporary_directory_path() . '/attachments.json' ), true );

    if( !isset( $attachments_data['base_url'] ) || $attachments_data['base_url'] == get_site_url() || !isset( $attachments_data[ 'map' ] ) || empty( $attachments_data[ 'map' ] ) ) {
      echo '<div class="wpep-notification wpep-notification-success">' . __( "Media Files import not required", "wpep" ) . '</div>';

      @unlink( $this->_get_temporary_directory_path() . '/attachments.json' );

      exit;
    }

    $unique_identifier = date( "Y-m-d" );

    $current_import_destination_folder = wpep_get_upload_dir( 'basedir' ) . DIRECTORY_SEPARATOR . 'import-' . $unique_identifier;
    $current_import_destination_url    = wpep_get_upload_dir( 'baseurl' ) . '/import-' . $unique_identifier;

    if( !is_dir( $current_import_destination_folder ) )
      mkdir( $current_import_destination_folder, 0755, true );

    $replace_attachment_url_map = [];

    foreach( $attachments_data[ 'map' ] as $previous_attachment_id => $previous_attachment_url ) {
      if( count( $replace_attachment_url_map ) > 3 )
        break;

      $extension = ( substr( $previous_attachment_url, strrpos( $previous_attachment_url, '.', -1 ), strlen( $previous_attachment_url ) ) );

      rename(
        $this->_get_temporary_directory_path() . DIRECTORY_SEPARATOR . $previous_attachment_id . $extension,
        $current_import_destination_folder . DIRECTORY_SEPARATOR . $previous_attachment_id . $extension
      );

      $new_attachment_url = $current_import_destination_url . '/' . $previous_attachment_id . $extension;

      $attachment = [
        'post_mime_type' => @mime_content_type( $current_import_destination_folder . '/' . $previous_attachment_id . $extension ),
        'guid'           => $new_attachment_url,
        'post_parent'    => 0,
        'post_title'     => 'Imported ' . $previous_attachment_id,
      ];

      $new_attachment_id = wp_insert_attachment( $attachment, 'wpep-import-' . $previous_attachment_id . $extension, 0 );
      update_post_meta( $new_attachment_id, '_wp_attached_file', $current_import_destination_folder . '/' . $previous_attachment_id . $extension );

      $attach_data = wp_generate_attachment_metadata( $new_attachment_id, get_attached_file( $new_attachment_id ) );

      wp_update_attachment_metadata( $new_attachment_id,  $attach_data );

      $replace_attachment_url_map[$previous_attachment_url] = $new_attachment_url;

      unset( $attachments_data['map'][ $previous_attachment_id ] );
    }

    $this->data_walker_setup( $replace_attachment_url_map );

    $content_entries = @json_decode( file_get_contents( $this->_get_temporary_directory_path() . '/data.json' ), true );

    $content_entries = $this->data_walker_replace_assets_from_entries( $content_entries );

    @file_put_contents( $this->_get_temporary_directory_path() . '/data.json', json_encode( $content_entries ) );

    if( !empty( $attachments_data[ 'map' ] ) ) {
      @file_put_contents( $this->_get_temporary_directory_path() . '/attachments.json', json_encode( $attachments_data ) );

      echo '<div class="wpep-notification wpep-notification-info">' . sprintf( __( "%s remaining media files to be imported", "wpep" ), count( $attachments_data[ 'map' ] ) ) . '</div>';
      exit;
    }

    echo '<div class="wpep-notification wpep-notification-success">' . __( "Media Files have been imported into the Media Library", "wpep" ) . '</div>';

    @unlink( $this->_get_temporary_directory_path() . '/attachments.json' );
    exit;
  }

}
