<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Container\FilterProxy as WPEP_FilterProxy;

class Post {

  use WPEP_FilterProxy;

  public function setup() {
    add_action( 'save_post', [ $this, '_handle_save' ], 10 );
    add_filter( 'wpep_admin_save_post_ajax', [ $this, '_wpep_admin_save_post_ajax' ], 10, 2 );

    add_action( 'edit_form_after_title', [ $this, '_display_builder' ] );
    add_filter( 'wpep_meta_boxes', [ $this, '_init_custom_meta_boxes' ] );

    if( wpep_get_setting_flag( 'enable-post-author-field' ) )
      add_filter( 'wp_dropdown_users_args', [ $this, '_wp_dropdown_users_args' ] );
  }

  public function _init() {
    if( wpep_get_setting( 'post-list-type-by-individual-author' ) === 'mine'
        && !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      add_filter('pre_get_posts', [ $this, '_pre_get_posts_restrict_individual_authors' ] );

    if( !current_user_can( 'manage_options' ) && current_user_can( WPEP_CAP_LEARNING_CENTER ) )
      add_action( 'admin_menu', [ $this, '_action_register_menu' ], 200 );
  }

  public function _handle_save( $post_id ) {
    global $post_type;

    if ( !is_admin() || !wpep_content_type()->is_enabled( $post_type ) )
      return;

    $builderIntegration = wpep_content_type()->standardContentTypes[ $post_type ]->get_builder_instance();

    if( $builderIntegration === false )
      return;

    $entity_id_map = ( isset( $_POST['wpep_entity_id_map'] ) ? $_POST['wpep_entity_id_map'] : [] );

    if( is_string( $entity_id_map ) )
      $entity_id_map = json_decode( stripslashes( $entity_id_map ), true );

    if( !is_array( $entity_id_map ) )
      $entity_id_map = [];

    $builderIntegration->save_from_post( $post_id, $entity_id_map );
  }

  public function _wpep_admin_save_post_ajax( $entity_id_map, $post_id ) {
    $post_type = get_post_type( $post_id );

    $builderIntegration = wpep_content_type()->standardContentTypes[ $post_type ]->get_builder_instance();

    if( $builderIntegration === false )
      return $entity_id_map;

    return $builderIntegration->save_from_post( $post_id, $entity_id_map );
  }

  public function _display_builder() {
    global $post_type;

    if ( !is_admin() || !wpep_content_type()->is_enabled( $post_type ) )
      return;

    $builderIntegration = wpep_content_type()->standardContentTypes[ $post_type ]->get_builder_instance();

    if( $builderIntegration === false )
      return;

    $builderIntegration->display();
  }

  public function _action_register_menu() {
    global $menu, $pagenow;

    if( $pagenow !== 'post-new.php' || !isset( $_GET['post_type'] ) )
      return;

    if( !wpep_content_type()->is_enabled( $_GET['post_type'] ) )
      return;

    $contentType = wpep_content_type()->contentTypes[ $_GET['post_type'] ];

    $menu_slug = 'post-new.php?post_type=' . $contentType->get_post_type();

    foreach($menu as $k=>$item){
      if ($item[2] == $menu_slug )
        return;
    }

    wpep_admin_register_menu(
      'wpep-access-helper',
      'Create ' . $contentType->get_singular_name(),
      'edit_' . $contentType->get_cap_name() . 's',
      $menu_slug,
      false
    );
  }

  public function _pre_get_posts_restrict_individual_authors( $query ) {
    global $pagenow;

    if( !$query->is_admin || $pagenow !== 'edit.php' )
      return $query;

    if( $query->post_type === 'any' )
      return $query;

    if( !wpep_content_type()->is_enabled( $query->get('post_type') ) )
      return $query;

    if( current_user_can( 'edit_others_' . wpep_content_type()->contentTypes[ $query->get('post_type') ]->get_cap_name() . 's' ) )
      return $query;

    $this->_filter_proxy_setup( [
      'views_edit-' . $query->get('post_type') => function( $views ) {
        unset($views['all']);
        unset($views['publish']);
        unset($views['draft']);
        unset($views['pending']);
        unset($views['yoast_cornerstone']);
        return $views;
      }
    ] );

    $query->set('author', get_current_user_id() );

    return $query;
  }

  public function _wp_dropdown_users_args( $args ) {
    if( !in_array( get_post_type(), wpep_content_primary_available_post_types() ) )
      return $args;

    unset( $args['who'] );

    $args['role__in'] = [ 'administrator', 'author', WPEP_USER_ROLE_INDIVIDUAL_CONTENT_MANAGER ];

    if( get_post_type() == WPEP_POST_TYPE_COURSE )
      $args['role__in'][] = WPEP_USER_ROLE_INDIVIDUAL_COURSES_MANAGER;
    else if( get_post_type() == WPEP_POST_TYPE_E_BOOK )
      $args['role__in'][] = WPEP_USER_ROLE_INDIVIDUAL_E_BOOKS_MANAGER;
    else if( get_post_type() == WPEP_POST_TYPE_VIDEO )
      $args['role__in'][] = WPEP_USER_ROLE_INDIVIDUAL_VIDEOS_MANAGER;
    else if( get_post_type() == WPEP_POST_TYPE_OFFER )
      $args['role__in'][] = WPEP_USER_ROLE_INDIVIDUAL_OFFERS_MANAGER;

    return $args;
  }

  public function _init_custom_meta_boxes(array $meta_boxes) {
    if( !wpep_get_setting_flag( 'enable-post-member-magnet-option' ) )
      return $meta_boxes;

    $meta_boxes['wpep_member_magnet']  = [
      'id'         => 'wpep_member_magnet',
      'title'      => __('Member Magnet Option Configuration', "wpep"),
      'post_types' => [ WPEP_POST_TYPE_COURSE, WPEP_POST_TYPE_ASSESSMENT, WPEP_POST_TYPE_E_BOOK, WPEP_POST_TYPE_OFFER ],
      'context'    => 'advanced',
      'priority'   => 'high',
      'fields'     => [
        [
          'name'    => __( 'Enable', "wpep" ),
          'desc'    => '<p>' . __( "The Member Magnet option allows you to make it mandatory for users to login before accessing the content.", "wpep" ) . '</p>',
          'id'      => 'wpep_member_magnet_option_status',
          'type'    => 'select',
          'options' => wpep_flag_setting_field_options()
        ],
        [
          'name'    => __( 'Default View', "wpep" ),
          'desc'    => '<p>' . __( "The page that will be displayed on page load, normally login", "wpep" ) . '</p>',
          'id'      => 'wpep_member_magnet_option_form_view',
          'type'    => 'select',
          'options' => [
            'login'          => __( 'Login', "wpep" ),
            'register'       => __( 'Register', "wpep" ),
            'express'        => __( "Express - Login & Register", "wpep" ),
            'reset_password' => __( 'Reset Password', "wpep" ),
          ]
        ],
        [
          'name'    => __('Content Before Authentication Form', "wpep"),
          'id'      => 'wpep_member_magnet_option_text_before',
          'type'    => 'editor'
        ],
        [
          'name'    => __('Content After Authentication Form', "wpep"),
          'id'      => 'wpep_member_magnet_option_text_after',
          'type'    => 'editor'
        ],
        [
          'name'        => __('Optional - Registration Redirect URL', "wpep"),
          'id'          => 'wpep_member_magnet_option_registration_redirect_url',
          'type'        => 'text',
          'description' => __( "Redirect the Student to a Custom Thank You Page, if an account was just created.", "wpep" )
        ],
      ]
    ];

    return $meta_boxes;
  }

}
