<?php

namespace WPEP\Container;

use WPEP\Controller as WPEP_Controller;

trait Migration {

  /**
   * @var Migration\Admin
   */
  public $admin;

  public $has_admin                    = false;
  public $namespace                    = false;
  public $plugin_file_path             = false;
  public $plugin_migration_folder_path = false;
  public $option_name                  = false;
  public $option_progress_name         = false;
  public $current_version              = false;
  public $current_version_available    = false;
  public $migration_list               = [];

  public function get_name() {
    return WPEP_PLUGIN_NAME;
  }

  public function get_menu_name() {
    return "Migration";
  }

  public function trait_setup( $variables ) {
    $this->_setup_variables( $variables );

    register_activation_hook( $this->plugin_file_path, [ $this, '_plugin_activation_hook' ] );

    if( method_exists( $this, 'wpmu_add_new_blog' ) )
      add_action( 'wpmu_new_blog', [ $this, 'wpmu_add_new_blog' ] );

    if( method_exists( $this, 'wpmu_drop_tables' ) )
      add_filter( 'wpmu_drop_tables', [ $this, 'wpmu_drop_tables' ] );

    if( $this->has_admin )
      $this->admin = new Migration\Admin( $this );

    WPEP_Controller::instance()->migrationLibrary->register( $this );
  }

  private function _setup_variables( $variables = [] ) {
    if( is_array( $variables ) && !empty( $variables ) ) {
      foreach( $variables as $variable_name => $variable_value )
        $this->$variable_name = $variable_value;
    }

    $this->current_version = get_option( $this->option_name, '0.0.0' );

    if( $this->namespace === false ) {
      $this->namespace = basename( $this->plugin_file_path );
      $this->namespace = str_replace( '.php', '',  $this->namespace );
    }
  }

  public function get_current_migration_version() {
    foreach( $this->migration_list as $migration_version => $migration_file )
      if( version_compare( $migration_version, $this->current_version, ">")  )
        return $migration_version;

    return false;
  }

  public function is_migration_required() {
    return version_compare( $this->current_version_available, $this->current_version, ">");
  }

  public function migrate_to_version( $version_number ) {
    @set_time_limit(0);
    @ini_set('memory_limit','512M');

    $last_migration = get_option( $this->option_progress_name, 0 );

    if( time() - $last_migration < 30 )
      return "<p>" . sprintf( __( "Migration currently marked in progress, it started %s seconds ago.", "wpep" ), ( $last_migration == 0 ? 'too many' : time() - $last_migration ) ). "</p>" .
             '<p><a class="button button-primary wpep-migration-continue">' . __( "Retry", "wpep" ) . '</a></p>';

    if( !version_compare( $version_number, $this->current_version, ">") && $this->current_version != 0 )
      return "<p>" . __( "This version has already been installed, skipping", "wpep" ) . "</p>";

    update_option( $this->option_progress_name, time() );

    $response = $this->_migrate_to_version( $version_number );

    delete_option( $this->option_progress_name );

    return $response;
  }

  private function _migrate_to_version( $migrate_to_version ) {
    $migrate_to_version_response = '';

    $migration_okay = true;

    foreach( $this->migration_list as $migration_version => $migration_file ) {
      if ( version_compare( $this->current_version, $migration_version ) != -1 )
        continue;

      if( !file_exists( $this->plugin_migration_folder_path . $migration_file ) ){
        $migrate_to_version_response .= '<p class="wpep-notification wpep-notification-danger">' . sprintf( __( "File not found %s.", "wpep" ), $this->plugin_migration_folder_path . $migration_file ). '</p>';
        $migrate_to_version_response .= '<p><a class="button button-primary wpep-migration-continue">' . __( "Retry", "wpep" ) . '</a></p>';
        $migration_okay = false;
        break;
      }

      if ( version_compare( $migrate_to_version, $migration_version, "<") )
        break;

      ob_start();

      require_once( $this->plugin_migration_folder_path . $migration_file );

      $migrate_to_version_response = ob_get_contents() . $migrate_to_version_response;

      ob_end_clean();

      if( strpos( $migrate_to_version_response, 'wpep-migration-failed-message' ) !== false
          || strpos( $migrate_to_version_response, 'wpep-migration-repeat-file-message' ) !== false ) {
        $migration_okay = false;

        break;
      }
    }

    if( $migration_okay ) {
      $this->current_version = $migrate_to_version;

      update_option( $this->option_name, $migrate_to_version );
    }

    return $migrate_to_version_response;
  }

  public function _plugin_activation_hook( $network_wide ) {
    if (function_exists( 'is_multisite' ) && is_multisite() && $network_wide ) {
      $this->_plugin_activation_hook_network();
      return;
    }

    $this->_plugin_activation_migration_current_site();
  }

  public function _plugin_activation_hook_network() {
    $base_blog_id = wpep_wpdb()->blogid;
    $blog_ids = wpep_wpdb()->get_col( 'SELECT blog_id FROM ' . wpep_wpdb()->blogs );

    foreach ( $blog_ids as $blog_id ) {
      switch_to_blog($blog_id);

      $this->_plugin_activation_migration_current_site();
    }

    switch_to_blog( $base_blog_id );

    return NULL;
  }

  private function _plugin_activation_migration_current_site() {
    if( !$this->handle_migration_in_background() ) {
      if( method_exists( $this,'_after_activation' ) )
        $this->_after_activation();

      return;
    }

    $table_structure = $this->get_table_structure();

    if( !empty( $table_structure ) ) {
      foreach( $table_structure as $table_name => $table_query )
        wpep_dbDelta( $table_query );

      $this->current_version = $this->current_version_available;

      update_option( $this->option_name, $this->current_version_available );
    }

    if( method_exists( $this,'_after_background_migration' ) )
      $this->_after_background_migration();

    if( method_exists( $this,'_after_activation' ) )
      $this->_after_activation();

    wpep_set_setting( 'configuration-setup-rules-flushed', '0' );
  }

  public function wpmu_add_new_blog( $blog_id ) {
    if ( !is_plugin_active_for_network( WPEP_PLUGIN_IDENTIFIER ) )
      return;

    switch_to_blog( $blog_id );

    $this->_setup_variables();
    restore_current_blog();
  }

  public function is_valid_table_structure() {
    $table_structure = $this->get_table_structure();

    if( empty( $table_structure ) )
      return true;

    foreach( $table_structure as $table_name => $table_query ) {
      $sql = wpep_wpdb()->prepare( "SHOW TABLES LIKE %s", wpep_wpdb()->esc_like( $table_name ) );

      if( wpep_wpdb()->get_var( $sql ) === null )
        return false;
    }

    return true;
  }

  public function is_valid_data_storage() {
    $table_data_verification_map = $this->get_table_data_verification_map();

    if( empty( $table_data_verification_map ) )
      return true;

    foreach( $table_data_verification_map as $table => $table_information ) {
      if( $table_information[ 'type' ] !== 'unique_data_column' )
        continue;

      $response = wpep_wpdb()->get_results( wpep_sql_query_data_verification_map( $table, $table_information ) );

      if( !empty( $response ) )
        return false;
    }

    return true;
  }

  public function fix_table_structure() {
    $table_structure = $this->get_table_structure();

    if( empty( $table_structure ) )
      return true;

    foreach( $table_structure as $table_name => $table_query ) {
      $sql = wpep_wpdb()->prepare( "SHOW TABLES LIKE %s", wpep_wpdb()->esc_like( $table_name ) );

      if( wpep_wpdb()->get_var( $sql ) != null )
        continue;

      wpep_dbDelta( $table_query );

      $sql = wpep_wpdb()->prepare( "SHOW TABLES LIKE %s", wpep_wpdb()->esc_like( $table_name ) );

      if( wpep_wpdb()->get_var( $sql ) === null )
        return false;
    }

    return true;
  }

  public function fix_data_storage() {
    $table_data_verification_map = $this->get_table_data_verification_map();

    if( empty( $table_data_verification_map ) )
      return true;

    foreach( $table_data_verification_map as $table => $table_information ) {
      if( $table_information[ 'type' ] !== 'unique_data_column' )
        continue;

      $data_verification_map_results = wpep_wpdb()->get_results( wpep_sql_query_data_verification_map( $table, $table_information ), ARRAY_A );

      if( empty( $data_verification_map_results ) )
        continue;

      $to_delete_primary_column_values = [];

      foreach( $data_verification_map_results as $data_verification_map_result ) {
        $primary_column_values = explode( ',', $data_verification_map_result[ 'entry_primary_column_values' ] );

        sort( $primary_column_values, SORT_NUMERIC );

        $primary_column_values = array_reverse( $primary_column_values );

        array_shift( $primary_column_values );

        $to_delete_primary_column_values = array_merge( $primary_column_values, $to_delete_primary_column_values );
      }

      if( empty( $to_delete_primary_column_values ) )
        continue;

      wpep_wpdb()->query( 'DELETE FROM `' . $table . '` WHERE ' . $table_information[ 'primary_column' ] . ' IN (' . implode( ',', $to_delete_primary_column_values ) . ')' );
    }

    return true;
  }

  public function get_table_structure() {
    return [];
  }

  public function get_table_list() {
    return [];
  }

  public function get_table_data_verification_map() {
    return [];
  }

  public function wpmu_drop_tables( $response ) {
    $table_list = $this->get_table_list();

    if( empty( $table_list ) )
      return $response;

    foreach( $table_list as $table_from_list )
      $response[] = $table_from_list;

    return $response;
  }

  public function handle_migration_in_background() {
    if( $this->current_version == '0.0.0' )
      return get_option( $this->option_name, false ) === false;

    return false;
  }

}