<?php

namespace WPEP\ContentType\Assessment;

use WPEP\Container\FormFields as WPEP_FormFields_Configuration;
use WPEP\Entity\Assessment as Entity_Assessment;

class Admin {

  private $_assessment_question_select_options_map = [];

  use WPEP_FormFields_Configuration;

  /**
   * @var Admin\Settings
   */
  public $settings;

  public function __construct() {
    $this->settings = new Admin\Settings();

    if( !is_admin() )
      return;

    $this->set_field_prefix( 'wpep_assessment' );

    add_filter('manage_posts_columns', [ $this, '_table_set_columns' ] );
    add_filter('manage_posts_custom_column', [ $this, '_table_custom_columns' ] );
  }

  public function get_value( $field_key, $field_default = '', $single = true ) {
    global $post;

    $field_value = get_post_meta( $post->ID, $field_key, $single );

    return $field_value == '' ? $field_default : $field_value;
  }

  public function course_builder_tab_content() {
    $this->display_assessment_for_course( 0, get_the_ID() );
  }

  public function display_assessment_for_course( $section_id = 0, $course_id = null ) {
    $course_id         = ( $course_id === null ? get_the_ID() : $course_id );
    $assessment_id     = ( is_numeric( $section_id ) ? wpep_get_course_assessment_id( $course_id, $section_id ) : 0 );
    $field_prefix      = 'wpep_assessment[' . $section_id . ']';

    $this->display_assessment_setup_functionality( $assessment_id, $field_prefix, false );
  }

  public function _handle_request_delete_questions_or_answers() {
    if( isset( $_POST['wpep_deleted_question'] ) ) {
      foreach( $_POST['wpep_deleted_question'] as $question_id ) {
        if( !is_numeric( $question_id ) )
          continue;

        Entity_Assessment::instance()->delete_question_by_id( $question_id );
      }
    }

    if( isset( $_POST['wpep_deleted_question_answer'] ) ) {
      foreach( $_POST['wpep_deleted_question_answer'] as $question_answer_id ) {
        if( !is_numeric( $question_answer_id ) )
          continue;

        Entity_Assessment::instance()->delete_answer_by_id( $question_answer_id );
      }
    }
  }

  public function save_assessment_questions( $assessment_questions, $assessment_id, $entity_id_map = [] ) {
    if( !is_array( $assessment_questions ) )
      return $entity_id_map;

    $question_position = 1;

    foreach( $assessment_questions as $question_id => $question_information ) {
      if( !isset( $question_information['title'] ) )
        continue;

      if( !is_numeric( $question_id ) && isset( $entity_id_map['question'][$question_id] ) )
        $question_id = $entity_id_map['question'][$question_id];

      $current_information = [
        'post_id'     => $assessment_id,
        'title'       => stripcslashes(
          $question_information['title'] !== ''
            ? $question_information['title']
            : __( "New Question", "wpep" )
        ),
        'type'        => ( isset( $question_information['type'] ) ? $question_information['type'] : wpep_get_assessment_field_type_default() ),
        'order'       => ( isset( $question_information['order'] ) ? intval( $question_information['order'] ) : $question_position ),
        'weight'      => ( isset( $question_information['weight'] ) ? intval( $question_information['weight'] ) : 1 ),
      ];

      if( isset( $question_information['content'] ) )
        $current_information['content'] = stripcslashes( $question_information['content'] );

      if( is_numeric( $question_id ) ) {

        Entity_Assessment::instance()->update_question( $current_information, [ 'id' => $question_id ] );

        $entity_id_map['question'][$question_id] = $question_id;

      } else {
        $insert_id = Entity_Assessment::instance()->insert_question( $current_information );

        $entity_id_map['question'][$question_id] = $insert_id;

        $question_id = $insert_id;
      }

      $question_position++;

      if( !isset( $question_information['answer'] ) )
        continue;

      $current_answer_position = 0;

      foreach( $question_information['answer'] as $question_answer_id => $question_answer_information ) {
        if( !isset( $question_answer_information['title'] ) )
          continue;

        $question_answer_information['title'] = stripcslashes( $question_answer_information['title'] );

        if( $question_answer_information['title'] === '' )
          continue;

        if( !is_numeric( $question_id ) && isset( $entity_id_map['question_answer'][$question_id] ) )
          $question_answer_id = $entity_id_map['question_answer'][$question_answer_id];

        $current_information = [
          'post_id'     => $assessment_id,
          'question_id' => $question_id,
          'title'       => $question_answer_information['title'],
          'is_correct'  => ( isset( $question_answer_information['is_correct'] ) ? $question_answer_information['is_correct'] : 0 ),
          'order'       => $current_answer_position
        ];

        if( isset( $question_answer_information['content'] ) )
          $current_information['content'] = stripcslashes($question_answer_information['content']);

        if( is_numeric( $question_answer_id ) ) {
          Entity_Assessment::instance()->update_question_answer( $current_information, [ "id" => $question_answer_id ] );

          $entity_id_map['question_answer'][$question_answer_id] = $question_answer_id;
        } else {
          $entity_id_map['question_answer'][$question_answer_id] = Entity_Assessment::instance()->insert_question_answer( $current_information );
        }

        $current_answer_position++;
      }
    }

    return $entity_id_map;
  }

  public function display_assessment_setup_functionality(
    $assessment_id,
    $field_prefix = 'wpep_assessment',
    $on_post_page = false,
    $text_editor_preload = null
  ) {
    if( $text_editor_preload === null )
      $text_editor_preload = wpep_get_setting_flag( 'lesson-wp-editor-preload' );

    $meta_field_prefix = $field_prefix . '[meta_information]';

    $unique_assessment_identifier = md5( $meta_field_prefix );

    echo    '<h3>' . __( "Assessment Settings", "wpep" ) . '</h3>';
    echo    '<div class="wpep-form-group wpep-form-group-assessment-general-information">';
    if( $on_post_page == false ) {
      echo      '<div class="wpep-form-group-row">';
      echo        '<label>' . __( "Status", "wpep" ) . '</label>';
      $this->display_select_field(
        'post_status',
        [
          'draft'   => __( 'Draft - Inactive', 'wpep' ),
          'private' => __( 'Private - Admin Available', "wpep" ),
          'publish' => __( 'Active - Available for all students', "wpep" )
        ],
        $field_prefix . '[post_status]',
        ( $assessment_id == 0 ? null : get_post_status( $assessment_id ) )
      );
      echo      '</div>';
      echo      '<div class="wpep-form-group-row">';
      echo        '<label>' . __( "Title", "wpep" ) . '</label>';
      $this->display_text_field(
        'title',
        $field_prefix . '[title]',
        ( $assessment_id == 0 ? '' : get_the_title( $assessment_id ) )
      );
      echo        '<p class="description">' . __( "Leaving the Title empty will result in this assessment not being saved.", "wpep" ) . '</p>';
      echo      '</div>';
    }

    echo      '<div class="wpep-form-group-row">';
    echo        '<label>' . __( "Type", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_assessment_type',
      [
        WPEP_ASSESSMENT_TYPE_STANDARD       => __( "Standard", "wpep" ),
        WPEP_ASSESSMENT_TYPE_ASSIGNMENT     => __( "Assignment", "wpep" ),
        WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING => __( "Simple Scoring - only scalar question types are available", "wpep" )
      ],
      $meta_field_prefix . '[wpep_assessment_type]',
      get_post_meta( $assessment_id, 'wpep_assessment_type', true ),
      'wpep-select-field wpep-select-field-assessment-type',
      WPEP_ASSESSMENT_TYPE_STANDARD,
      [
        'data-wpep-builder-dynamic-conditional-display-prefix' => 'data-wpep-assessment-type-',
        'data-wpep-conditional-display'                        => 'data-wpep-assessment-type-' . $unique_assessment_identifier
      ]
    );
    echo      '</div>';

    echo      '<div class="wpep-form-group-row" data-wpep-assessment-type-' . $unique_assessment_identifier. '="' . WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING . '">';
    echo        '<label>' . __( "Allowed Minutes", "wpep" ) . '</label>';
    $this->display_text_field(
      'wpep_available_minutes',
      $meta_field_prefix . '[wpep_available_minutes]',
      intval( get_post_meta( $assessment_id, 'wpep_available_minutes', true ) )
    );
    echo        '<p class="description">' . __( "The number of minutes the student may take to complete this assessment, leave 0 for no time restriction.", "wpep" ) . '</p>';
    echo      '</div>';

    //// Start Submission Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-select-combo" data-wpep-assessment-type-' . $unique_assessment_identifier. '="' . WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING . '">';
    echo        '<label>' . __( "Submission", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_submission_restraint',
      [
        2   => __( 'Students can always change their answers and may take the Test any number of times.', "wpep" ),
        1   => [
          'label'                                                 => __( 'Students can change their answers until the Submission is being reviewed by the Teacher.', 'wpep' ),
          'wpep-assessment-type-' . $unique_assessment_identifier => WPEP_ASSESSMENT_TYPE_STANDARD
        ],
        0   => __( 'Students can Submit answers only once.', 'wpep' ),
      ],
      $meta_field_prefix . '[wpep_submission_restraint]',
      intval( get_post_meta( $assessment_id, 'wpep_submission_restraint', true ) )
    );
    $this->display_select_field(
      'wpep_paginate_questions',
      [
        0   => __( 'Display all Questions on one Page', 'wpep' ),
        1   => sprintf( __( 'Display %s question at a time.', 'wpep' ), "1" ),
        2   => sprintf( __( 'Display %s question at a time.', 'wpep' ), "2" ),
        3   => sprintf( __( 'Display %s question at a time.', 'wpep' ), "3" ),
      ],
      $meta_field_prefix . '[wpep_paginate_questions]',
      intval( get_post_meta( $assessment_id, 'wpep_paginate_questions', true ) )
    );
    echo      '</div>';
    //// End Submission Settings

    $display_welcome_page = get_post_meta( $assessment_id, 'wpep_enable_welcome_page', true );

    //// Start Submission Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-select-combo" data-wpep-assessment-type-' . $unique_assessment_identifier. '="' . WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING . '">';
    echo        '<label>' . __( "Interaction", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_enable_welcome_page',
      [
        1   => __( 'Display the Welcome Page at the beginning of the Testing Journey.', 'wpep' ),
        0   => __( "Do not Display the Welcome Page at the beginning of the Testing Journey.", 'wpep' ),
      ],
      $meta_field_prefix . '[wpep_enable_welcome_page]',
      ( $display_welcome_page === null ? 1 : $display_welcome_page )
    );
    $this->display_select_field(
      'wpep_enable_view_questions',
      [
        0   => [
          'label'                                                 => __( "Disable the Questions Summary View at the end of the Testing Journey.", 'wpep' ),
          'wpep-assessment-type-' . $unique_assessment_identifier => WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING
        ],
        1   => [
          'label'                                                 => __( 'Enable the Questions Summary View at the end of the Testing Journey.', 'wpep' ),
          'wpep-assessment-type-' . $unique_assessment_identifier => WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_ASSIGNMENT
        ]
      ],
      $meta_field_prefix . '[wpep_enable_view_questions]',
      intval( get_post_meta( $assessment_id, 'wpep_enable_view_questions', true ) )
    );
    echo      '</div>';
    //// End Submission Settings

    $grading_option = get_post_meta( $assessment_id, 'wpep_grading_option', true );

    //// Start Submission Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-select-combo" data-wpep-assessment-type-' . $unique_assessment_identifier . '="' . WPEP_ASSESSMENT_TYPE_STANDARD . '">';
    echo        '<label>' . __( "Grading", "wpep" ) . '</label>';
    $this->display_select_field(
      'wpep_grading_option',
      [
        WPEP_ASSESSMENT_GRADING_DISABLED       => __( 'Disabled', 'wpep' ),
        WPEP_ASSESSMENT_GRADING_MANUAL         => __( 'Manual', "wpep" ),
        WPEP_ASSESSMENT_GRADING_MANUAL_PARTIAL => __( 'Semi Automatic, requires manual grading by the Teacher. (Question Type: Text, Essay, File Upload)', 'wpep' ),
        WPEP_ASSESSMENT_GRADING_AUTO           => __( 'Automatic, all answers that cannot be graded are always correct.', "wpep" )
      ],
      $meta_field_prefix . '[wpep_grading_option]',
      ( $grading_option === '' || $grading_option === false ? 3 : $grading_option )
    );

    $grading_requirement = get_post_meta( $assessment_id, 'wpep_grading_requirement', true );

    $this->display_select_field(
      'wpep_grading_requirement',
      wpep_get_grading_requirement_options(),
      $meta_field_prefix . '[wpep_grading_requirement]',
      ( $grading_requirement === '' ? 70 : intval( $grading_requirement ) )
    );
    echo      '</div>';
    //// End Submission Settings

    //// Start Shuffle Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-checkbox-combo wpep-form-assessment-shuffle-options" data-wpep-assessment-type-' . $unique_assessment_identifier. '="' . WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING . '">';
    echo        '<span>' . __( "Shuffle", "wpep" ) . '</span>';
    echo        '<label class="shuffle-questions-container">';
    $this->display_checkbox_field(
      'shuffle_questions',
      get_post_meta( $assessment_id, 'wpep_shuffle_questions', true ),
      $meta_field_prefix . '[wpep_shuffle_questions]',
      0
    );
    echo        __( "Questions", "wpep" ) . '</label>';

    echo        '<label>';
    $this->display_checkbox_field(
      'shuffle_answers',
      get_post_meta( $assessment_id, 'wpep_shuffle_answers', true ),
      $meta_field_prefix . '[wpep_shuffle_answers]',
      0
    );
    echo        __( "Answers", "wpep" ) . '</label>';
    echo    '</div>';
    //// End Shuffle Settings

    //// Start Allow Answers Save Settings
    echo      '<div class="wpep-form-group-row wpep-form-group-row-checkbox-combo wpep-form-assessment-saving-options" data-wpep-assessment-type-' . $unique_assessment_identifier. '="' . WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING . '">';
    echo        '<span>' . __( "Misc", "wpep" ) . '</span>';
    echo        '<label class="shuffle-questions-container">';
    $this->display_checkbox_field(
      'allow_answers_save',
      get_post_meta( $assessment_id, 'disable_answers_save', true ),
      $meta_field_prefix . '[disable_answers_save]',
      0
    );
    echo        __( "Disable Answers Save", "wpep" ) . '</label>';
    echo        '<label class="shuffle-questions-container">';
    $this->display_checkbox_field(
      'disable_score_display',
      get_post_meta( $assessment_id, 'disable_score_display', true ),
      $meta_field_prefix . '[disable_score_display]',
      0
    );
    echo        __( "Disable Score Display", "wpep" ) . '</label>';
    echo      '</div>';
    //// End Allow Answers Save Settings

    echo    '</div>';

    echo    '<h3>' . __( "Assessment Description", "wpep" ) . '</h3>';

    echo    '<div class="wpep-form-group textarea primary-content-editor" data-wpep-editor-field-suffix="[wpep_assessment_description]" data-assessment-id="' . $assessment_id . '">';
    if( $text_editor_preload || $on_post_page ) {
      wpep_editor(
        get_post_meta( $assessment_id, 'wpep_assessment_description', true ),
        'ad-' . $unique_assessment_identifier,
        $meta_field_prefix . '[wpep_assessment_description]'
      );

      echo '<p class="description">' . __( "Displayed under the Title while taking the Assessment.", "wpep" ) . '</p>';
    }
    echo     '</div>';

    do_action( "wpep_administration_assessment_meta_options_block", $meta_field_prefix, $assessment_id );

    echo   '<div class="wpep-toggle-container" data-wpep-assessment-type-' . $unique_assessment_identifier. '="' . WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING . '">';
    echo   '<h3>' . vsprintf( __( "Assessment %s / %s Message", "wpep" ), [
        '<span data-wpep-toggle="success" class="wpep-assessment-message-view-success">' . __( "Success", "wpep" ) . '</span>',
        '<span data-wpep-toggle="failed" class="wpep-assessment-message-view-failed">' . __( "Failed", "wpep" ) . '</span>'
      ] ) . '</h3>';

    echo        '<div class="wpep-form-group" data-wpep-toggled="success" data-assessment-id="' . $assessment_id . '" data-assessment-type="success">';
    echo        '<label data-wpep-assessment-type-' . $unique_assessment_identifier . '="' . WPEP_ASSESSMENT_TYPE_STANDARD . '">';
    $this->display_checkbox_field(
      'wpep_success_message_disable_indicators',
      get_post_meta( $assessment_id, 'wpep_success_message_disable_indicators', true ),
      $meta_field_prefix . '[wpep_success_message_disable_indicators]',
      0
    );
    echo        __( "Hide Visual Success Indicators: Test Result Score & number of correctly answered questions from the Assessment Success Page.", "wpep" ) . '</label>';

    echo        '<label>';
    $this->display_checkbox_field(
      'wpep_success_message_hide_title',
      get_post_meta( $assessment_id, 'wpep_success_message_hide_title', true ),
      $meta_field_prefix . '[wpep_success_message_hide_title]',
      0
    );
    echo        __( "Hide Assessment Title from the Assessment Success Page.", "wpep" ) . '</label>';
    echo        '</div>';

    echo        '<div class="wpep-form-group textarea primary-content-editor" data-wpep-toggled="success" data-assessment-id="' . $assessment_id . '" data-assessment-type="success" data-wpep-editor-field-suffix="[wpep_success_message]">';
    if( $text_editor_preload || $on_post_page ) {
      $assessment_content = get_post_meta( $assessment_id, 'wpep_success_message', true );

      if( $assessment_content == '' )
        $assessment_content = wpep_get_setting_translated( 'assessment-success-message' );

      wpep_editor(
        $assessment_content,
        'asm-' . $unique_assessment_identifier,
        $meta_field_prefix . '[wpep_success_message]'
      );
    }
    echo        '</div>';

    echo        '<div class="wpep-form-group" data-wpep-toggled="failed" data-assessment-id="' . $assessment_id . '" data-assessment-type="failed">';
    echo        '<label data-wpep-assessment-type-' . $unique_assessment_identifier . '="' . WPEP_ASSESSMENT_TYPE_STANDARD . '">';
    $this->display_checkbox_field(
      'wpep_failed_message_disable_indicators',
      get_post_meta( $assessment_id, 'wpep_failed_message_disable_indicators', true ),
      $meta_field_prefix . '[wpep_failed_message_disable_indicators]',
      0
    );
    echo        __( "Hide Visual Failure Indicators: Test Result Score & number of correctly answered questions from the Assessment Failure Page.", "wpep" ) . '</label>';
    echo        '<label>';
    $this->display_checkbox_field(
      'wpep_failed_message_hide_title',
      get_post_meta( $assessment_id, 'wpep_failed_message_hide_title', true ),
      $meta_field_prefix . '[wpep_failed_message_hide_title]',
      0
    );
    echo        __( "Hide Assessment Title from the Assessment Failure Page.", "wpep" ) . '</label>';
    echo        '</div>';

    echo        '<div class="wpep-form-group textarea primary-content-editor" data-wpep-toggled="failed" data-assessment-id="' . $assessment_id . '" data-assessment-type="failed" data-wpep-editor-field-suffix="[wpep_failed_message]">';
    if( $text_editor_preload || $on_post_page ) {
      $assessment_content = get_post_meta( $assessment_id, 'wpep_failed_message', true );

      if( $assessment_content == '' )
        $assessment_content = wpep_get_setting_translated( 'assessment-status-failed' );

      wpep_editor(
        $assessment_content,
        'afm-' . $unique_assessment_identifier,
        $meta_field_prefix . '[wpep_failed_message]'
      );
    }
    echo        '</div>';
    echo   '</div>';

    echo   '<h3>' . __( "Assessment Questions", "wpep" ) . '</h3>';

    echo    '<div data-wpep-builder-entity-container="question"
                  data-wpep-builder-entity-deleted-input="wpep_deleted_question">';

    $assessment_questions = ( is_numeric( $assessment_id ) && $assessment_id != 0 ? Entity_Assessment::instance()->get_questions_by_assessment_id( $assessment_id ) : [] );

    if( !empty( $assessment_questions ) ) {
      foreach( $assessment_questions as $question_id => $question ) {
        $this->_display_assessment_question( $field_prefix, $question, "visible", null, $unique_assessment_identifier );
      }
    } else {
      $this->_display_assessment_question(
        $field_prefix,
        null,
        "visible",
        null,
        $unique_assessment_identifier
      );
    }

    $this->_display_assessment_question(
      $field_prefix,
      null,
      "template",
      false,
      $unique_assessment_identifier
    );

    echo      '<div data-wpep-builder-entity-triggers-container="footer">';
    echo        '<span data-wpep-builder-entity-trigger="add_entity" class="button button-primary">' . __( "Add Question", "wpep" ) . '</span>';
    echo      '</div>';
    echo      '<div class="clear"></div>';
    echo    '</div>';
  }

  /**
   * @param $field_prefix
   * @param null $question
   * @param string $status
   * @param null $editor_preload
   * @param string|null $unique_assessment_identifier
   */
  private function _display_assessment_question( $field_prefix, $question = null, $status = 'visible', $editor_preload = null, $unique_assessment_identifier = null ) {
    $editor_preload = ( $editor_preload === null ? wpep_get_setting_flag( 'lesson-wp-editor-preload' ) : $editor_preload );

    global $wp_version;

    $current_question_id     = ( isset( $question->id ) ? $question->id : uniqid('question_') );
    $current_question_prefix = $field_prefix . '[question]' . '[' . $current_question_id . ']';
    $current_question_type   = ( isset( $question->type ) ? $question->type : wpep_get_assessment_field_type_default() );

    echo '<fieldset data-wpep-builder-entity="question" 
                    data-wpep-builder-entity-status="' . $status . '" 
                    data-wpep-builder-entity-id="' . $current_question_id . '"
                    data-field-group-prefix="' . $current_question_prefix . '">';
    echo    '<div class="header-section header-section-dual-field" data-wpep-builder-entity-triggers="2">';
    echo      '<span class="element-move ' . ( version_compare( $wp_version, '4.5', '<') ? 'fallback' : '' ) . '"></span>';
    $this->display_text_field(
      'title',
      $current_question_prefix . '[title]',
      ( isset( $question->title ) ? $question->title : '' ),
      "wpep-question-title",
      __( "New Question", "wpep" )
    );

    echo      '<span data-wpep-builder-entity-trigger="delete_entity" data-wpep-builder-entity-trigger-icon="trash"></span>';
    echo      '<span data-wpep-builder-entity-settings-container-trigger="question_content" data-wpep-builder-entity-trigger-icon="eye"></span>';

    $this->display_select_field(
      'type',
      $this->_get_assessment_question_select_options( $unique_assessment_identifier ),
      $current_question_prefix . '[type]',
      $current_question_type,
      'wpep-select-field',
      '',
      [
        'data-wpep-builder-entity-conditional-child-entities' => 'data-has-answers'
      ]
    );

    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo    '<div class="content-section">';
    echo      '<div class="wpep-form-group" data-wpep-builder-entity-settings-container="question_content">';

    echo      '<div class="wpep-form-group-row">';
    echo        '<label>' . __( "Weight", "wpep" ) . '</label>';
    $this->display_select_field(
      'question-weight',
      wpep_get_indexed_number_list(0, 15, 'This question is worth %value% points.'),
      $current_question_prefix . '[weight]',
      isset( $question->weight ) ? $question->weight : 1,
      'wpep-select-field wpep-select-field-weight',
      '',
      [
        'data-default-select-value' => '1'
      ]
    );
    echo      '</div>';

    echo        '<div class="wpep-form-group textarea primary-content-editor">';

    if( $editor_preload ) {
      wpep_editor(
        html_entity_decode( isset( $question->content ) ? $question->content : '' ),
        'question-' . $current_question_id,
        $current_question_prefix . '[content]'
      );
    }

    echo        '</div>';
    echo      '</div>';
    echo      '<div data-wpep-builder-entity-container="question_answer" 
                    data-field-group-prefix="' . $current_question_prefix . '[answer]"
                    data-wpep-builder-entity-deleted-input="wpep_deleted_question_answer"
                    data-wpep-builder-entity-parent="question"
                    data-wpep-builder-entity-container-expanded="' . ( wpep_get_assessment_field_types()[$current_question_type]['has_answers'] ? '1' : '0' ) . '">';

    if( isset( $question->answers ) && !empty( $question->answers ) )
      foreach( $question->answers as $question_answer )
        $this->_display_assessment_question_answer( $current_question_prefix, $question_answer );

    $this->_display_assessment_question_answer( $current_question_prefix, null, "template" );

    echo      '<div data-wpep-builder-entity-footer="question_answer">';
    echo        '<span data-wpep-builder-entity-trigger="add_entity" data-wpep-builder-entity-trigger-icon="plus"></span>';
    echo      '</div>';
    echo      '<div class="clear"></div>';
    echo      '</div>';
    echo    '</div>';
    echo '</fieldset>';
  }

  private function _display_assessment_question_answer(
    $current_question_prefix,
    $question_answer = null,
    $status = "visible"
  ) {
    global $wp_version;

    $current_answer_id     = ( isset( $question_answer->id ) ? $question_answer->id : uniqid('answer_') );
    $current_answer_prefix = $current_question_prefix . '[answer][' . $current_answer_id . ']';
    $is_correct = ( isset( $question_answer->is_correct ) ? $question_answer->is_correct : ( $status === 'visible' ? 1 : 0 ) );

    echo '<fieldset data-wpep-builder-entity="question_answer" 
                    data-wpep-builder-entity-status="' . $status . '" 
                    data-wpep-builder-entity-id="' . $current_answer_id . '"
                    data-field-group-prefix="' . $current_answer_prefix . '">';
    echo    '<div class="header-section" data-wpep-builder-entity-triggers="2">';

    $this->display_hidden_field(
      'is_correct',
      $is_correct,
      $current_answer_prefix . '[is_correct]',
      'wpep-assessment-question-answer-is-correct'
    );

    echo      '<span class="element-move ' . ( version_compare( $wp_version, '4.5', '<') ? 'fallback' : '' ) . '"></span>';
    echo      '<span data-wpep-builder-entity-trigger="flag" data-wpep-builder-entity-trigger-input=".wpep-assessment-question-answer-is-correct" data-wpep-builder-entity-trigger-status="' . $is_correct . '"></span>';
    $this->display_text_field(
      'title',
      $current_answer_prefix . '[title]',
      ( isset( $question_answer->title ) ? $question_answer->title : '' ),
      "wpep-question-answer-title",
      __( "New Answer", "wpep" )
    );
    echo      '<span data-wpep-builder-entity-trigger="delete_entity" data-wpep-builder-entity-trigger-icon="trash"></span>';
    echo      '<div class="wpep-clearfix"></div>';
    echo    '</div>';
    echo '</fieldset>';
  }

  private function _get_assessment_question_select_options( $unique_assessment_identifier ) {
    $unique_assessment_identifier = ( $unique_assessment_identifier === null || $unique_assessment_identifier === false ? '' : $unique_assessment_identifier );

    if( !empty( $this->_assessment_question_select_options_map[ $unique_assessment_identifier ] ) )
      return $this->_assessment_question_select_options_map[ $unique_assessment_identifier ];

    $this->_assessment_question_select_options_map[$unique_assessment_identifier] = [];

    foreach( wpep_get_assessment_field_types() as $assessment_field_type_key => $assessment_field_type_information ) {
      $current_information = [
        'label'             => $assessment_field_type_information['label'],
        'has_answers'       => $assessment_field_type_information['has_answers'],
        'multiple_answers'  => $assessment_field_type_information['multiple_answers'],
        'is_score_able'     => $assessment_field_type_information['is_score_able']
      ];


      if( $unique_assessment_identifier !== '' ) {
        $current_information[ 'wpep-assessment-type-' . $unique_assessment_identifier ] = (
        isset( $assessment_field_type_information[ 'is_score_able' ] ) && $assessment_field_type_information[ 'is_score_able' ]
          ? WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_ASSIGNMENT . ' ' . WPEP_ASSESSMENT_TYPE_SIMPLE_SCORING
          : WPEP_ASSESSMENT_TYPE_STANDARD . ' ' . WPEP_ASSESSMENT_TYPE_ASSIGNMENT
        );
      }

      $this->_assessment_question_select_options_map[ $unique_assessment_identifier ][ $assessment_field_type_key ] = $current_information;
    }

    return $this->_assessment_question_select_options_map[ $unique_assessment_identifier ];
  }

  public function _table_set_columns($columns) {
    global $post_type;

    if ( $post_type != WPEP_POST_TYPE_ASSESSMENT)
      return $columns;

    unset(
      $columns['date']
    );

    $new_columns = [
      'course'    => __('Course', "wpep"),
      'shortcode' => __('Shortcode', "wpep"),
    ];

    $new_columns['date'] = __('Date', "wpep");

    return array_merge($columns, $new_columns);
  }

  public function _table_custom_columns($column) {
    global $post;

    if( $post->post_type != WPEP_POST_TYPE_ASSESSMENT )
      return;

    if( $column == 'course' ) {
      $course_id = wpep_get_assessment_course_id( $post->ID );

      if( $course_id !== 0 )
        echo '<a href="' . wpep_post_admin_edit_link( $course_id ) . '">' . get_the_title( $course_id ) . '</a>';
      else
        echo '-';
    }

    if ( $column == 'shortcode' ) {
      echo wpep_admin_shortcode_builder(
        WPEP_SHORT_CODE_PREFIX . 'assessment',
        [
          'id'  => $post->ID,
          'wpep_admin_shortcode_builder_wpep_assessment_options'
        ],
        __( "ShortCode", "wpep" ),
        true
      );

      return;
    }
  }

}