<?php

use WPEP\Entity\Course as EntityCourse;

/**
 * @return \WPEP\ContentType\Assessment
 */
function wpep_content_type_assessment_instance() {
  return \WPEP\ContentType::instance()->contentTypes[ WPEP_POST_TYPE_ASSESSMENT ];
}

/**
 * @param $assessment_id
 * @param int $user_id
 * @return string
 */
function wpep_user_assessment_status( $assessment_id, $user_id = 0 ) {
  return wpep_content_type_assessment_instance()->user->get_status( $assessment_id, $user_id );
}

function wpep_user_assessment_preload_for_course_id( $course_id, $user_id = 0 ) {
  wpep_content_type_assessment_instance()->delivery->course->preload_by_user_id( $course_id, $user_id );
}

/**
 * @param $course_id
 * @param int $section_id
 * @param int $user_id
 * @return int
 */
function wpep_user_assessment_id_by_course_id( $course_id, $section_id = 0, $user_id = 0 ) {
  return wpep_content_type_assessment_instance()->delivery->course->get_assessment_id_by_user_id( $course_id, $section_id, $user_id );
}

/**
 * @param $lesson_id
 * @param bool $user_id
 * @param array $assessment_user_status
 * @return array
 */
function wpep_user_assessment_requirements_for_lesson( $lesson_id, $user_id = false, $assessment_user_status = [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_FAILED ] ) {
  $user_id        = ( $user_id == false ? get_current_user_id() : $user_id );
  $lesson_section = EntityCourse::instance()->get_section_by_lesson_id( $lesson_id );

  if( $lesson_section === null )
    return [];

  return wpep_user_assessment_requirements_for_section( $lesson_section, $user_id, $assessment_user_status );
}

function wpep_user_assessment_requirements_for_section( $lesson_section, $user_id = false, $assessment_user_status = [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_FAILED ] ) {
  $user_id        = ( $user_id == false ? get_current_user_id() : $user_id );

  if( is_numeric( $lesson_section ) )
    $lesson_section = EntityCourse::instance()->get_section( $lesson_section );

  if( !isset( $lesson_section->post_id ) )
    return [];

  // The User is not required to finish any assessments prior to getting access to this Lesson.
  if( !wpep_course_assessments_required( $lesson_section->post_id ) )
    return [];

  $response = [];

  $section_list = EntityCourse::instance()->get_sections_by_course_id( $lesson_section->post_id );

  foreach( $section_list as $section ) {
    if( $section->order >= $lesson_section->order )
      break;

    $assessment_id = wpep_user_assessment_id_by_course_id( $lesson_section->post_id, $section->id, $user_id );

    if( $assessment_id == 0 )
      continue;

    if( in_array( wpep_user_assessment_status( $assessment_id, $user_id ), $assessment_user_status ) )
      $response[] = $assessment_id;
  }

  return $response;
}

/**
 * @param $course_id
 * @param bool $user_id
 * @param array $assessment_user_status
 * @return array
 */
function wpep_get_assessment_requirements_for_course( $course_id, $user_id = false, $assessment_user_status = [ WPEP_USER_ASSESSMENT_STATUS_PENDING, WPEP_USER_ASSESSMENT_STATUS_FAILED ] ) {
  $user_id        = ( $user_id == false ? get_current_user_id() : $user_id );

  $response = [];

  $section_list = EntityCourse::instance()->get_sections_by_course_id( $course_id );

  foreach( $section_list as $section ) {
    $assessment_id = wpep_user_assessment_id_by_course_id( $course_id, $section->id, $user_id );

    if( $assessment_id == 0 )
      continue;

    if( in_array( wpep_user_assessment_status( $assessment_id, $user_id ), $assessment_user_status ) )
      $response[] = $assessment_id;
  }

  // Final Assessment
  $assessment_id = wpep_user_assessment_id_by_course_id( $course_id, 0, $user_id );

  if( $assessment_id == 0 )
    return $response;

  if( in_array( wpep_user_assessment_status( $assessment_id, $user_id ), $assessment_user_status ) )
    $response[] = $assessment_id;

  return $response;
}

function wpep_get_assessment_course_id( $assessment_id ) {
  return intval( get_post_meta( $assessment_id, 'wpep_course_id', true ) );
}

function wpep_get_assessment_course_section_id( $assessment_id ) {
  return intval( get_post_meta( $assessment_id, 'wpep_section_id', true ) );
}

function wpep_get_assessment_type( $assessment_id ) {
  $assessment_type = get_post_meta( $assessment_id, 'wpep_assessment_type', true );

  if( $assessment_type === '' )
    $assessment_type = WPEP_ASSESSMENT_TYPE_STANDARD;

  return $assessment_type;
}

/**
 * @return string
 */
function wpep_get_assessment_field_type_default() {
  return 'input_text';
}

function wpep_get_grading_requirement_options() {
  return wpep_get_indexed_number_list(1, 100, __( "required Score to Pass the Test %value%%", "wpep" ) );
}

/**
 * @return array
 */
function wpep_get_assessment_field_types() {
  $assessment_field_types = [
    'input_text'  => [
      'label'             => __( "Free Text", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ],
    'textarea'  => [
      'label'             => __( "Essay", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ],
    'editor'  => [
      'label'             => __( "Quill Editor", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ],
    'radio'  => [
      'label'             => __( "Single Answer", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ],
    'checkbox'  => [
      'label'             => __( "Multiple Answer", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 1,
      'is_score_able'     => 0
    ],
    'select'  => [
      'label'             => __( "Single Select", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ],
    'select_scalar_5'  => [
      'label'              => __( "Scalar Selection 1 to 5", "wpep" ),
      'has_answers'        => 0,
      'multiple_answers'   => 0,
      'predefined_answers' => [
        1 => sprintf( __( "Scalar %s", "wpep" ), 1 ),
        2 => sprintf( __( "Scalar %s", "wpep" ), 2 ),
        3 => sprintf( __( "Scalar %s", "wpep" ), 3 ),
        4 => sprintf( __( "Scalar %s", "wpep" ), 4 ),
        5 => sprintf( __( "Scalar %s", "wpep" ), 5 )
      ],
      'is_score_able'     => 1,
      'min_score'         => 1,
      'max_score'         => 5
    ],
    'select_scalar_10'  => [
      'label'              => __( "Scalar Selection 1 to 10", "wpep" ),
      'has_answers'        => 0,
      'multiple_answers'   => 0,
      'predefined_answers' => [
        1  => sprintf( __( "Scalar %s", "wpep" ), 1 ),
        2  => sprintf( __( "Scalar %s", "wpep" ), 2 ),
        3  => sprintf( __( "Scalar %s", "wpep" ), 3 ),
        4  => sprintf( __( "Scalar %s", "wpep" ), 4 ),
        5  => sprintf( __( "Scalar %s", "wpep" ), 5 ),
        6  => sprintf( __( "Scalar %s", "wpep" ), 6 ),
        7  => sprintf( __( "Scalar %s", "wpep" ), 7 ),
        8  => sprintf( __( "Scalar %s", "wpep" ), 8 ),
        9  => sprintf( __( "Scalar %s", "wpep" ), 9 ),
        10 => sprintf( __( "Scalar %s", "wpep" ), 10 ),
      ],
      'is_score_able'     => 1,
      'min_score'         => 1,
      'max_score'         => 10
    ],
    'input_match' => [
      'label'             => __( "Text Match", "wpep" ),
      'has_answers'       => 1,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ],
    'file' => [
      'label'             => __( "File Upload", "wpep" ),
      'has_answers'       => 0,
      'multiple_answers'  => 0,
      'is_score_able'     => 0
    ]
  ];

  return $assessment_field_types;
}

/**
 * @param $course_id
 * @param array|null $assessment_status
 * @return array
 */
function wpep_get_course_assessment_ids( $course_id, $assessment_status = null ) {
  return wpep_content_type_assessment_instance()->delivery->course->get_assessment_ids( $course_id, $assessment_status );
}

/**
 * @param $course_id
 * @param int $section_id
 * @param array|null $assessment_status
 * @return int|mixed
 */
function wpep_get_course_assessment_id( $course_id, $section_id = 0, $assessment_status = null ) {
  return wpep_content_type_assessment_instance()->delivery->course->get_assessment_id( $course_id, $section_id, $assessment_status );
}

/**
 * @param $assessment_id
 * @param bool $user_status
 * @return bool
 */
function wpep_allow_assessment_submission( $assessment_id, $user_status = false ) {
  if( wpep_get_assessment_type( $assessment_id ) === WPEP_ASSESSMENT_TYPE_ASSIGNMENT )
    return true;

  if( $user_status == false )
    wpep_user_assessment_status( $assessment_id );

  if( $user_status == WPEP_USER_ASSESSMENT_STATUS_PENDING )
    return true;

  if( get_post_meta( $assessment_id, 'wpep_submission_restraint', true ) == 0 )
    return false;

  if( in_array( $user_status, [ WPEP_USER_ASSESSMENT_STATUS_FAILED, WPEP_USER_ASSESSMENT_STATUS_COMPLETED ] )
    && get_post_meta( $assessment_id, 'wpep_submission_restraint', true ) == 1 )
    return false;

  return true;
}