<?php

namespace WPEP\ContentType\Certification;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

class Delivery {

  /**
   * Runs after WPEP has been fully loaded.
   * @return void
   */
  public function init() {
    add_action( 'wpep_user_set_course_data', [ $this, 'wpep_course_completed_action' ], 20, 6 );

    if( wpep_assessments_enabled() )
      add_action( 'wpep_user_set_assessment_data_' . WPEP_USER_ASSESSMENT_STATUS, [ $this, 'wpep_assessment_status_change' ], 20, 5 );

    add_filter( 'template_include', [ $this, 'template_include' ], 0);

    if( wpep_get_setting( 'certification-download-box-place' ) == 'top' ) {
      add_filter( 'wpep_single_content_course_before_html', [ $this, 'before_course_content' ], 20, 2 );
    } else {
      add_filter( 'wpep_single_content_course_after_html', [ $this, 'after_course_content' ], 20, 2 );
    }

    add_action( 'wpep_template_assessment_message_operations_after', [ $this, 'after_assessment_primary_button' ], 20, 2 );

    add_filter( 'wpep_content_user_profile_tab_list', [ $this, 'user_profile_tab_list' ], 20, 2 );
  }

  /**
   * @param $key
   * @param $value
   * @param $course_id
   * @param $section_id
   * @param $lesson_id
   * @param $user_id
   */
  public function wpep_course_completed_action( $key, $value, $course_id, $section_id, $lesson_id, $user_id ) {
    if( $key != WPEP_USER_COURSE_COMPLETED )
      return;

    if( $value != true )
      return;

    if( $user_id == 0 )
      return;

    $course_certification_status = intval( get_post_meta( $course_id, 'wpep_certificate_status', true ) );

    if( $course_certification_status === WPEP_COURSE_CERTIFICATE_STATUS_DISABLED )
      return;

    $award_certificate = wpep_content_type_course_instance()->user->flag->has_completed_course(
      $course_id, $user_id, ( $course_certification_status === WPEP_COURSE_CERTIFICATE_STATUS_ENABLED_WITH_ASSESSMENTS ? true : false )
    );

    if( !$award_certificate )
      return;

    $this->prepare_certificate_by_parent_post_id( $user_id, $course_id );
  }

  public function wpep_assessment_status_change( $value, $post_id, $question_id, $question_answer_id, $user_id ) {
    if( $value != WPEP_USER_ASSESSMENT_STATUS_COMPLETED )
      return;

    if( $user_id == 0 )
      return;

    $course_id = wpep_get_assessment_course_id( $post_id );

    $course_certification_status = intval( get_post_meta( $course_id, 'wpep_certificate_status', true ) );

    if( $course_certification_status !== WPEP_COURSE_CERTIFICATE_STATUS_ENABLED_WITH_ASSESSMENTS )
      return;

    $award_certificate = wpep_content_type_course_instance()->user->flag->has_completed_course(
      $course_id, $user_id, true
    );

    if( !$award_certificate )
      return;

    $this->prepare_certificate_by_parent_post_id( $user_id, $course_id );
  }

  /**
   * @param $user_id
   * @param $parent_post_id
   * @param bool $certificate_id
   * @return bool
   */
  public function prepare_certificate_by_parent_post_id( $user_id, $parent_post_id, $certificate_id = false ) {
    if( $user_id == 0 )
      return false;

    $certificate_id = ( $certificate_id === false ? $this->get_certificate_id_by_parent_post_id( $user_id, $parent_post_id ) : $certificate_id );

    if( $certificate_id === 0 )
      return $this->_generate_user_certificate( $user_id, $parent_post_id, $certificate_id );

    if( !file_exists( get_post_meta( $certificate_id, 'certificate_path', true ) ) )
      return $this->_generate_user_certificate( $user_id, $parent_post_id, $certificate_id );

    $date = get_post_meta( $parent_post_id, 'wpep_certificate_reset_date', true );

    if( $date == '' )
      return true;

    if( WPEP_COURSE_CERTIFICATE_ERROR_BACKGROUND_IMAGE === get_post_meta( $certificate_id, 'certificate_url', true ) )
      return true;

    $timestamp = strtotime( get_post_meta( $parent_post_id, 'wpep_certificate_reset_date', true ) ) + 86400;

    if( get_post_modified_time( "G", false, $certificate_id ) > $timestamp )
      return true;

    unlink( get_post_meta( $certificate_id, 'certificate_path', true ) );

    return $this->_generate_user_certificate( $user_id, $parent_post_id, $certificate_id );
  }

  /**
   * @param $user_id
   * @param $parent_post_id
   * @param int $certificate_id
   * @return bool
   */
  private function _generate_user_certificate( $user_id, $parent_post_id, $certificate_id = 0 ) {
    $image_url      = get_post_meta( $parent_post_id, 'wpep_certificate_image', true );
    $content_parts  = get_post_meta( $parent_post_id, 'wpep_certificate_content_map', false );

    if( $image_url == '' || empty( $content_parts ))
      return false;

    if( !function_exists( 'imagettfbbox' ) ) {
      if( $certificate_id === 0 ) {
        $certificate_id = wp_insert_post([
          'post_type'   => WPEP_POST_TYPE_CERTIFICATE,
          'author'      => $user_id,
          'post_parent' => $parent_post_id,
          'post_title'  => sprintf( __( "%s's certificate for %s", "wpep" ), get_user_by( "id", $user_id )->display_name, get_the_title( $parent_post_id ) ),
          'post_status' => 'publish'
        ]);
      } else {
        wp_update_post( [
          'ID'          => $certificate_id,
          'post_title'  => sprintf( __( "%s's certificate for %s", "wpep" ), get_user_by( "id", $user_id )->display_name, get_the_title( $parent_post_id ) ),
          'post_status' => 'publish'
        ] );
      }

      if( is_int( $certificate_id ) ) {
        update_post_meta(
          $certificate_id,
          'certificate_path',
          str_replace(
            WPEP_BASE_URL_PATH,
            WPEP_BASE_PATH . '/',
            WPEP_COURSE_CERTIFICATE_ERROR_BACKGROUND_IMAGE
          )
        );
        update_post_meta(
          $certificate_id,
          'certificate_url',
          WPEP_COURSE_CERTIFICATE_ERROR_BACKGROUND_IMAGE
        );
      }

      return true;
    }

    $image_path = str_replace( get_site_url(), ABSPATH , $image_url );
    $image_path = str_replace( "//", "/", $image_path );

    $system_log = WPEP_Entity_SystemLog::instance( 'certification', 'generate', time() );

    $image_info = wp_check_filetype( $image_path );

    $image_object = (
    $image_info['ext'] == 'jpg'
      ? imagecreatefromjpeg( $image_path )
      : ( $image_info['ext'] == 'png' ? imagecreatefrompng( $image_path ) : imagecreatefromgif( $image_path ) )
    );

    $user_information = get_user_by( 'id', $user_id );

    $full_name = trim( $user_information->first_name . ' ' . $user_information->last_name );

    $replace_map = [
      '{full_name}'         => ( $full_name == '' ? $user_information->display_name : $full_name ),
      '{first_name}'        => $user_information->first_name,
      '{last_name}'         => $user_information->last_name,
      '{issue_date}'        => wpep_format_date(),
      '{course_name}'       => get_the_title( $parent_post_id ),
      '{post_name)'         => get_the_title( $parent_post_id ),
      '{user_site_name}'    => get_user_meta( $user_id, 'wpep_user_url', true )
    ];

    if( get_post_type( $parent_post_id ) === WPEP_POST_TYPE_COURSE ) {
      $replace_map = $replace_map + [
          '{course_started}'    => wpep_format_date( wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_STARTED, $parent_post_id, false, false, $user_id, true ) ),
          '{course_completed}'  => wpep_format_date( wpep_content_type_course_instance()->user->data->get_value( WPEP_USER_COURSE_COMPLETED_AT, $parent_post_id, false, false, $user_id, true ) )
        ];

      $course_certification_status = intval( get_post_meta( $parent_post_id, 'wpep_certificate_status', true ) );

      if( $course_certification_status === WPEP_COURSE_CERTIFICATE_STATUS_ENABLED_WITH_ASSESSMENTS ) {
        $assessment_id = wpep_user_assessment_id_by_course_id( $parent_post_id, 0, $user_id );

        if( $assessment_id != 0 ) {
          $replace_map['{assessment_start_time}'] = wpep_content_type_assessment_instance()->user->data->get_value(
            WPEP_USER_ASSESSMENT_START_TIME,
            $assessment_id,
            0,
            0,
            $user_id
          );

          $replace_map['{assessment_submission_time}'] = wpep_content_type_assessment_instance()->user->data->get_value(
            WPEP_USER_ASSESSMENT_SUBMISSION_TIME,
            $assessment_id,
            0,
            0,
            $user_id
          );

          $replace_map['{assessment_status}'] = wpep_content_type_assessment_instance()->user->data->get_value(
            WPEP_USER_ASSESSMENT_STATUS,
            $assessment_id,
            0,
            0,
            $user_id
          );

          $replace_map['{assessment_grading}'] = wpep_get_percentage(
            wpep_content_type_assessment_instance()->user->data->get_value(
              WPEP_USER_ASSESSMENT_SCORE,
              $assessment_id,
              0,
              0,
              $user_id
            )
          );
        }
      }
    }

    if( wpep_has_sid_support() )
      $replace_map['{student_identification}'] = get_user_meta( $user_information->ID, WPEP_USER_META_STUDENT_IDENTIFICATION, true );

    $system_log->started_arguments = [
      'image_path'  => $image_path,
      'replace_map' => $replace_map
    ];

    $system_log_completed_args_texts = [];

    foreach( $content_parts as $content_part ) {
      $text_font  = wpep_image_builder_get_font_path( $content_part['font-family'] );

      if( $text_font === false )
        $text_font = wpep_image_builder_get_font_path();

      $size_x     = intval( str_replace( 'px', '', $content_part['width'] ) );
      $size_y     = intval( str_replace( 'px', '', $content_part['height'] ) );
      $font_size  = intval( str_replace( 'px', '', $content_part['font-size'] ) );
      $position_x = intval( str_replace( 'px', '', $content_part['left'] ) );
      $position_y = intval( str_replace( 'px', '', $content_part['top'] ) );

      $text = $content_part['content'];

      foreach( $replace_map as $replace_key => $replace_value )
        $text = str_replace( $replace_key, $replace_value, $text );

      $system_log_completed_args_texts[ $content_part['content'] ] = $text;

      $text_box_calculation = wpep_image_builder_imagettfbbox( $font_size, 0, $text_font, $text );

      $position_x += ( $size_x > $text_box_calculation['width']
        ? intval( ( $size_x - $text_box_calculation['width'] ) / 2 )
        : 0
      );

      $position_y += ( $text_box_calculation['height'] + intval( ( $size_y - $text_box_calculation['height'] ) / 2 ) );


      list($r,$g,$b) = array_map('hexdec', str_split( ltrim( $content_part['color'], "#" ), 2 ) );

      imagettftext(
        $image_object,
        $font_size,
        0,
        $position_x,
        $position_y,
        imagecolorallocate($image_object, $r, $g, $b),
        $text_font,
        $text
      );
    }

    $upload_dir = wp_upload_dir();

    if( !is_dir( $upload_dir['basedir'] . '/wpep-certificates' ) ) {
      mkdir($upload_dir['basedir'] . '/wpep-certificates', 0755, true);

      if( !is_dir( $upload_dir['basedir'] . '/wpep-certificates' ) ) {
        $system_log->completed_arguments = [
          'error' => 'Invalid Directory : ' . $upload_dir['basedir'] . '/wpep-certificates'
        ];
        $system_log->save();

        return false;
      }
    }

    $suffix = '/wpep-certificates/' . md5( $user_id . '_' . $parent_post_id ) . '.jpg';

    $response = imagejpeg($image_object, $upload_dir['basedir'] . $suffix, 100 );

    if( $response == false || !file_exists( $upload_dir['basedir'] . $suffix ) ) {
      file_put_contents( $upload_dir['basedir'] . '/wpep-certificates/test.txt', 'test.txt' );

      $system_log->completed_arguments = [
        'status'              => 'error',
        'imagejpeg_response'  => intval( $response ),
        'file_exists'         => intval( file_exists( $upload_dir['basedir'] . $suffix ) ),
        'certificate_path'    => $upload_dir['basedir'] . $suffix,
        'certificate_url'     => $upload_dir['baseurl'] . $suffix,
        'file_test_passed'    => intval( file_get_contents( $upload_dir['basedir'] . '/wpep-certificates/test.txt' ) === 'test.txt' ),
      ];

      $system_log->completed_arguments[ 'file_test_removed' ] = intval( unlink( $upload_dir['basedir'] . '/wpep-certificates/test.txt' ) );
      $system_log->save();

      return false;
    }

    imagedestroy($image_object);

    if( $certificate_id === 0 ) {
      $certificate_id = wp_insert_post([
        'post_type'   => WPEP_POST_TYPE_CERTIFICATE,
        'author'      => $user_id,
        'post_parent' => $parent_post_id,
        'post_title'  => sprintf( __( "%s's certificate for %s", "wpep" ), get_user_by( "id", $user_id )->display_name, get_the_title( $parent_post_id ) ),
        'post_status' => 'publish'
      ]);
    } else {
      wp_update_post( [
        'ID'          => $certificate_id,
        'post_title'  => sprintf( __( "%s's certificate for %s", "wpep" ), get_user_by( "id", $user_id )->display_name, get_the_title( $parent_post_id ) ),
        'post_status' => 'publish'
      ] );
    }

    if( is_int( $certificate_id ) ) {
      update_post_meta( $certificate_id, 'certificate_path', $upload_dir['basedir'] . $suffix );
      update_post_meta( $certificate_id, 'certificate_url', $upload_dir['baseurl'] . $suffix );
    }

    $system_log->completed_arguments = [
      'certificate_id'        => $certificate_id,
      'certificate_generated' => ( file_exists( $upload_dir['basedir'] . $suffix ) ? 1 : 0 ),
      'certificate_path'      => $upload_dir['basedir'] . $suffix,
      'certificate_url'       => $upload_dir['baseurl'] . $suffix,
      'texts'                 => $system_log_completed_args_texts
    ];

    $system_log->completed_at = time();
    $system_log->save();

    return true;
  }

  /**
   * @param $user_id
   * @param $post_id
   * @return int
   */
  public function get_certificate_id_by_parent_post_id( $user_id, $post_id ) {
    if( $post_id == 0 )
      return 0;

    $certificates = get_posts( [
        'post_type'   => WPEP_POST_TYPE_CERTIFICATE,
        'author'      => $user_id,
        'post_parent' => $post_id
      ]
    );

    if( empty( $certificates ) )
      return 0;

    return array_shift( $certificates )->ID;
  }

  /**
   * @param $template
   * @return mixed
   */
  public function template_include( $template ) {
    if( !is_single() )
      return $template;

    if( get_post_type() != WPEP_POST_TYPE_CERTIFICATE )
      return $template;

    global $post;

    $this->prepare_certificate_by_parent_post_id( get_current_user_id(), $post->post_parent, get_the_ID() );

    $certificate_path = get_post_meta( get_the_ID(), 'certificate_path', true );

    if( !file_exists( $certificate_path ) )
      $certificate_path = WPEP_COURSE_CERTIFICATE_ERROR_BACKGROUND_IMAGE;

    $image_info = wp_check_filetype( $certificate_path );

    $image_object = (
    $image_info['ext'] == 'jpg'
      ? imagecreatefromjpeg( $certificate_path )
      : ( $image_info['ext'] == 'png' ? imagecreatefrompng( $certificate_path ) : imagecreatefromgif( $certificate_path ) )
    );

    if( $image_object === false ) {
      header('Content-Type: image/jpeg');

      echo file_get_contents( $certificate_path );
    } else {
      header('Content-Type: image/jpeg');

      imagejpeg($image_object, null, 100 );

      imagedestroy($image_object);
    }

    if( isset( $_GET['view-type'] ) && $_GET['view-type'] === 'image' )
      exit;

    WPEP_Entity_ActionTracking::instance()->track_event(
      get_the_ID(),
      get_current_user_id(),
      WPEP_ACTION_TRACKING_TYPE_DOWNLOAD,
      $certificate_path
    );

    exit;
  }

  /**
   * @param $course_id
   * @param string $container_class
   * @param string $button_position
   * @param string|bool $title
   * @param object|bool $certificate
   * @return string
   */
  public function get_certificate_box(
    $course_id,
    $container_class = 'wpep-hero-section wpep-hero-section-compact wpep-certificate-box',
    $button_position = 'right',
    $title = false,
    $certificate = false
  ) {
    if( get_current_user_id() == 0 )
      return '';

    $course_certification_status = intval( get_post_meta( $course_id, 'wpep_certificate_status', true ) );

    if( $course_certification_status === WPEP_COURSE_CERTIFICATE_STATUS_DISABLED )
      return '';

    $certificate = ( $certificate == false ? $this->get_certificate_id_by_parent_post_id( get_current_user_id(), $course_id ) : $certificate );

    if( $certificate == false ) {
      $award_certificate = wpep_content_type_course_instance()->user->flag->has_completed_course(
        $course_id, get_current_user_id(), ( $course_certification_status === WPEP_COURSE_CERTIFICATE_STATUS_ENABLED_WITH_ASSESSMENTS ? true : false )
      );

      if( !$award_certificate )
        return '';

      if( !$this->prepare_certificate_by_parent_post_id( get_current_user_id(), $course_id ) )
        return '';

      $certificate = $this->get_certificate_id_by_parent_post_id( get_current_user_id(), $course_id );

      if( $certificate == false )
        return '';
    }

    if( $title == false )
      $title = wpep_get_setting_translated( 'certification-download-box-title' );

    $certificate_box = '';
    $certificate_box .= '<div class="' . $container_class . '">';
    $certificate_box .=   '<h2>' . $title . '</h2>';
    $certificate_box .=   wpep_controller()->content->miscellaneous->link( [
      'position'        => $button_position,
      'text'            => wpep_get_setting_translated( 'certification-download-box-button' ),
      'force_download'  => true,
      'download_name'   => sanitize_title( get_the_title( $certificate ) ) . '.jpg'
    ], get_permalink( $certificate ) );
    $certificate_box .= '</div>';

    return $certificate_box;
  }

  public function before_course_content( $content, $course_id ) {
    return $this->get_certificate_box( $course_id ) . $content;
  }

  public function after_course_content( $content, $course_id ) {
    return $this->get_certificate_box( $course_id ) . $content;
  }

  public function after_assessment_primary_button( $user_status ) {
    if( $user_status != WPEP_USER_ASSESSMENT_STATUS_COMPLETED )
      return;

    $course_id = get_post_meta( get_the_ID(), 'wpep_course_id', true );

    echo $this->get_certificate_box( $course_id );
  }

  public function user_profile_tab_list( $tab_list, $user_id ) {
    if( $user_id === 0 )
      return $tab_list + [
            'wpep_certifications' => [
              'title'         => wpep_get_setting_translated( 'certification-user-profile-title' ),
              'content'       => ''
            ]
        ];

    if( !wpep_content_type()->contentTypes[ WPEP_POST_TYPE_CERTIFICATE ]->current_user_can_view_certificates_for( $user_id ) )
      return $tab_list;

    $certificates = ( $user_id == 0 ? [] : get_posts( [
      'post_type'   => WPEP_POST_TYPE_CERTIFICATE,
      'author'      => $user_id,
      'numberposts' => -1
    ] ) );

    if( empty( $certificates ) )
      return $tab_list;

    $tab_list['wpep_certifications'] = [
      'title'         => wpep_get_setting_translated( 'certification-user-profile-title' ),
      'content'       => [
        'type'        => 'xhr',
        'action'      => 'wpep/content-type/wpep_certificate/template-user-content',
        'action_args' => [
          'user_id' => $user_id
        ]
      ],
      'entries_count' => count( $certificates )
    ];

    return $tab_list;
  }

}