<?php

namespace WPEP\ContentType;

use WPEP\API as WPEP_API;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Video extends Skeleton\Library {

  public function setup() {
    $this->_register_api_methods();
  }

  /**
   * @return Video\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new Video\LearningJourney( $this );
  }

  public function get_post_supports() {
    return [ 'title', 'excerpt', 'editor', "custom-fields" ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_VIDEO;
  }

  public function get_singular_name() {
    return __( 'Video', "wpep" );
  }

  public function get_plural_name() {
    return __( 'Videos', "wpep" );
  }

  public function get_label_all_items() {
    return __( 'All Videos', "wpep" );
  }

  public function get_index_page_settings_field_map() {
    return [
      'general' => [
        'name'      => __( "General", "wpep" ),
        'has_title' => false,
        'alias'     => 'general',
        'fields'    => [
          'video-archive-title'  => [
            'name'    =>  __( 'Archive Title', "wpep" ),
            'alias'   => 'video-archive-title',
            'type'    => 'text_translated',
          ],
          'video-watch-now'  => [
            'name'    =>  __( 'Watch Now', "wpep" ),
            'alias'   => 'video-watch-now',
            'type'    => 'text_translated',
          ]
        ]
      ]
    ];
  }

  public function get_settings_defaults() {
    return [
      'video-archive-title'  => "Archive",
      'video-watch-now'      => "Watch Now",
    ];
  }

  public function standard_get_post_meta_fields() {
    return [
      [
        'name' => __( 'Enter the Video Link here', "wpep" ),
        'id'   => '_video_link',
        'type' => 'text',
      ],
      [
        'name'    => __('Button Text', "wpep"),
        'id'      => 'wpep_post_button_text',
        'type'    => 'text',
        'desc'    => __( 'The Button Text on the Index Page, leave this empty for default behaviour.', "wpep" )
      ],
      [
        'name'    => __('Button Color', "wpep"),
        'id'      => 'wpep_post_button_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different color for the button, leave this empty for default behaviour.', "wpep" )
      ],
      [
        'name'    => __('Button Text Color', "wpep"),
        'id'      => 'wpep_post_button_text_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different text color for the button, leave this empty for default behaviour.', "wpep" )
      ],
    ];
  }

  /**
   * @return mixed
   */
  public function get_cap_name() {
    return WPEP_POST_CAP_VIDEO;
  }

  public function get_grid_content_options() {
    return [
      'videos_per_row' => [
        'name'    => __( "Videos Per Row", "wpep" ),
        'type'    => 'select',
        'options' => [
          ''  => __( "Default", "wpep" ),
          1   => __( "One per row", "wpep" ),
          0   => __( "Do not display", "wpep" )
        ]
      ],
    ];
  }

  public function process_grid_content_options( $filter_proxy_list, $attributes ) {
    $videos_per_row       = isset($attributes['videos_per_row']) ? intval( $attributes['videos_per_row'] )  : '';

    if( $videos_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_videos_per_row'] = function( $current_per_row ) use ( $videos_per_row ) {
        return $videos_per_row;
      };

    return $filter_proxy_list;
  }

  public function library_output_grid_content_html( $course_categories = [], $post_list = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_videos_per_row', 1 );

    if( $items_per_row == 0 )
      return;

    if( empty( $post_list ) ) {
      $post_list = $this->library_get_grid_post_list( $course_categories );

      if( empty( $post_list ) )
        return;
    }

    $front_post = ( get_post_type() == WPEP_POST_TYPE_VIDEO && isset( $post_list[get_the_ID()] ) ? $post_list[get_the_ID()] : false );

    echo '<div class="wpep-video-grid-items-container" data-wpep-content-type="' . $this->get_post_type() . '" data-items-per-row="' . $items_per_row . '">';

    echo $this->get_single_video_content( $front_post );

    $css_state_class = ( count( $post_list ) == 1 ? 'wpep-hidden' : 'wpep-visible' );

    echo '<h2 class="' . $css_state_class . ' video-archive-title">' . wpep_get_setting_translated( 'video-archive-title' ) . '</h2>';
    echo '<div class="' . $css_state_class . ' video-archive-container">';

    foreach ($post_list as $key => $value) {
      $link = apply_filters( 'wpep_primary_content_item_link', get_permalink($value), $value->ID );

      echo wpep_controller()->template->get_template( 'primary/item-video.php', [
        'item'                  => $value,
        'link'                  => $link,
        'container_class'       => 'video-grid-item video-grid-item-' . $value->ID . ' ' . ( ( isset( $front_post->ID ) && $value->ID == $front_post->ID ? 'video-grid-item-open' : '' ) )
      ] );
    }

    echo '</div>';
    echo '<div class="wpep-clearfix"></div>';

    echo '</div>';
  }

  public function library_allow_content() {
    return apply_filters( 'wpep_primary_content_grid_videos_per_row', 1 ) != 0;
  }

  /**
   * @param $video_post
   * @return string
   */
  public function get_single_video_content( $video_post ) {
    if( $video_post === false )
      return '<div class="video-page-container wpep-hidden"></div>';

    $video_post = ( is_object( $video_post ) ? $video_post : get_post( $video_post ) );
    $video_link = get_post_meta( $video_post->ID, '_video_link', true );

    if( get_current_user_id() != 0 ) {
      if( !WPEP_Entity_ActionTracking::instance()->is_event_tracked( [
        'post_id' => $video_post->ID,
        'user_id' => get_current_user_id(),
        'type'    => WPEP_ACTION_TRACKING_TYPE_ACCESS_CONTENT,
        'segment' => $video_link
      ] ) ) {
        WPEP_Entity_ActionTracking::instance()->track_event(
          $video_post->ID,
          get_current_user_id(),
          WPEP_ACTION_TRACKING_TYPE_ACCESS_CONTENT,
          $video_link
        );
      }
    }

    global $post;

    $previous_post = $post;
    $post = $video_post;

    setup_postdata( $post );

    $resp = '';

    $resp .= '<div class="video-page-container">';
    $resp .=  '<h2 class="video-title">' . $video_post->post_title . '</h2>';
    $resp .=  '<h3 class="video-excerpt wpep-content-from-editor">' . $video_post->post_excerpt . '</h3>';
    $resp .=  '<div class="video-container">' . wpep_get_video_source_output( $video_link ) . '</div>';
    $resp .=  '<div class="description wpep-content-from-editor">' . wpautop( $video_post->post_content ). '</div>';
    $resp .=  '<div class="wpep-clearfix"></div>';
    $resp .= '</div>';

    $resp = apply_filters( 'the_content', $resp );

    $post = $previous_post;
    setup_postdata( $post );

    return $resp;
  }

  private function _register_api_methods() {
    $method_map = [
      'template-single-content' => [
        'callback' => '\WPEP\ContentType\Video\API::template_single_content',
        'type'    => WPEP_API_METHOD_TYPE_PUBLIC
      ]
    ];

    WPEP_API::instance()->register_content_type( $this->get_post_type(), $method_map );
  }

}