<?php

namespace WPEP\Coordinator\Content;

class Library {

  private $_is_setup = false;

  /**
   * @var Library\Block
   */
  public $block;
  /**
   * @var Library\CategoryList
   */
  public $categoryList;
  /**
   * @var Library\GridItem
   */
  public $gridItem;

  public function __construct() {
    if( !wpep_request()->is_request_type( 'frontend' ) )
      return;

    $this->_setup();
  }

  public function _setup() {
    if( $this->_is_setup )
      return;

    $this->block        = new Library\Block();
    $this->categoryList = new Library\CategoryList();
    $this->gridItem     = new Library\GridItem();

    add_action( 'wpep_primary_content_grid_list', [ $this, 'output_grid_list' ], 1 );
  }

  public function get_layout_name() {
    $layout_name = apply_filters( "wpep_primary_content_grid_layout_name", WPEP_INDEX_PAGE_LAYOUT_TOP_NAVIGATION );
    $layout_name = stripslashes( $layout_name );

    return $layout_name;
  }

  public function has_categories( $categories = false ) {
    return apply_filters( "wpep_content_primary_has_categories",
      ( is_numeric( $categories ) ? $categories : count( ( !is_array( $categories ) ? $this->get_current_categories() : $categories ) ) ) >= 2
    );
  }

  public function has_categories_navigation( $categories = false, $category_requirement = true ) {
    $items_per_row = apply_filters(
      'wpep_template_category_list_items_per_row',
      wpep_get_setting( 'index-page-categories-per-row', 0 )
    );

    return apply_filters( 'wpep_primary_content_category_navigation_enabled', ( $items_per_row != 1 && ( !$category_requirement || $this->has_categories( $categories ) ) ) );
  }

  public function has_all_categories_navigation() {
    return apply_filters( 'wpep_primary_content_all_categories_navigation_enabled', wpep_get_setting_flag( 'index-page-enable-filter-all-categories' ) );
  }

  /**
   * @param bool $with_children
   * @return array|\WP_Term
   */
  public function get_current_categories( $with_children = false ) {
    $category_list = $this->get_term_course_categories( 0, false );

    if( !$with_children ) {
      $category_list_copy = $category_list;

      if( count( $category_list ) == 1
        && apply_filters( 'wpep_primary_content_grid_categories_secondary_as_primary_view', wpep_get_setting_flag( 'index-page-single-category-primary-view' ) )
        && $this->has_categories_navigation( $category_list, false )
        && $this->get_layout_name() == WPEP_INDEX_PAGE_LAYOUT_TOP_NAVIGATION ) {
        $primary_category_id     = $category_list[ key( $category_list ) ]->term_id;
        $current_sub_categories  = get_term_children( $primary_category_id, 'course_category' );

        if( count( $current_sub_categories ) >= 2 )
          $category_list = get_terms( [
            'taxonomy'   => WPEP_TAXONOMY_CATEGORY,
            'include'    => $current_sub_categories,
            'orderby'    => 'meta_value_num',
            'meta_key'   => 'wpep_order'
          ] );

        foreach( $category_list as $category_item_key => $category_item )
          if($category_item->parent != $primary_category_id )
            unset( $category_list[ $category_item_key ] );
      }

      if( empty( $category_list ) )
        $category_list = $category_list_copy;
    } else {
      foreach( $category_list as $category_index => $category_item ) {
        $category_list[ $category_index ]->children = $this->get_term_course_categories( $category_item->term_id, true );
      }
    }

    return $category_list;
  }

  public function get_term_course_categories( $parent = 0, $hide_empty = false ) {
    $category_list_args = [
      'taxonomy'   => WPEP_TAXONOMY_CATEGORY,
      'hide_empty' => $hide_empty,
      'parent'     => $parent,
      'orderby'    => 'meta_value_num',
      'meta_key'   => 'wpep_order'
    ];

    $category_list_args = apply_filters( 'wpep_primary_content_category_list_args', $category_list_args );

    $category_list = get_terms( $category_list_args );
    $category_list = apply_filters( 'wpep_primary_content_grid_categories', $category_list );

    if( !empty( $category_list ) && wpep_get_setting_flag( 'index-page-enforce-ordering' ) )
      $category_list = wpep_term_list_ordering( $category_list );

    return $category_list;
  }

  public function output_grid_list() {
    do_action( "wpep_primary_content_before_grid" );

    $entries_per_page = intval( apply_filters( 'wpep_primary_content_entries_per_page', wpep_get_setting_flag( 'index-page-enable-pagination' ) ? wpep_get_setting( 'index-page-entries-per-page' ) : 0 ) );
    $options = [
      'entries_per_page'       => $entries_per_page,
      'current_page'           => ( isset( $_POST[ 'wpep_library_page'] )  ? intval( $_POST[ 'wpep_library_page'] ) : ( isset( $_GET[ 'wpep-library-page'] )  ? intval( $_GET[ 'wpep-library-page'] ) : 1 ) ),
      'search'                 => ( isset( $_POST[ 'wpep_library_search']) ? sanitize_text_field( $_POST[ 'wpep_library_search'] ) : ( isset( $_GET[ 'wpep-library-search']) ? sanitize_text_field( $_GET[ 'wpep-library-search'] ) : '' ) ),
      'require_all_categories' => false
    ];

    // pagination_xhr

    if( isset( $_POST['wpep_library_categories'] ) ) {
      $course_categories = array_map( 'intval', $_POST['wpep_library_categories'] );
      $options['require_all_categories'] = wpep_get_setting_flag( 'index-page-categories-filter-require-all' );
    } elseif( isset( $_GET['wpep-library-categories'] ) ) {
      $course_categories = array_map( 'intval', explode( "-", $_GET['wpep-library-categories'] ) );
      $options['require_all_categories'] = wpep_get_setting_flag( 'index-page-categories-filter-require-all' );
    } else {
      $course_categories = $this->get_current_categories();

      $course_categories = is_array( $course_categories ) ? wpep_utility_map_to_array_assoc( $course_categories, 'term_id', 'term_id' ) : $course_categories;

      if( $entries_per_page != 0 ) {
        if( isset( $_GET[ 'wpep-library-base-category'] ) ) {
          $options['current_category'] = $_GET[ 'wpep-library-base-category'];
        } else if( wpep_content_primary_layout_name() === WPEP_INDEX_PAGE_LAYOUT_TOP_NAVIGATION
          && count( $course_categories ) >=2
          && $this->has_all_categories_navigation()
        ) {
          $options['current_category'] = 0;
        } else if( !$options['require_all_categories'] && wpep_content_primary_layout_name() === WPEP_INDEX_PAGE_LAYOUT_TOP_NAVIGATION ) {
          $course_categories = array_shift( $course_categories );
        }

        if( isset( $options['current_category'] ) && $options['current_category'] != 0 && is_numeric( $options['current_category'] ) )
          $course_categories = $options['current_category'];
      }
    }

    if( $options['require_all_categories'] )
      add_filter( 'wpep_primary_content_list_args', [ $this, '_output_grid_list_require_all_categories' ], 100 );

    wpep_content_type()->library_output_grid_content_html( $course_categories, $options );

    echo '<div class="wpep-clearfix"></div>';

    do_action( "wpep_primary_content_after_grid" );

    if( $options['require_all_categories'] )
      remove_filter( 'wpep_primary_content_list_args', [ $this, '_output_grid_list_require_all_categories' ], 100 );
  }

  public function _output_grid_list_require_all_categories( $args ) {
    if( !isset( $args[ 'tax_query' ] ) )
      return $args;

    if( !isset( $args[ 'tax_query']['course_category'] ) )
      return $args;

    $args[ 'tax_query']['relation'] = 'AND';

    foreach( $args[ 'tax_query']['course_category']['terms'] as $term_id )
      $args[ 'tax_query']['course_category_' . $term_id ] = [
        'taxonomy' => WPEP_TAXONOMY_CATEGORY,
        'field'    => 'term_id',
        'terms'    => $term_id
      ];

    unset( $args[ 'tax_query']['course_category'] );

    return $args;
  }

}
