<?php

namespace WPEP\Coordinator\Content;

class Search {

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'search', [ $this, '_display_form_from_attributes' ] );
    add_action( 'widgets_init', [ $this, '_register_widget' ], 10 );
  }

  public function _display_form_from_attributes( $attributes ) {
    $component_args = [ 'action' => 'wpep/misc/content-type-search' ];

    if( isset( $attributes['post_type'] ) && $attributes['post_type'] !== '' )
      $component_args[ 'post_type_list' ] = $attributes['post_type'];

    if( isset( $attributes['include_categories'] ) && $attributes['include_categories'] !== '' )
      $component_args[ 'include_categories' ] = $attributes['include_categories'];

    if( isset( $attributes['exclude_categories'] ) && $attributes['exclude_categories'] !== '' )
      $component_args[ 'exclude_categories' ] = $attributes['exclude_categories'];

    return '<div data-ec-component="search" 
                 data-ec-component-args="' . esc_attr( json_encode( $component_args ) ) . '"></div>';
  }

  public function _register_widget() {
    register_widget('\WPEP\Coordinator\Content\Search\Widget' );
  }

  /**
   * @param $keyword
   * @param array $post_types
   * @param int $result_entries_count
   * @param array $search_rules
   * @return array
   */
  public function get_as_array( $keyword, $post_types = [], $result_entries_count = 10, $search_rules = [] ) {
    $post_list = $this->get_as_posts( $keyword, $post_types, $result_entries_count, $search_rules );
    $response  = [];

    foreach( $post_list as $post ) {
      $response[] = wpep_content_type()->library_get_search_details( [
        'ID'        => $post->ID,
        'post_type' => $post->post_type,
        'title'     => $post->post_title,
        'thumbnail' => wpep_get_placeholder_image(),
        'permalink'             => false,
        'thumbnail_ribbon_text' => '',
      ], $post );
    }

    return $response;
  }

  /**
   * @param $search
   * @param array $post_types
   * @param int $result_entries_count
   * @param array $search_rules
   * @return array
   */
  public function get_as_posts( $search, $post_types = [], $result_entries_count = 10, $search_rules = [] ) {
    if( is_string( $post_types ) && $post_types !== '' )
      $post_types = explode( ",", $post_types );

    if( !is_array( $post_types ) || empty( $post_types ) ) {
      $post_types = wpep_content_type()->library_post_types();
    } else {
      foreach( $post_types as $k => $post_type )
        if( !in_array( $post_type, wpep_content_type()->library_post_types() ) )
          unset( $post_types[ $k ] );
    }

    $search = sanitize_text_field( $search );

    $query_args = [
      'post_type'   => $post_types,
      'post_status' => 'publish',
      'meta_query'  => [
        'wpep_search' => [
          'relation' => 'OR',
          [
            'key'     => 'wpep_show_on_index',
            'value'   => '1',
            'compare' => '=',
          ],
          [
            'key'     => 'wpep_show_on_index',
            'compare' => 'NOT EXISTS'
          ]
        ]
      ],
      'paged'          => false,
      'posts_per_page' => $result_entries_count
    ];

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $query_args['post_status'] = wpep_post_monitored_post_status_list();

    if( isset( $search_rules['include_categories'] ) && $search_rules['include_categories'] !== '' ) {
      $include_categories = $search_rules['include_categories'];

      if( !is_array( $include_categories ) )
        $include_categories = explode( ",", $include_categories );

      $include_categories = array_map( 'absint', $include_categories );

      $query_args['tax_query'] = [
        'wpep_search_include_categories'  => [
          'taxonomy'         => WPEP_TAXONOMY_CATEGORY,
          'field'            => 'id',
          'terms'            => $include_categories,
          'operator'         => 'IN'
        ]
      ];
    } else if( isset( $search_rules['exclude_categories'] ) && $search_rules['exclude_categories'] !== '' ) {
      $exclude_categories = $search_rules['exclude_categories'];

      if( !is_array( $exclude_categories ) )
        $exclude_categories = explode( ",", $exclude_categories );

      $exclude_categories = array_map( 'absint', $exclude_categories );

      $query_args['tax_query'] = [
        'wpep_search_exclude_categories'  => [
          'taxonomy'         => WPEP_TAXONOMY_CATEGORY,
          'field'            => 'id',
          'terms'            => $exclude_categories,
          'operator'         => 'NOT IN'
        ]
      ];
    }

    if( $search != '' ) {
      $query_args['s'] = $search;
      unset( $query_args['orderby'] );
      unset( $query_args['order'] );
    }

    return get_posts( $query_args );
  }

}
