<?php

namespace WPEP\Coordinator\Content\UserProfile;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FilterProxy as FilterProxy;

class Delivery {

  use FilterProxy;

  public function __construct() {
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'profile', [ $this, 'primary' ] );
    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'public_profile_link', [ $this, 'public_profile_link'] );

    if( wpep_get_setting_flag( 'user-profile-tab-author-content-enabled' ) )
      add_filter( 'wpep_content_user_profile_tab_list', [ $this, '_author_profile_tabs' ], 30, 2 );

    if( wpep_get_setting_flag( 'user-profile-custom-tabs-enabled' ) )
      add_filter( 'wpep_content_user_profile_tab_list', [ $this, '_custom_profile_tabs' ], 150, 2 );

    if( wpep_request()->is_request_type( 'admin' ) )
      return;
  }

  public function get_tab_list( $user_id = false ) {
    $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

    if( $user_id == 0 && !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return [];

    $tab_list = [];

    if( wpep_get_setting_flag( 'user-profile-tab-my-content-enabled' )
        && wpep_controller()->contentLibrary->is_integrated()
        && \WPEP\ACL\User::profile( $user_id ) ) {
      $tab_list[ 'my_content' ] = [
        'title'   => wpep_get_setting_translated( 'user-profile-tab-my-content' ),
        'content' => [
          'type'        => 'xhr',
          'action'      => 'wpep/user/template-content-purchased',
          'action_args' => [
            'user_id' => $user_id
          ]
        ]
      ];
    }

    $has_statistics_current_user = ( $user_id == 0 );

    if( !$has_statistics_current_user )
      $has_statistics_current_user = wpep_get_setting_flag( 'enable-student-supervisor' )
                                        && user_can( $user_id, WPEP_CAP_SUPERVISE_STUDENTS )
                                        && ( $user_id == get_current_user_id() || current_user_can( WPEP_CAP_MANAGE_MEMBERS ) );

    if( !$has_statistics_current_user )
      $has_statistics_current_user = !empty( \wpep_controller()->contentLibrary->get_sub_account_receiver_user_ids_from_provider_user_id( $user_id ) );

    if( $has_statistics_current_user ) {
      $tab_list['statistics_current_user'] = [
        'title'   => wpep_get_setting( 'user-profile-tab-statistics-current-user' ),
        'content' => [
          'type'        => 'xhr',
          'action'      => 'wpep/system/user/template-statistics-current-user',
          'action_args' => [
            'user_id' => $user_id
          ]
        ]
      ];
    }

    $tab_list = apply_filters( 'wpep_content_user_profile_tab_list', $tab_list, $user_id );

    if( !is_array( $tab_list ) || empty( $tab_list ) )
      return $tab_list;

    return wpep_user_profile_handle_tab_ordering( $tab_list );
  }

  public function primary($attributes, $content = null) {
    $user_information = WPEP_Controller::instance()->content->userProfile->get_display_user();

    if( $user_information == false )
      return WPEP_Controller::instance()->authentication->get_form( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    return WPEP_Controller::instance()->template->get_template(
      'user-profile.php', [
        'user_information' => $user_information,
        'include_header'   => false,
        'include_footer'   => false
      ]
    );
  }

  public function public_profile_link( $attributes, $content = '' ) {
    $content = ( is_string( $content ) ? $content : '' );
    $link    = wpep_user_profile_public_link( get_current_user_id() );

    if( $link === false )
      return '';

    if( $content !== '' )
      return '<a href="' . $link . '">' . $content . '</a>';

    return $link;
  }

  public function _author_profile_tabs( $tabs, $user_id ) {
    if( $user_id === 0 ) {
      if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
        return $tabs + [
            'author_content_list' => [
              'title'   => wpep_get_setting_translated( 'user-profile-tab-author-content' ),
              'content' => '<div class="wpep-notification wpep-notification-info">' . __( "Admin Notification : This tab will display for users that have created content on the platform, ideal for instructors.", "wpep" ) . '</div>'
            ]
        ];

      return $tabs;
    }

    if( !wpep_content_type()->user_has_created_content( $user_id ) )
      return $tabs;

    $tabs[ 'author_content_list' ] = [
      'title'   => wpep_get_setting_translated( 'user-profile-tab-author-content' ),
      'content' => [
        'type'        => 'xhr',
        'action'      => 'wpep/user/template-content-created',
        'action_args' => [
          'user_id' => $user_id
        ]
      ]
    ];

    return $tabs;
  }

  public function _custom_profile_tabs( $tabs, $user_id ) {
    $custom_tabs = wpep_get_setting( 'user-profile-custom-tabs' );

    if( !is_array( $custom_tabs ) )
      return $tabs;

    if( empty( $custom_tabs ) )
      return $tabs;

    if( !\WPEP\ACL\User::profile( $user_id ) )
      return $tabs;

    $current_tabs = [];

    foreach( $custom_tabs as $tab_title => $tab_content ) {
      $current_tabs[ 'custom_' . sanitize_title( $tab_title ) ] = [
        'title'         => $tab_title,
        'content'       => wpep_format_content( $tab_content )
      ];
    }

    return (
    wpep_get_setting( 'user-profile-custom-tabs-position' ) === 'left' ?
      $current_tabs + $tabs
      : $tabs + $current_tabs
    );
  }

}