<?php

namespace WPEP\Coordinator\Discussion;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Entity\Discussion as WPEP_Entity_Discussion;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

class EmailNotifications {

  public function setup() {
    add_action( 'wpep_cron_discussion_email_notifications', [ $this, '_send_discussion_email_notifications' ] );
  }

  public function _send_discussion_email_notifications() {
    $discussion_integrations = WPEP_Controller::instance()->discussion->all();

    if( empty( $discussion_integrations ) )
      return;

    foreach( $discussion_integrations as $discussion_integration_alias => $discussionIntegration ) {
      if( $discussionIntegration->get_email_notifications_status() == true )
        continue;

      unset( $discussion_integrations[$discussion_integration_alias] );
    }

    if( empty( $discussion_integrations ) )
      return;

    $system_log = WPEP_Entity_SystemLog::instance( 'cron', 'discussion_email_notifications', time() );

    $current_user_id = get_current_user_id();

    $notifications_data = WPEP_Entity_Discussion::instance()->get_email_notifications_data( $discussion_integrations, 30 );

    $system_log->started_arguments = $notifications_data;

    if( !isset( $notifications_data['user_id_map'] ) ) {
      $system_log->completed_at = time();
      $system_log->save();

      return;
    }

    $system_log->completed_arguments = [ 'processed_user_ids' => [] ];
    $system_log->save();


    foreach( $notifications_data['user_id_map'] as $user_id => $user_discussions_map ) {
      // Set Current User.
      wp_set_current_user( $user_id );

      foreach( $user_discussions_map as $user_discussion_id => $user_discussion_last_activity ) {
        $discussionIntegration = WPEP_Controller::instance()->discussion->get( $notifications_data['discussions'][ $user_discussion_id ]->namespace );

        if( !$discussionIntegration->current_user_can_view( $notifications_data['discussions'][ $user_discussion_id ] )
            || !wpep_user_has_notification( $user_id, 'discussion' ) ) {
          WPEP_Entity_Discussion::instance()->set_discussion_user_data(
            WPEP_DISCUSSION_USER_DATA_EMAIL_NOTIFICATION_STATUS,
            0,
            $user_discussion_id,
            $user_id
          );
          unset( $user_discussions_map[ $user_discussion_id ] );
        }
      }

      if( empty( $user_discussions_map ) )
        continue;

      $system_log->completed_arguments[ 'processed_user_ids' ][] = $user_id;

      $this->send_notifications_for_user( $user_id, $user_discussions_map, $notifications_data['discussions'] );
    }

    if( $current_user_id != 0 )
      wp_set_current_user( $current_user_id );

    $system_log->completed_at = time();
    $system_log->save();
  }

  public function send_notifications_for_user( $user_id, $user_discussions_map = [], $discussions = [] ) {
    $email_title   = WPEP_Settings::instance()->get_option( 'comments-email-notification-title' );
    $email_content = WPEP_Settings::instance()->get_option( 'comments-email-notification-content' );

    $discussions_map = [];
    $comments_per_discussion = floor( 10 / count( $user_discussions_map ) );
    $current_comment_limit = ( count( $user_discussions_map ) == 1 ? 5 : $comments_per_discussion );

    foreach( $user_discussions_map as $discussion_id => $discussion_last_user_activity ) {
      $get_discussion_comments_args = [
        'discussion_id' => $discussion_id,
        'created_at'    => [ '>' => gmdate( 'Y-m-d H:i:s', $discussion_last_user_activity ) ],
        'parent_id'     => [ 'relevant' => $user_id ]
      ];

      $discussion_comments = WPEP_Entity_Discussion::instance()->get_discussion_comments(
        $get_discussion_comments_args,
        [
          'limit'     => $current_comment_limit,
          'order_by'  => 'created_at',
          'do_not_order_by_pinned' => true
        ]
      );

      WPEP_Entity_Discussion::instance()->set_discussion_user_data( WPEP_DISCUSSION_USER_DATA_LAST_EMAIL_NOTIFICATION, time(), $discussion_id, $user_id );

      if( empty( $discussion_comments ) ) {
        unset( $discussions_map[ $discussion_id ] );

        continue;
      }

      $discussion_comments = wpep_utilities_map_object( $discussion_comments, 'id' );

      foreach( $discussion_comments as $discussion_comment_id => $discussion_comment ) {
        if( $discussion_comment->parent_id == 0 )
          continue;

        if( !isset( $discussion_comments[ $discussion_comment->parent_id ] ) )
          $discussion_comments[ $discussion_comment->parent_id ] = WPEP_Entity_Discussion::instance()->get_discussion_comment( [ 'id' => $discussion_comment->parent_id ] );

        if( !isset( $discussion_comments[ $discussion_comment->parent_id ]->replies ) )
          $discussion_comments[ $discussion_comment->parent_id ]->replies = [ $discussion_comment ];
        else {
          $discussion_comments[ $discussion_comment->parent_id ]->replies[] = $discussion_comment;

          usort($discussion_comments[ $discussion_comment->parent_id ]->replies, function($a, $b) {
            return strtotime($a->last_activity) - strtotime($b->last_activity);
          });
        }
        unset($discussion_comments[$discussion_comment_id]);
      }

      usort($discussion_comments, function($a, $b) {
        return strtotime($b->last_activity) - strtotime($a->last_activity);
      });

      $discussions_map[ $discussion_id ] = $discussions[ $discussion_id ];
      $discussions_map[ $discussion_id ]->user_last_activity = wpep_format_date_with_time( $discussion_last_user_activity );
      $discussions_map[ $discussion_id ]->comments = $discussion_comments;
    }

    if( empty( $discussions_map ) )
      return true;

    $email_content = str_replace( '[discussion_activity_number]', count( $discussions_map ), $email_content );
    $email_content = str_replace( '[discussion_activity_preview]', wpep_template_get( 'email-template/discussion-notifications.php', [
      'discussions_map' => $discussions_map,
      'user_id'         => $user_id
    ] ), $email_content );

    wpep_controller()->userEngagement->send_email(
      $user_id,
      $email_title,
      $email_content,
      true
    );

    return true;
  }

}