<?php

namespace WPEP\Coordinator;

use WPEP\Container\LicenseAndUpdates as WPEP_Container_LicenseAndUpdates;

class LicenseLibrary {

  /**
   * @var WPEP_Container_LicenseAndUpdates[]
   */
  protected $_integrations = [];

  /**
   * @var array
   */
  private $_license_verification_history;

  public function setup() {
    if( !wpep_request()->is_request_type( 'admin' ) )
      return;

    add_action( 'init', [ $this, '_init' ], 5 );
  }

  public function _init() {
    $this->_license_verification_history = get_option( 'wpep_license_verification', [] );

    if( get_transient( 'wpep_license_verification_status' ) == 'delay' )
      return;

    foreach( $this->get_integrations() as $integration_alias => $integration ) {
      if( !isset( $this->_license_verification_history[ $integration_alias ] ) ) {
        $this->_license_verification_history[ $integration_alias ] = time();

        update_option( 'wpep_license_verification', $this->_license_verification_history );

        set_transient( 'wpep_license_verification_status', 'delay', MINUTE_IN_SECONDS );

        return;
      }

      if( $this->_license_verification_history[ $integration_alias ] + 86400 < time() ) {
        $integration->refresh_license_status();

        $this->_license_verification_history[ $integration_alias ] = time();

        update_option( 'wpep_license_verification', $this->_license_verification_history );

        set_transient( 'wpep_license_verification_status', 'delay', MINUTE_IN_SECONDS );

        return;
      }
    }

    // After everything is checked, we delay the verifications by 1 hour.
    set_transient( 'wpep_license_verification_status', 'delay', HOUR_IN_SECONDS );
  }

  public function get_license_list() {
    if( $this->_license_verification_history === null )
      $this->_license_verification_history = get_option( 'wpep_license_verification', [] );

    $response = [];

    foreach( $this->get_integrations() as $integration_alias => $integration ) {
      $response[ $integration_alias ] = [
        'name'               => $integration->get_product_name(),
        'alias'              => $integration_alias,
        'is_valid'           => $integration->is_valid_license(),
        'is_bundled_license' => $integration->in_bundled_license(),
        'license'            => $integration->get_license(),
        'last_verification'  => ( isset( $this->_license_verification_history[ $integration_alias ] ) ? $this->_license_verification_history[ $integration_alias ] : 0 )
      ];

      $response[ $integration_alias ][ 'status' ] = ( $response[ $integration_alias ][ 'is_valid' ] ? 'success' : 'danger' );
    }

    return $response;
  }

  /**
   * @param WPEP_Container_LicenseAndUpdates $license_integration
   */
  public function register( $license_integration ) {
    $this->_integrations[ $license_integration->get_plugin_alias() ] = $license_integration;
  }

  /**
   * @return WPEP_Container_LicenseAndUpdates[]
   */
  public function get_integrations(){
    return $this->_integrations;
  }

  /**
   * @param $alias
   * @return bool|WPEP_Container_LicenseAndUpdates
   */
  public function get_integration( $alias ) {
    return ( isset( $this->_integrations[ $alias ] ) ? $this->_integrations[ $alias ] : false );
  }

  public function has_valid_licenses() {
    foreach( $this->get_license_list() as $license )
      if( $license[ 'is_valid' ] == false )
        return false;

    return true;
  }

}