<?php

namespace WPEP\Coordinator;

class Menu {

  protected $_menu_item_list  = [];
  public $item_list_arguments = [
    'title' => 'title',
    'url'   => 'href'
  ];

  public function setup() {
    add_filter('wp_setup_nav_menu_item', [ $this, '_nav_menu_item_filter' ] );
  }

  public function _nav_menu_item_filter( $item ) {
    global $pagenow;

    if( $pagenow == 'nav-menus.php'
        || $pagenow == 'customize.php'
        || defined('DOING_AJAX' ) )
      return $item;

    if( !isset( $item->url ) )
      return $item;

    if( strpos( $item->url, '#' . WPEP_MENU_PREFIX ) !== 0 )
      return $item;

    $args = [];

    $identifier = trim( $item->url, '#' );
    $identifier = substr( $identifier, strlen( WPEP_MENU_PREFIX ) );

    if( strpos( $identifier, '::' ) !== false ) {
      $args = explode( "::", $identifier );

      $identifier = array_shift( $args );

      reset( $args );
    }

    $current_item_information = $this->get_item( $identifier );

    if( $current_item_information === false )
      return $item;

    foreach( $this->item_list_arguments as $object_attribute => $information_argument ) {
      if( !isset( $current_item_information[ $information_argument ] ) )
        continue;

      if( is_callable( $current_item_information[ $information_argument ] ) )
        $item->$object_attribute = $current_item_information[ $information_argument ]( $item, $args );
      else
        $item->$object_attribute = $current_item_information[ $information_argument ];
    }

    return $item;
  }

  public function get_item( $identifier ) {
    $available_items = $this->get_items();

    if( !isset( $available_items[ $identifier ] ) )
      return false;

    return $available_items[ $identifier ];
  }

  public function get_items() {
    if( !empty( $this->_menu_item_list ) )
      return $this->_menu_item_list;

    $this->_menu_item_list = [
      'login' => [
        'name'  => __( "Login", "wpep" ),
        'href'  => function( $item_data, $args ) {
          return wp_login_url( ( isset( $args[ 0 ] ) ? $args[ 0 ] : get_site_url() ) );
        }
      ],
      'logout' => [
        'name' => __( "Logout", "wpep" ),
        'href' => function( $item_data, $args ) {
          return wp_logout_url( ( isset( $args[ 0 ] ) ? $args[ 0 ] : get_site_url() ) );
        }
      ],
      'login_logout' => [
        'name'  => __( "Login|Logout", "wpep" ),
        'title' => function( $item_data ) {
          $titles = explode('|', $item_data->title );

          if(!is_user_logged_in()) {
            return esc_html(isset( $titles[0] ) ? $titles[0] : __( 'Login', 'wpep' ) );
          } else {
            return esc_html(isset( $titles[1] ) ? $titles[1] : __( 'Logout', 'wpep') );
          }
        },
        'href'  => function( $item_data, $args ) {
          if( !is_user_logged_in() )
            return wp_login_url( ( isset( $args[ 0 ] ) ? $args[ 0 ] : get_site_url() ) );

          return wp_logout_url( ( isset( $args[ 1 ] ) ? $args[ 1 ] : get_site_url() ) );
        }
      ]
    ];

    if( wpep_public_profiles_enabled() ) {
      $this->_menu_item_list[ 'my_public_profile' ] = [
        'name'  => __( "My Profile", "wpep" ),
        'href'  => function() {
          if( !is_user_logged_in() )
            return wp_login_url();

          return wpep_user_profile_public_link( get_current_user_id() );
        }
      ];

      $this->_menu_item_list[ 'login_my_public_profile' ] = [
        'name'  => __( "Login|My Profile", "wpep" ),
        'title' => function( $item_data ) {
          $titles = explode('|', $item_data->title );

          if(!is_user_logged_in()) {
            return esc_html(isset( $titles[0] ) ? $titles[0] : __( 'Login', 'wpep' ) );
          } else {
            return esc_html(isset( $titles[1] ) ? $titles[1] : __( 'My Profile', 'wpep') );
          }
        },
        'href'  => function() {
          if( !is_user_logged_in() )
            return wp_login_url();

          return wpep_user_profile_public_link( get_current_user_id() );
        }
      ];
    }

    if( wpep_discussion_direct_url_enabled() ) {
      $this->_menu_item_list[ 'my_discussions' ] = [
        'name'  => __( "My Discussions", "wpep" ),
        'title' => function( $item_data ) {
          if( wpep_get_setting_flag( 'comments-direct-url-menu-item-bubble' ) ) {
            return '<div data-wpep-discussion-direct-url-menu-block="pending" data-wpep-discussion-direct-url-menu-item-click="' . intval( wpep_get_setting_flag( 'comments-direct-url-menu-item-bubble-item-click' ) ) . '">' .
                      '<div class="wpep-discussion-direct-url-menu-block-title-container">' .
                          '<p class="wpep-discussion-direct-url-menu-block-title">' . esc_html($item_data->title ) . '</p>' .
                          '<p class="wpep-discussion-direct-url-menu-block-icon-container"><span class="wpep-discussion-direct-url-menu-block-icon">' . wpep_mini_spinner_loader() . '</span></p>' .
                      '</div>' .
                      '<div class="wpep-discussion-direct-url-menu-block-notifications"></div>' .
                   '</div>';
          }

          return $item_data->title;
        },
        'href'  => function() {
          if( !is_user_logged_in() )
            return wp_login_url( wpep_discussion_direct_url_link() );

          return wpep_discussion_direct_url_link();
        }
      ];

      $this->_menu_item_list[ 'login_my_discussions' ] = [
        'name'  => __( "Login|My Discussions", "wpep" ),
        'title' => function( $item_data ) {
          $titles = explode('|', $item_data->title );

          if(!is_user_logged_in()) {
            return esc_html(isset( $titles[0] ) ? $titles[0] : __( 'Login', 'wpep' ) );
          } else {
            if( wpep_get_setting_flag( 'comments-direct-url-menu-item-bubble' ) ) {
              return '<div data-wpep-discussion-direct-url-menu-block="pending" data-wpep-discussion-direct-url-menu-item-click="' . intval( wpep_get_setting_flag( 'comments-direct-url-menu-item-bubble-item-click' ) ) . '">' .
                        '<div class="wpep-discussion-direct-url-menu-block-title-container">' .
                            '<p class="wpep-discussion-direct-url-menu-block-title">' . esc_html(isset( $titles[1] ) ? $titles[1] : __( 'My Discussions', 'wpep' ) ) . '</p>' .
                            '<p class="wpep-discussion-direct-url-menu-block-icon-container"><span class="wpep-discussion-direct-url-menu-block-icon">' . wpep_mini_spinner_loader() . '</span></p>' .
                        '</div>' .
                        '<div class="wpep-discussion-direct-url-menu-block-notifications"></div>' .
                     '</div>';
            }

            return esc_html(isset( $titles[1] ) ? $titles[1] : __( 'My Discussions', 'wpep') );
          }
        },
        'href'  => function() {
          if( !is_user_logged_in() )
            return wp_login_url( wpep_discussion_direct_url_link() );

          return wpep_discussion_direct_url_link();
        }
      ];
    }

    $this->_menu_item_list = apply_filters( 'wpep_menu_item_list', $this->_menu_item_list );

    return $this->_menu_item_list;
  }

}