<?php

namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;
use WPEP\Container\FilterProxy as FilterProxy;

class UserEngagement {

  use FilterProxy;

  /**
   * @var UserEngagement\Assessment
   */
  public $assessment;

  /**
   * @var UserEngagement\AutoLogin
   */
  public $autoLogin;

  /**
   * @var UserEngagement\Misc
   */
  public $misc;

  /**
   * @var UserEngagement\WeeklyRecap
   */
  public $weeklyRecap;

  /**
   * @var int|\WP_User
   */
  public $loop_user = 0;
  public $loop_courses_started = [];
  public $loop_courses_completed = [];
  public $loop_courses_recommended = [];
  public $loop_courses_updated = [];
  public $loop_lessons_completed_count = 0;

  private $_all_updated_courses = [];
  private $_temporary_shortcodes_registered = [];

  public function __construct() {
    $this->assessment       = new UserEngagement\Assessment();
    $this->autoLogin        = new UserEngagement\AutoLogin();
    $this->misc             = new UserEngagement\Misc();
    $this->weeklyRecap      = new UserEngagement\WeeklyRecap();

    if( !empty( wpep_get_setting( 'user-register-tag-email-marketing-map' ) ) )
      add_action( 'user_register', [ $this, "_on_add_user" ], 15 );

    if( wpep_get_setting( 'user-delete-tag-email-marketing' ) !== '' )
      add_action( 'delete_user', [ $this, "_on_delete_user" ], 15 );

    add_action( "wp_login", [ $this, '_on_user_login' ], 10, 2 );
    add_action( "clear_auth_cookie", [ $this, '_on_user_logout'], 10 );
  }

  public function _on_add_user( $user_id ) {
    $email_marketing_map = wpep_get_setting( 'user-register-tag-email-marketing-map' );

    foreach( $email_marketing_map as $email_marketing_alias => $email_marketing_tag ) {
      if( empty( $email_marketing_tag ) )
        continue;

      $integration = wpep_controller()->emailMarketing->get_integration( $email_marketing_alias );

      if( $integration === false )
        continue;

      $integration->add_tag( $email_marketing_tag, $user_id );
    }
  }

  public function _on_delete_user( $user_id ) {
    try {
      wpep_controller()->emailMarketing->tag_user( wpep_get_setting( 'user-delete-tag-email-marketing' ), $user_id );
    } catch ( \Exception $e ) {
      return false;
    }

    return true;
  }

  public function get_user_list() {
    return wpep_utility_map_to_array_assoc(
      wpep_content_type_course_instance()->user->data->get(
        WPEP_USER_COURSE_STARTED, false, false, false, false, false
      ),
      'user_id',
      'user_id'
    );

  }

  private function _pre_register_temporary_shortcodes() {
    if( !empty( $this->_temporary_shortcodes_registered ) )
      return;

    global $shortcode_tags;

    $this->_temporary_shortcodes_registered = $shortcode_tags;
    remove_all_shortcodes();
  }

  private function _register_temporary_content_shortcodes() {
    add_shortcode( 'video', function() {
      return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_VIDEO . '"/>';
    });

    if( wpep_get_setting_flag( 'enable-backward-compatibility-short-codes' ) ) {
      add_shortcode( 'wpep_video', function() {
        return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_VIDEO . '"/>';
      });
    }

    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'link', function( $link, $attributes = [] ) {
      $text = isset($attributes['text']) ? $attributes['text'] : $link;

      return '<a href="' . esc_attr( $link ) . '">' . sanitize_text_field( $text ) . '</a>';
    });

    add_shortcode( WPEP_SHORT_CODE_PREFIX . 'video', function() {
      return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_VIDEO . '"/>';
    });

    add_shortcode( 'audio', function() {
      return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_AUDIO . '"/>';
    });

    add_shortcode( "user_name", function() {
      return wpep_content_user_profile_format_name( get_user_by( 'id', $this->loop_user->ID ) );
    } );
    add_shortcode( "name", function() {
      return wpep_content_user_profile_format_name( get_user_by( 'id', $this->loop_user->ID ) );
    } );
    add_shortcode( 'email_address', function() {
      return $this->loop_user->user_email;
    } );
    add_shortcode( "site_url", function() {
      return get_site_url();
    } );
    add_shortcode( "date", function() {
      return wpep_format_date();
    } );
    add_shortcode( "date_time", function() {
      return wpep_format_date_with_time();
    } );
    add_shortcode( "number_lessons_completed", function() {
      return $this->loop_lessons_completed_count;
    } );

    add_shortcode( "list_courses_updated", function() {
      if( empty( $this->loop_courses_updated ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_updated,
        'course_object_date'   => 'post_modified'
      ] );
    } );
    add_shortcode( "list_courses_started", function() {
      if( empty( $this->loop_courses_started ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_started,
        'course_user_data_key' => WPEP_USER_COURSE_STARTED
      ] );
    } );
    add_shortcode( "list_courses_completed", function() {
      if( empty( $this->loop_courses_completed ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_completed,
        'course_user_data_key' => WPEP_USER_COURSE_COMPLETED_AT
      ] );
    } );
    add_shortcode( "list_courses_recommended", function() {
      if( empty( $this->loop_courses_recommended ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_recommended
      ] );
    } );
  }

  private function _register_temporary_conditional_shortcodes() {
    add_shortcode( "condition_has_updated_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_updated ) ? do_shortcode( $content ) : '';
    } );

    add_shortcode( "condition_has_started_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_started ) ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_has_completed_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_completed ) ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_has_recommended_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_recommended ) ? do_shortcode( $content ) : '';
    } );

    add_shortcode( "condition_has_activity", function( $atts, $content ) {
      return !empty( $this->loop_courses_started ) || !empty( $this->loop_courses_completed ) || $this->loop_lessons_completed_count != 0 ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_no_activity", function( $atts, $content ) {
      return empty( $this->loop_courses_started ) && empty( $this->loop_courses_completed ) && $this->loop_lessons_completed_count == 0 ? do_shortcode( $content ) : '';
    } );
  }

  private function _remove_temporary_content_shortcodes() {
    global $shortcode_tags;

    foreach( $shortcode_tags as $shortcode_tag => $shortcode_func )
      if( strpos( $shortcode_tag, 'condition_' ) !== 0 )
        unset( $shortcode_tags[ $shortcode_tag ] );
  }

  private function _remove_temporary_shortcodes() {
    if( empty( $this->_temporary_shortcodes_registered ) )
      return;

    global $shortcode_tags;

    $shortcode_tags = $this->_temporary_shortcodes_registered;

    $this->_temporary_shortcodes_registered = [];
  }

  private function _set_recently_updated_courses() {
    $this->_filter_proxy_setup( [
      'wpep_primary_content_list_args'  => function( $args ) {
        $args['orderby']    = 'modified';
        $args['order']      = 'desc';

        if( !isset( $args['date_query'] ) )
          $args['date_query'] = [];

        $args['date_query'][] = [
          'column'  => 'post_modified',
          'after'   => date( 'Y-m-d H:i:s', strtotime( "-1 week" ) )
        ];

        return $args;
      }
    ] );

    $this->_all_updated_courses = wpep_content_type_course_instance()->library_get_grid_post_list();

    $this->_filter_proxy_remove();
  }

  public function send_email( $send_email_to, $email_title, $email_content, $is_simple = false ) {
    if( empty( $send_email_to ) )
      return false;

    $user_list = [];

    if( !is_array( $send_email_to ) ) {
      if( is_numeric( $send_email_to ) ) {
        $send_email_to = get_user_by( 'ID', $send_email_to );

        if( $send_email_to === null )
          return false;

        $user_list = [ $send_email_to ];
      } elseif( $send_email_to instanceof \WP_User ) {
        $user_list = [ $send_email_to ];
      }
    } else {
      $user_list = get_users( [
        'include' => $send_email_to
      ] );
    }

    if( empty( $user_list ) )
      return false;

    $this->_pre_register_temporary_shortcodes();

    $this->_register_temporary_conditional_shortcodes();

    $current_user_id = get_current_user_id();

    if( !$is_simple ) {
      if( strpos( $email_content, 'list_courses_started' ) === false
        && strpos( $email_content, 'condition_has_started_courses' ) === false )
        $this->loop_courses_started = null;

      if( strpos( $email_content, 'list_courses_completed' ) === false
        && strpos( $email_content, 'condition_has_completed_courses' ) === false )
        $this->loop_courses_completed = null;

      if( strpos( $email_content, 'list_courses_recommended' ) === false
          && strpos( $email_content, 'condition_has_recommended_courses' ) === false )
        $this->loop_courses_recommended = null;

      if( strpos( $email_content, 'number_lessons_completed' ) === false )
        $this->loop_lessons_completed_count = null;

      if( strpos( $email_content, 'list_courses_updated' ) !== false && empty( $this->_all_updated_courses ) )
        $this->_set_recently_updated_courses();
      else
        $this->_all_updated_courses = [];
    }

    foreach( $user_list as $user ) {
      if( $current_user_id != $user->ID )
        wp_set_current_user( $user->ID );

      $this->loop_user = $user;

      if( !$is_simple ) {
        $past_started_courses = wpep_content_type_course_instance()->user->get_started_course_list( $user->ID, [ false, strtotime( "-1 week" ) ], true );

        $this->loop_courses_updated = [];

        if( $this->loop_courses_started !== null )
          $this->loop_courses_started   = wpep_content_type_course_instance()->user->get_started_course_list( $user->ID, [ strtotime( "-1 week" ), time() ], true );

        if( $this->loop_courses_completed !== null )
          $this->loop_courses_completed = wpep_content_type_course_instance()->user->get_completed_course_list( $user->ID, [ strtotime( "-1 week" ), time() ] );

        if( $this->loop_lessons_completed_count !== null )
          $this->loop_lessons_completed_count = wpep_content_type_course_instance()->user->get_completed_lesson_count( $this->loop_user->ID, strtotime( "-1 week" ), time() );

        if( $this->loop_courses_recommended !== null )
          $this->loop_courses_recommended = wpep_content_type_course_instance()->user->get_recommended_ids( $this->loop_user->ID, 4 );

        if( !empty( $this->_all_updated_courses ) ) {
          foreach( $this->_all_updated_courses as $updated_course ) {
            if( !WPEP_Controller::instance()->contentLibrary->has_access_to_post( $updated_course->ID ) )
              continue;

            if( !in_array( $updated_course->ID, $past_started_courses ) )
              continue;

            $this->loop_courses_updated[] = $updated_course;
          }
        }
      }

      $current_email_content = do_shortcode( $email_content );

      $current_email_content = wpautop( $current_email_content );

      $this->_register_temporary_content_shortcodes();

      $current_email_title   = do_shortcode( $email_title );
      $current_email_content = do_shortcode( $current_email_content );

      $this->_remove_temporary_content_shortcodes();

      WPEP_Controller::instance()->email->send( $user, $current_email_title, $current_email_content, false );
    }

    $this->_remove_temporary_shortcodes();

    if( $current_user_id != get_current_user_id() )
      wp_set_current_user( $current_user_id, wpep_content_user_profile_format_name( $current_user_id ) );

    $this->loop_courses_started = [];
    $this->loop_courses_completed = [];
    $this->loop_courses_recommended = [];
    $this->loop_lessons_completed_count = 0;

    return true;
  }
  
  public function get_available_shortcode_list( $full_list = true, $user_registration = false ) {
     $response = '<h2>' . __( "General ShortCodes", "wpep" ) . '</h2>';
     $response .= '<ul>' .
                    '<li>[name] - ' . __( "The name of the person that receives this email.", "wpep" ) . '</li>' .
                    '<li>[email_address] - ' . __( "The email address of the person that receives this email.", "wpep" ) . '</li>';

     if( $user_registration )
       $response .= '<li>[password] - ' . __( "The password generated for this user account", "wpep" ) . '</li>';

     $response .=   '<li>[date] - ' . __( "The current date when the email is sent.", "wpep" ) . '</li>' .
                    '<li>[date_time] - ' . __( "The current date & time when the email is sent.", "wpep" ) . '</li>' .
                    '<li>[site_url] - ' . __( "The current website url.", "wpep" ) . '</li>';
     $response .= '</ul>';

     if( !$full_list )
       return $response;

     $response .= '<h2>' . __( "Content ShortCodes", "wpep" ) . '</h2>';

     $response .= '<ul>' .
                    '<li>[number_lessons_completed] - ' . __( "The number of lessons completed this week.", "wpep" ) . '</li>' .
                    '<li>[list_courses_completed] - ' . __( "List of Courses Completed this Week.", "wpep" ) . '</li>' .
                    '<li>[list_courses_started] - '  . __( "List of Courses Started this Week.", "wpep" ) . '</li>' .
                    '<li>[list_courses_recommended] - '  . __( "List of Courses Recommended, the User must have access to the course, and not start it yet.", "wpep" ) . '</li>' .
                    '<li>[list_courses_updated] - '  . __( "List of Courses that have been updated in the past week.", "wpep" ) . '</li>' .
                  '</ul>';
     $response .= '<h2>' . __( "Condition ShortCodes, will display the content within, if the condition is met", "wpep" ) . '</h2>';
     $response .= '<ul>' .
                    '<li>[condition_has_completed_courses] - ' . __( "The user has to complete at least 1 course.", "wpep" ) . '</li>' .
                    '<li>[condition_has_started_courses] - ' . __( "The user has to start at least 1 course.", "wpep" ) . '</li>' .
                    '<li>[condition_has_recommended_courses] - ' . __( "The user has at least 1 recommended course.", "wpep" ) . '</li>' .
                    '<li>[condition_has_activity] - ' . __( "The user had activity this week.", "wpep" ) . '</li>' .
                    '<li>[condition_no_activity] - ' . __( "The user didn't have any activity this week.", "wpep" ) . '</li>' .
                    '<li>[condition_has_updated_courses] - ' . __( "There is at least 1 course that has been updated in the past week.", "wpep" ) . '</li>';
     $response .= '</ul>';
    
    return  $response;
  }

  /**
   * @param $user_login
   * @param \WP_User $user
   */
  public function _on_user_login( $user_login, $user ) {
    WPEP_Entity_ActionTracking::instance()->track_event( 0, $user->ID, WPEP_ACTION_TRACKING_TYPE_LOGIN, "wordpress" );
  }

  public function _on_user_logout() {
    if( !is_user_logged_in() )
      return;

    WPEP_Entity_ActionTracking::instance()->track_event( 0, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_LOGOUT, "wordpress" );
  }

}