<?php

namespace WPEP;

use WPEP\Container\DataWalker;
use WPEP\Entity\SystemLog as WPEP_Entity_SystemLog;

/**
 * Class DataSystem
 * @package WPEP
 * @author Robert Rusu
 */
class DataSystem {

  use DataWalker;

  protected static $instance;

  public static function instance() {
    if (!isset(self::$instance))
      self::$instance = new self();

    return self::$instance;
  }

  private $_storage = [];

  public function setup() {
    add_action( 'wpep_cron_data_system_background_backup', [ $this, 'wpep_cron_daily' ] );
  }

  public function wpep_cron_daily() {
    if( wpep_get_setting( 'data-system-enable-background-backups' ) == false )
      return;

    $system_log = WPEP_Entity_SystemLog::instance( 'cron', 'background_backup', time() );

    $system_log->save();

    ignore_user_abort(false);

    $this->create_backup();

    $vault_item_expiration_days = intval( wpep_get_setting( 'data-system-backups-cap' ) );

    $system_log->completed_arguments = [
      'expiration_days' => $vault_item_expiration_days
    ];
    $system_log->completed_at = time();
    $system_log->save();

    if( $vault_item_expiration_days == 0 )
      return;

    $vault_items = $this->get_backup_list();
    $allow_deletion = false;

    foreach( $vault_items as $vault_item ) {
      $vault_age_days = ( ( time() - strtotime( $vault_item->post_date ) ) / ( 60 * 60 * 24 ) );
      if( $vault_age_days <= $vault_item_expiration_days ) {
        $allow_deletion = true;
        continue;
      }
      if( !$allow_deletion )
        continue;
      wp_delete_attachment( $vault_item->ID );
    }

  }

  public function data_import_array( $content_array ) {
    @set_time_limit(0);
    @ini_set('memory_limit', '2048M');

    if( isset( $content_array['settings' ] ) )
      $this->update_settings( $content_array['settings'] );

    if( isset( $content_array['users'] ) )
      $this->import_users( $content_array['users'] );

    if( isset( $content_array['taxonomies' ] ) ) {
      $content_array += ['taxonomies'];

      unset( $content_array['taxonomies'] );
    }

    $id_mapping = [];

    $id_mapping += wpep_content_taxonomy()->data_import_array( $content_array, $id_mapping );
    $post_id_mapping = wpep_content_type()->data_import_array( $content_array, $id_mapping );

    $id_mapping += $post_id_mapping;

    $post_id_mapping_plain = [];

    foreach( $post_id_mapping as $post_type => $post_ids )
      $post_id_mapping_plain += $post_ids;

    wpep_content_type()->data_import_array_post_connection( $content_array, $post_id_mapping_plain );

    $content_array = apply_filters( 'wpep_data_system_import_array', $content_array, $id_mapping, $post_id_mapping_plain );

    return $content_array;
  }

  public function create_backup() {
    if( !class_exists( '\ZipArchive' ) )
      return false;

    @set_time_limit(0);
    @ini_set('memory_limit', '2048M');

    $current_vault_file_path = $this->data_export_array_to_zip( $this->data_export_array() );

    $attachment = [
      'post_mime_type' => 'application/zip',
      'guid'           => $current_vault_file_path,
      'post_parent'    => 0,
      'post_title'     => sprintf( __( "%s Platform Backup %s", "wpep" ), WPEP_PLUGIN_NAME, wpep_format_date_with_time() ),
    ];

    $new_attachment_id = wp_insert_attachment( $attachment, '/vault-' . time() . '.zip', 0 );
    update_post_meta( $new_attachment_id, '_wp_attached_file', $current_vault_file_path );
    update_post_meta( $new_attachment_id, '_wpep_exported', wpep_format_date_with_time( time(), 'standard' ) );

    if( !function_exists( 'wp_generate_attachment_metadata' ) )
      require ( ABSPATH . 'wp-admin/includes/image.php' );

    $attach_data = wp_generate_attachment_metadata( $new_attachment_id, get_attached_file( $new_attachment_id ) );

    wp_update_attachment_metadata( $new_attachment_id,  $attach_data );

    return true;
  }

  /**
   * @param $data_export_array
   * @param string $prefix
   * @return string $current_vault_file_path
   */
  public function data_export_array_to_zip( $data_export_array, $prefix = 'vault-' ) {
    $current_vault_destination = wpep_get_upload_dir() . '/backups';
    $current_vault_file_path = $current_vault_destination . DIRECTORY_SEPARATOR . $prefix . time() . '.zip';

    _wpep_http_locked_directory_check( $current_vault_destination );

    $zip_archive = new \ZipArchive();
    $zip_archive->open( $current_vault_file_path, \ZipArchive::CREATE );

    $zip_archive->addFromString( 'data.json', json_encode( $data_export_array, JSON_PRETTY_PRINT ) );

    $attachment_list = $this->data_walker_get_assets( $data_export_array, $this->data_asset_map() );

    if( !empty( $attachment_list ) && is_array( $attachment_list ) ) {
      $attachment_meta_data_list = [];

      foreach( $attachment_list as $attachment_id => $attachment_url ) {
        if( !is_numeric( $attachment_id ) )
          continue;

        foreach( get_post_meta( $attachment_id ) as $attachment_post_meta_key => $attachment_post_meta_value ) {
          if( strpos( $attachment_post_meta_key, 'wpep_' ) !== 0 )
            continue;

          if( !isset( $attachment_meta_data_list[$attachment_id] ) || !is_array( $attachment_meta_data_list[$attachment_id] ) )
            $attachment_meta_data_list[$attachment_id] = [];

          $attachment_meta_data_list[ $attachment_id ][ $attachment_post_meta_key ] = $attachment_post_meta_value;
        }
      }

      $zip_archive->addFromString(
        'attachments.json',
        json_encode( [
          'base_url' => get_site_url(),
          'map'      => $attachment_list,
          'meta_map' => $attachment_meta_data_list
        ], JSON_PRETTY_PRINT ) );

      foreach( $attachment_list as $attachment_id => $attachment_url ) {
        $attached_file = get_attached_file( $attachment_id );
        $attached_file_content = @file_get_contents( ( $attached_file == false ? $attachment_url : $attached_file ) );

        if( $attached_file_content == false )
          continue;

        $zip_archive->addFromString(
          $attachment_id . ( substr( $attachment_url, strrpos( $attachment_url, '.', -1 ), strlen( $attachment_url ) ) ),
          $attached_file_content
        );
      }
    }

    $zip_archive->close();

    return $current_vault_file_path;
  }

  public function get_backup_list() {
    return get_posts( [
      'post_type'   => 'attachment',
      'meta_key'    => '_wpep_exported',
      'meta_compare'=> 'EXISTS',
      'numberposts' => -1,
      'orderby'     => 'date',
      'order'       => 'DESC'
    ] );
  }

  public function data_export_array() {
    $response = [];

    $response += wpep_content_taxonomy()->data_export_array();
    $response += wpep_content_type()->data_export_array();

    $response['users'] = $this->get_users();
    $response['settings'] = $this->get_settings();

    $response = apply_filters( 'wpep_data_system_export_array', $response );

    return $response;
  }

  public function data_asset_map() {
    $response = [];

    $response += wpep_content_taxonomy()->data_asset_map();
    $response += wpep_content_type()->data_asset_map();

    return $response;
  }

  public function get_users( $refresh = false ) {
    if( isset( $this->_storage['users'] ) && $refresh == false )
      return $this->_storage['users'];

    $response = [];

    $users = get_users( [ 'paged' => -1, 'orderby' => 'ID' ] );

    foreach( $users as $user ) {
      $response[ $user->ID ] = [
        'id'               => $user->ID,
        'username'         => $user->user_login,
        'email_address'    => $user->user_email,
        'first_name'       => $user->first_name,
        'last_name'        => $user->last_name,
        'user_status'      => $user->user_status,
        'description'      => get_the_author_meta( 'description', $user->ID ),
        'user_role'        => $user->get_role(),
        'meta_information' => []
      ];

      foreach( wpep_get_user_meta_information_list_keys() as $user_meta_key )
        $response[ $user->ID ][ 'meta_information' ][ $user_meta_key ] = get_user_meta( $user->ID, $user_meta_key, false );

    }

    $this->_storage['users'] = $response;

    return $this->_storage['users'];
  }

  public function get_user_ids_mapped_by_email( $refresh = false ) {
    if( isset( $this->_storage['user_email_map'] ) && $refresh == false )
      return $this->_storage['user_email_map'];

    $this->_storage['user_email_map'] = wpep_utility_map_to_array_assoc( $this->get_users(), 'email_address', 'id' );

    return $this->_storage['user_email_map'];
  }

  public function get_settings() {
    $response = [];

    $option_list = wpep_wpdb()->get_col( 'SELECT option_name FROM ' . wpep_wpdb()->options . ' WHERE option_name like "wpep_%"' );

    foreach( $option_list as $option_key ) {
      if( wpep_string_ends_with( $option_key, [
        '_report_pushed',
        '_migration_version',
        '_license',
        '_license_key',
        '_license_status',
        '_license_status_information',
        '_license_verification',
        '_license_bundled_products',
        '_crypto_key'
      ] ) )
        continue;

      $response[$option_key] = get_option( $option_key, '' );
    }

    return $response;
  }

  public function update_settings( $settings ) {
    foreach( $settings as $option_key => $option_value )
      update_option( $option_key, $option_value );
  }

  public function import_users( $import_users ) {
    $user_role_list_assoc = wpep_get_user_role_list_array_assoc( [] );
    $users = $this->get_users();

    $current_users = wpep_utility_map_to_array_assoc( $users , 'username', 'email_address' );

    $has_new = false;

    foreach( $import_users as $import_user_id => $import_user ) {
      if( in_array( $import_user['email_address'], $current_users ) )
        continue;

      if( !isset( $import_user['username'] ) )
        $import_user['username'] = $import_user['email_address'];

      if( isset( $current_users[ $import_user['username'] ] ) )
        continue;

      $user_id = wp_create_user( $import_user['username'], wp_generate_password(), $import_user['email_address'] );

      $has_new = true;

      wp_update_user( [
        'ID'          => $user_id,
        'first_name'  => $import_user['first_name'],
        'last_name'   => $import_user['last_name'],
        'description' => $import_user['description']
      ] );

      if( isset( $import_user[ 'user_role' ] ) && isset( $user_role_list_assoc[ $import_user[ 'user_role' ] ] ) ) {
        $user_instance = new \WP_User( $user_id );

        $user_instance->set_role( $import_user[ 'user_role' ] );
      }

      if( isset( $import_user[ 'meta_information' ] ) ) {
        foreach( $import_user[ 'meta_information' ] as $user_meta_key => $user_meta_entries ) {
          if( is_array( $user_meta_entries ) ) {
            foreach( $user_meta_entries as $user_meta_entry )
              add_user_meta( $user_id, $user_meta_key, $user_meta_entry );

            continue;
          }

          update_user_meta( $user_id, $user_meta_key, $user_meta_entries );
        }
      }
    }

    if( !$has_new )
      return;

    $this->get_users( true );
  }

  public function get_registered_users_count( $args = false ) {
    $query_entries_total = 'SELECT COUNT( * ) FROM `' . wpep_wpdb()->users . '` WHERE 1 = 1';

    if( isset( $args['time_from'] ) )
      $query_entries_total .= ' AND `user_registered` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to'] ) )
      $query_entries_total .= ' AND `user_registered` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    if( isset( $args[ 'user_ids' ] ) ) {
      if( is_numeric( $args[ 'user_ids' ] ) )
        $query_entries_total .= ' AND `ID` = ' . intval( $args[ 'user_ids' ] );
      else if( is_array( $args[ 'user_ids' ] ) )
        $query_entries_total .= ' AND `ID` IN (' . implode( ',', array_map( 'intval', $args[ 'user_ids' ] ) ) . ')';
    }

    return absint( wpep_wpdb()->get_var( $query_entries_total ) );
  }

  public function get_table_list_with_data() {
    $prefix_length = strlen( wpep_wpdb()->prefix );
    $prefix = wpep_wpdb()->prefix;

    $tables = wpep_wpdb()->get_col( 'SHOW TABLES' );
    $query_cap = ceil( count( $tables ) / 10 );
    $response = [];

    while( $query_cap > 0 && !empty( $loop_entries = array_slice( $tables, count( $response ), 10 ) ) ) {
      $loop_query = '';

      foreach( $loop_entries as $loop_entry ) {
        if( $loop_query != '' )
          $loop_query .= ' UNION ALL ';

        $loop_entry_clean = ( strpos( $loop_entry, $prefix ) === 0 ? substr( $loop_entry, $prefix_length ) : $loop_entry );

        $loop_query .= 'SELECT COUNT( * ) as table_entries, "' . $loop_entry_clean . '" as table_name FROM `' . $loop_entry . '`';
      }

      $results = wpep_wpdb()->get_results( $loop_query );

      foreach( $results as $result )
        $response[ $result->table_name ] = [
          'name'    => $result->table_name,
          'table'   => wpep_wpdb()->prefix . $result->table_name,
          'entries' => $result->table_entries
        ];

      $query_cap--;
    }

    return $response;
  }

  /**
   * @param $alias
   * @return \WPEP_Data_Third_Party_Import_Integration|false
   */
  public function get_third_party_import_instance( $alias ) {
    $importer_instance = $this->get_third_party_import_instances();

    return ( isset( $importer_instance[ $alias ] ) ? $importer_instance[ $alias ] : false );
  }

  /**
   * @param bool $refresh
   * @return \WPEP_Data_Third_Party_Import_Integration[]
   */
  public function get_third_party_import_instances( $refresh = false ) {
    if( !isset( $this->_storage[ 'third_party_import_instances' ] ) || $refresh )
      $this->_storage[ 'third_party_import_instances' ] = apply_filters( 'wpep_data_system_third_party_import_instances', [] );

    return $this->_storage[ 'third_party_import_instances' ];
  }

  public function is_third_party_import_available() {
    foreach( $this->get_third_party_import_instances() as $instance ) {
      foreach( $instance->get_content_types_list() as $content_type )
        if( !wpep_content_type()->is_enabled( $content_type ) )
          return false;

      if( $instance->import_available() || $instance->get_import_status() !== 'pending' )
        return true;
    }

    return false;
  }

}
