<?php

namespace WPEP\PlatformHealth;

use WPEP\Controller as WPEP_Controller;
use WPEPCronController as WPEP_Cron;

class Report {

  private $_get_report_count_users = [];

  public function get() {
    if ( ! function_exists( 'get_plugins' ) )
      require_once ABSPATH . 'wp-admin/includes/plugin.php';

    if( empty( $this->_get_report_count_users ) )
      $this->_get_report_count_users = count_users();

    $active_plugins   = $this->_get_active_plugin_list();
    $inactive_plugins = $this->_get_inactive_plugin_list();

    $response = [];

    if( WPEP_Cron::has_jobs() ) {
      $response['cron_jobs'] = [
        'name'    => 'Cron Jobs',
        'type'    => 'alert',
        'entries' => $this->_get_cron_jobs_report()
      ];
    }

    $response['server_environment'] = [
      'name'    => 'Server Environment',
      'type'    => 'alert',
      'entries' => $this->_get_server_environment()
    ];

    $response['wordpress_environment'] = [
      'name'    => 'WordPress Environment',
      'type'    => 'alert',
      'entries' => $this->_get_wordpress_environment()
    ];

    $response['database_environment'] = [
      'name'    => 'Database Environment',
      'type'    => 'alert',
      'entries' => $this->_get_database_environment()
    ];

    $response['database_tables'] = [
      'name'    => 'Database Tables',
      'type'    => 'normal',
      'entries' => $this->_get_database_tables()
    ];

    $custom_template_data = $this->_get_custom_template_data();

    if( !empty( $custom_template_data ) ) {
      $response['custom_template_data'] = [
        'name'    => 'Custom Templates',
        'type'    => 'normal',
        'entries' => $custom_template_data
      ];
    }

    $response['wpep_role_list'] = [
      'name'    => sprintf( __( '%s Role List Status', 'wpep'), WPEP_PLUGIN_NAME ),
      'type'    => 'alert',
      'entries' => $this->_get_user_role_list()
    ];

    $response['wpep_settings'] = [
      'name'    => sprintf( __( '%s Settings', 'wpep'), WPEP_PLUGIN_NAME ),
      'type'    => 'normal',
      'entries' => $this->_get_wpep_settings()
    ];

    $response['wpep_post_settings'] = [
      'name'    => sprintf( __( '%s Post Types Settings', 'wpep'), WPEP_PLUGIN_NAME ),
      'type'    => 'normal',
      'entries' => $this->_get_post_type_settings()
    ];

    if( in_array( WPEP_POST_TYPE_COURSE, wpep_content_type()->library_post_types() ) ) {
      $course_data_integrity = $this->_get_course_data_integrity();

      if( !empty( $course_data_integrity ) ) {
        $response['wpep_course_settings_data'] = [
          'name'    => sprintf( __( '%s Course Data', 'wpep'), WPEP_PLUGIN_NAME ),
          'type'    => 'normal',
          'entries' => $course_data_integrity
        ];
      }

      $response['wpep_course_data'] = [
        'name'    => sprintf( __( '%s User Course Data', 'wpep'), WPEP_PLUGIN_NAME ),
        'type'    => 'normal',
        'entries' => $this->_get_course_user_data_integrity()
      ];
    }

    $response['license_status'] = [
      'name'    => 'License Status & Verification',
      'type'    => 'alert',
      'entries' => $this->_get_license_status_list()
    ];

    $response['active_plugin_list'] = [
      'name'    => 'Active Plugins - ' . count( $active_plugins ) . ' / ' . ( count( $active_plugins ) + count( $inactive_plugins ) ),
      'type'    => 'alert',
      'entries' => $active_plugins
    ];

    $response['inactive_plugin_list'] = [
      'name'    => 'Inactive Plugins - ' . count( $inactive_plugins ) . ' / ' . ( count( $active_plugins ) + count( $inactive_plugins ) ),
      'type'    => 'alert',
      'entries' => $inactive_plugins
    ];

    return $response;
  }

  public function get_dashboard() {
    $response = [];

    if( WPEP_Cron::has_jobs() ) {
      $response['cron_jobs'] = [
        'name'    => 'Cron Jobs',
        'type'    => 'alert',
        'entries' => $this->_get_cron_jobs_report()
      ];
    }

    $response['server_environment'] = [
      'name'    => 'Server Environment',
      'type'    => 'alert',
      'entries' => $this->_get_server_environment()
    ];

    $response['database_environment'] = [
      'name'    => 'Database Environment',
      'type'    => 'alert',
      'entries' => $this->_get_database_environment()
    ];

    $custom_template_data = $this->_get_custom_template_data();

    if( !empty( $custom_template_data ) ) {
      $response['custom_template_data'] = [
        'name'    => 'Custom Templates',
        'type'    => 'normal',
        'entries' => $custom_template_data
      ];
    }

    $response['wpep_role_list'] = [
      'name'    => sprintf( __( '%s Role List Status', 'wpep'), WPEP_PLUGIN_NAME ),
      'type'    => 'alert',
      'entries' => $this->_get_user_role_list()
    ];

    return $response;
  }

  private function _get_wordpress_environment() {
    $wp_memory_limit = wpep_let_to_num( WP_MEMORY_LIMIT );
    if ( function_exists( 'memory_get_usage' ) ) {
      $wp_memory_limit = max( $wp_memory_limit, wpep_let_to_num( @ini_get( 'memory_limit' ) ) );
    }

    $has_wp_cron          = ! ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON );
    $has_wp_debug         = ( defined( 'WP_DEBUG' ) && WP_DEBUG );

    $my_theme = wp_get_theme();

    $disable_xml_rpc_text = ( wpep_get_setting_flag( 'disable-xmlrpc' )
      ? sprintf( 'Yes - %s Setting' , WPEP_PLUGIN_NAME )
      : ( apply_filters( 'xmlrpc_enabled', true ) == false ? 'Yes - from Filter' : false )
    );

    $wp_get_ready_cron_jobs_total = 0;
    $wp_get_ready_cron_jobs = wp_get_ready_cron_jobs();

    foreach( $wp_get_ready_cron_jobs as $t => $cronhooks ) {
      $wp_get_ready_cron_jobs_total += count( $cronhooks );
    }

    $response = [
      'home_url'                  => get_option( 'home' ),
      'site_url'                  => get_option( 'siteurl' ),
      'wpep_version'              => WPEP_VERSION,
      'wp_version'                => get_bloginfo( 'version' ),
      'active_theme'              => sprintf( "%s on version %s", $my_theme->get( 'Name' ), $my_theme->get( 'Version' ) ),
      'wp_multisite'              => is_multisite(),
      'language'  => get_locale(),
      'timezone'  => get_option( 'timezone_string', __( "Default", "wpep" ) ) . ' ( GMT ' . get_option('gmt_offset' ) . ' ) ' . wpep_format_date_with_time(),
      'initial_db_version' => get_option( 'initial_db_version', 0 ),
      'default_user_role'  => get_option( 'default_role' ),
      'registered_users'   => $this->_get_report_count_users[ 'total_users' ],
      'wp_get_ready_cron_jobs_timestamps' => count( $wp_get_ready_cron_jobs ),
      'wp_get_ready_cron_jobs_total'      => $wp_get_ready_cron_jobs_total,
      'wp_memory_limit'           => [
        'status'  => ( $wp_memory_limit < 67108864 ? 'warning' : 'success' ),
        'type'    => 'storage_size',
        'value'   =>  $wp_memory_limit
      ],
      'wp_cron'                   => [
        'status'  => ( $has_wp_cron ? 'info' : 'success' ),
        'value'   => $has_wp_cron
      ],
      'wp_debug'                  => [
        'status'  => ( $has_wp_debug ? 'info' : 'success' ),
        'value'   => $has_wp_debug
      ]
    ];

    if( $disable_xml_rpc_text !== false )
      $response[ 'disable_xml_rpc' ] = [
        'status'  => 'info',
        'value'   => $disable_xml_rpc_text
      ];

    return $response;
  }

  private function _get_server_environment() {
    // Figure out cURL version, if installed.
    $curl_version = '';
    if ( function_exists( 'curl_version' ) ) {
      $curl_version = curl_version();
      $curl_version = $curl_version['version'] . ', ' . $curl_version['ssl_version'];
    }

    $mysql_version = ( ! empty( wpep_wpdb()->is_mysql ) ? wpep_wpdb()->db_version() : '' );

    $post_max_size   = wpep_let_to_num( ini_get( 'post_max_size' ) );
    $max_upload_size = wp_max_upload_size();
    $server_info     = $_SERVER['SERVER_SOFTWARE'];

    return [
      'server_info'               => [
        'status'  => ( $server_info == '' ? 'info' : 'success' ),
        'value'   => $server_info
      ],
      'php_version'               => [
        'status'  => ( version_compare( phpversion(), '7.1', '<' ) ? 'danger' : ( version_compare( phpversion(), '7.2', '<' ) ? 'warning' : 'success' ) ),
        'value'   => phpversion()
      ],
      'php_post_max_size'         => [
        'status'  => ( $post_max_size >= 64000000 ? 'success' : 'warning' ),
        'type'    => 'storage_size',
        'value'   => $post_max_size
      ],
      'php_max_execution_time'    => [
        'status'  => ( ini_get( 'max_execution_time' ) >= 30 ? 'success' : 'warning' ),
        'type'    => 'php_var',
        'value'   => ini_get( 'max_execution_time' )
      ],
      'php_max_input_vars'        => [
        'status'  => ( ini_get( 'max_input_vars' ) >= 3000 ? 'success' : ( ini_get( 'max_input_vars' ) >= 1000 ? 'info' : 'warning' ) ),
        'type'    => 'php_var',
        'value'   => ini_get( 'max_input_vars' )
      ],
      'max_upload_size'           => [
        'status'  => ( $max_upload_size >= 16000000 ? 'success' : 'warning' ),
        'type'    => 'storage_size',
        'value'   => $max_upload_size
      ],
      'mysql_version'             => [
        'status'  => ( version_compare( $mysql_version, '5.6', '<' ) ? ( version_compare( $mysql_version, '5.2', '<' ) ? 'danger' : 'info' ) : 'success' ),
        'value'   => $mysql_version
      ],
      'curl_version'              => [
        'status'  => ( $curl_version == '' ? 'danger' : 'success' ),
        'value'   => $curl_version
      ],
      'file_get_contents'         => [
        'status'  => ( function_exists( 'file_get_contents' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'file_get_contents' ) ? 'Available' : 'Not Available' )
      ],
      'file_put_contents' => [
        'status'  => ( function_exists( 'file_put_contents' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'file_put_contents' ) ? 'Available' : 'Not Available' )
      ],
      'allow_url_fopen' => [
        'status'  => ( ini_get('allow_url_fopen') ? 'success' : 'warning' ),
        'type'    => 'php_var',
        'value'   => ( ini_get('allow_url_fopen') ? 'Available' : 'Not Available' )
      ],
      'imagettftext'      => [
        'status'  => ( function_exists( 'imagettftext' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'imagettftext' ) ? 'Available' : 'Not Available' )
      ],
      'imagejpeg'         => [
        'status'  => ( function_exists( 'imagejpeg' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'imagejpeg' ) ? 'Available' : 'Not Available' )
      ],
      'gzip'   => [
        'status'  => ( is_callable( 'gzopen' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( is_callable( 'gzopen' ) ? 'Available' : 'Not Available' )
      ],
      'mbstring' => [
        'status'  => ( extension_loaded( 'mbstring' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( extension_loaded( 'mbstring' ) ? 'Available' : 'Not Available' )
      ]
    ];
  }

  private function _get_cron_jobs_report() {
    $response = [];

    foreach( WPEP_Cron::get_job_list_report() as $job_name => $data ) {
      $status = 'success';

      if( $data['scheduled'] === 0 )
        $status = 'danger';
      elseif( $data['scheduled'] < time() )
        $status = ( time() - $data['scheduled'] > HOUR_IN_SECONDS ? 'danger' : 'warning' );

      $response[ $job_name ] = [
        'status'  => $status,
        'value'   => ( $data['scheduled'] === 0 ? __( "Missing", "wpep" ) : ( wpep_format_date_with_time( $data['scheduled'] ) . ' - ' . ( empty( $data['schedule_name' ] ) ? '???' : $data['schedule_name' ] ) ) )
      ];
    }

    return $response;
  }

  private function _get_database_environment() {
    $response = [];

    foreach( WPEP_Controller::instance()->migrationLibrary->get_migration_instances() as $migration_instance ) {
      $response[ $migration_instance->namespace . '_migration' ] = [
        'status'  => ( $migration_instance->current_version == $migration_instance->current_version_available ? 'success' : 'danger' ),
        'type'    => 'version',
        'value'   => $migration_instance->current_version
      ];

      foreach( $migration_instance->get_table_list() as $table_name ) {
        $table_exists = wpep_wpdb()->get_var("SHOW TABLES LIKE '$table_name'") == $table_name;

        $response[ $table_name ] = [
          'status'  => ( $table_exists ? 'success' : 'danger' ),
          'type'    => 'table',
          'value'   => ( $table_exists ? 'ok' : 'missing' )
        ];
      }
    }

    return $response;
  }

  private function _get_database_tables() {
    $response = [];

    $table_list = wpep_data_system()->get_table_list_with_data();

    foreach( $table_list as $table_name => $table_data ) {
      $response[ $table_name ] = [
        'type'    => 'table',
        'value'   => $table_data[ 'entries' ]
      ];
    }

    return $response;
  }

  private function _get_custom_template_data() {
    $response = [];

    $template_path      = WPEP_BASE_PATH . '/templates/';
    $template_file_list = wpep_file_system_get_file_list( $template_path );

    foreach( $template_file_list as $template_file_path ) {
      $path = substr( $template_file_path, strlen( $template_path ) );

      if( wpep_string_ends_with( $path, [ '.DS_Store' ] ) || in_array( $path, [ 'index.php' ] ) )
        continue;

      $located_template = WPEP_Controller::instance()->template->locate_template( $path );
      $located_template = str_replace('\\', '/', $located_template );

      if( $located_template === $template_file_path )
        continue;

      $response[ $path ] = [
        'type'  => 'template',
        'value' => substr( $located_template, strlen( ABSPATH ) ) . ' - ' . wpep_format_date_with_time( filemtime( $located_template ) )
      ];
    }

    return $response;
  }

  private function _get_wpep_settings() {
    $response = [
      'wpep_theme'                       => wpep_get_setting( 'wpep-theme' ),
      'primary_color'                    => wpep_get_setting( 'wpep-custom-color' ),
      'secondary_color'                  => wpep_get_setting( 'wpep-custom-color-secondary' ),
      'index_page_start_text_color'      => wpep_get_setting( 'index-page-start-text-color' ),
      'index_page_start_background'      => wpep_get_setting( 'index-page-start-text-color-background' ),
      'internal_auth_status_level'       => wpep_get_setting_flag( 'user-auth-status-level' ),
      'wpep_editor_integration_enabled'  => $this->_bool_val( wpep_get_setting_flag( 'admin-editor-integration' ) ),
      'custom_css_by_file_enabled'       => $this->_bool_val( wpep_get_setting_flag( 'wpep-custom-css-by-file' ) ),
      'assessments_enabled'              => $this->_bool_val( wpep_assessments_enabled() ),
      'timing_options_enabled'           => $this->_bool_val( wpep_get_setting_flag( 'enable-course-timing-option' ) ),
      'single_category_view_enabled'     => $this->_bool_val( wpep_get_setting_flag( 'index-page-single-category-primary-view' ) ),
      'lesson_incomplete_enabled'        => $this->_bool_val( wpep_get_setting_flag( 'enable-lesson-incomplete-option' ) ),
      'admin_course_save_alert_enabled'  => $this->_bool_val( wpep_get_setting_flag( 'admin-course-save-alert' ) ),
      'lesson_wp_editor_preload_enabled' => $this->_bool_val( wpep_get_setting_flag( 'lesson-wp-editor-preload' ) ),
      'lesson_open_on_add_enabled'       => $this->_bool_val( wpep_get_setting_flag( 'lesson-wp-editor-enabled-new' ) ),
      'wpep_editor_setting'              => wpep_get_setting_flag( 'wpep-editor-setup' ),
      'new_course_sections'              => wpep_get_setting_flag( 'admin-course-preset-sections' ),
      'new_course_lessons'               => wpep_get_setting_flag( 'admin-course-preset-lessons' )
    ];

    return $response;
  }

  private function _get_post_type_settings() {
    $response = [];

    foreach( wpep_content_type()->standardContentTypes as $standardContentType ) {
      if( $standardContentType->get_post_type() === WPEP_POST_TYPE_ASSESSMENT )
        continue;

      $alias = wpep_label_to_alias( $standardContentType->get_plural_name() );

      $response[ $alias . '_created' ] = wpep_utility_array_sum( wp_count_posts( $standardContentType->get_post_type() ) );
      if( $standardContentType->is_publicly_queryable() ) {
        $response[ $alias . '_archive' ] = $this->_bool_val( wpep_get_setting_flag( 'content-type-' . $standardContentType->get_post_type() . '-enable-archive' ) );
        $response[ $alias . '_slug' ]    = wpep_get_setting( 'content-type-' . $standardContentType->get_post_type() . '-rewrite-slug' );
      }
    }

    if( wpep_assessments_enabled() ) {
      $response[ 'assessments_created' ] = wpep_utility_array_sum( wp_count_posts( WPEP_POST_TYPE_ASSESSMENT ) );
      $response[ 'assessments_slug' ]    = wpep_get_setting( 'content-type-assessment-rewrite-slug' );
    }

    $response['categories_slug']    = wpep_get_setting( 'rewrite-slug-categories' );
    $response['categories_created'] = wp_count_terms( WPEP_TAXONOMY_CATEGORY );

    return $response;
  }

  private function _get_license_status_list() {
    $response = [];

    foreach( WPEP_Controller::instance()->licenseLibrary->get_license_list() as $integration_alias => $license_information ) {
      $response[$integration_alias] = [
        'type'   => 'table',
        'status' => ( $license_information['is_valid'] ? 'success' : 'danger' ),
        'value'  => ( $license_information['license'] === '' ? 'missing' : ( $license_information['is_valid'] ? 'valid' : 'invalid' ) . ' - ' . ( $license_information['last_verification'] == 0 ? 'never' : wpep_format_date_with_time( $license_information['last_verification'] ) ) )
      ];
    }

    return $response;
  }

  private function _get_user_role_list() {
    $response = [];

    foreach( WPEP_Controller::instance()->capabilities->get_role_list() as $role_key => $role_status )
      $response[$role_key] = [
        'type'   => 'user_role',
        'status' => ( $role_status == 'ok' ? 'success' : 'danger' ),
        'value'  =>  $role_status . (
          isset( $this->_get_report_count_users[ 'avail_roles' ][ $role_key ] )
            ? ' - ' . (
            $this->_get_report_count_users[ 'avail_roles' ][ $role_key ] === 1
              ? __( "1 user", "wpep" )
              : sprintf( __( "%s users", "wpep" ), $this->_get_report_count_users[ 'avail_roles' ][ $role_key ] )
            )
            : ''
          )
      ];

    return $response;
  }

  private function _get_course_data_integrity() {
    $entries = wpep_wpdb()->get_results( 'SELECT COUNT( * ) as course_count, meta_value as course_type FROM `' . wpep_wpdb()->postmeta . '` WHERE `meta_key` = "course_type" GROUP BY `meta_value`' );

    if( empty( $entries ) )
      return [];

    $response = [];

    foreach( $entries as $entry ) {
      $response[ 'course_type_' . $entry->course_type ] = [
        'type'  => 'table',
        'value' => $entry->course_count
      ];
    }

    return $response;
  }

  private function _get_course_user_data_integrity() {
    $response = [];

    foreach( [
               WPEP_USER_COURSE_STARTED,
               WPEP_USER_COURSE_COMPLETED,
               WPEP_USER_COURSE_COMPLETED_AT,
               WPEP_USER_COURSE_IS_BOOKMARKED
             ] as $course_data_key ) {
      $response[ $course_data_key ] = [
        'type'  => 'table',
        'value' => count( wpep_content_type_course_instance()->user->data->get_value( $course_data_key, false, false, false, false, false ) )
      ];
    }

    $response[ WPEP_USER_COURSE_STARTED . '_users' ] = [
      'type'  => 'table',
      'value' => count(
        wpep_utility_map_to_array_assoc(
          wpep_content_type_course_instance()->user->data->get( WPEP_USER_COURSE_STARTED, false, false, false, false, false ),
          'user_id',
          'user_id'
        )
      )
    ];

    $response[ WPEP_USER_COURSE_COMPLETED . '_users' ] = [
      'type'  => 'table',
      'value' => count(
        wpep_utility_map_to_array_assoc(
          wpep_content_type_course_instance()->user->data->get( WPEP_USER_COURSE_COMPLETED, false, false, false, false, false ),
          'user_id',
          'user_id'
        )
      )
    ];

    return $response;
  }

  private function _get_active_plugin_list() {
    $response = [];

    foreach( get_plugins() as $plugin_path => $plugin_information ) {
      if( !is_plugin_active( $plugin_path ) )
        continue;

      $response[$plugin_information['Name']] = [
        'type'  => 'plugin',
        'value' => $plugin_information['Version']
      ];
    }

    return $response;
  }

  private function _get_inactive_plugin_list() {
    $response = [];

    foreach( get_plugins() as $plugin_path => $plugin_information ) {
      if( is_plugin_active( $plugin_path ) )
        continue;

      $response[$plugin_information['Name']] = [
        'type'  => 'plugin',
        'value' => $plugin_information['Version']
      ];
    }

    return $response;
  }

  private function _bool_val( $value ) {
    return !! $value;
  }

}