<?php

use WPEP\Controller as WPEP_Controller;
use WPEP\ContentType as WPEP_ContentType;
use WPEP\ContentTaxonomy as WPEP_ContentTaxonomy;
use WPEP\DataSystem as WPEP_DataSystem;
use WPEP\PlatformHealth as WPEP_PlatformHealth;
use WPEP\Request as WPEP_Request;
use WPEP\Settings as WPEP_Settings;

/**
 * @return WPEP_Controller
 */
function wpep_controller() {
  return WPEP_Controller::instance();
}

/**
 * @return WPEP_ContentType
 */
function wpep_content_type() {
  return WPEP_ContentType::instance();
}

/**
 * @return WPEP_ContentTaxonomy
 */
function wpep_content_taxonomy() {
  return WPEP_ContentTaxonomy::instance();
}

/**
 * @return WPEP_Request
 */
function wpep_request() {
  return WPEP_Request::instance();
}

/**
 * @return WPEP_DataSystem
 */
function wpep_data_system() {
  return WPEP_DataSystem::instance();
}

/**
 * @return WPEP_PlatformHealth
 */
function wpep_platform_health() {
  return WPEP_PlatformHealth::instance();
}

/**
 * @return string
 */
function wpep_get_current_url() {
  return WPEP_Request::instance()->get_current_url();
}

/**
 * @return WPEP_Settings
 */
function wpep_settings_instance() {
  return WPEP_Settings::instance();
}

/**
 * @param $option_name
 * @param string $option_default
 * @return string|array
 */
function wpep_get_setting( $option_name, $option_default = '' ) {
  return WPEP_Settings::instance()->get_option( $option_name, $option_default );
}

/**
 * @param $option_name
 * @return string
 */
function wpep_get_setting_translated( $option_name ) {
  return WPEP_Settings::instance()->get_option_translated( $option_name );
}

/**
 * @param $option_name
 * @return string
 */
function wpep_get_setting_flag( $option_name ) {
  return intval( WPEP_Settings::instance()->get_option( $option_name ) );
}

/**
 * @param $option_name
 * @param $option_value
 * @param bool $is_silent
 * @return bool
 */
function wpep_set_setting( $option_name, $option_value, $is_silent = false ) {
  return WPEP_Settings::instance()->update_settings( [
    $option_name => $option_value
  ], $is_silent );
}

/**
 * @param $option_name
 * @return bool
 */
function wpep_delete_setting( $option_name ) {
  return WPEP_Settings::instance()->delete_setting( $option_name );
}


/**
 * @param $template_name
 * @param array $params
 * @return string
 */
function wpep_template_get( $template_name, $params = []) {
  return WPEP_Controller::instance()->template->get_template( $template_name, $params );
}

/**
 * @param $template_name
 * @param array $params
 */
function wpep_template_load( $template_name, $params = []) {
  WPEP_Controller::instance()->template->load_template( $template_name, $params );
}

/**
 * WPEP Proxy of WordPress get_header()
 */
function wpep_template_header() {
  WPEP_Controller::instance()->template->get_header();
}

/**
 * WPEP Proxy of WordPress get_footer()
 */
function wpep_template_footer() {
  WPEP_Controller::instance()->template->get_footer();
}

function wpep_has_sid_support() {
  return wpep_get_setting_flag( 'enable-sid-support' );
}

function wpep_has_embed_support() {
  return apply_filters( "wpep_has_embed_support_enabled", false );
}

function wpep_has_engagement_embed_support() {
  return wpep_has_embed_support() && wpep_get_setting_flag( 'user-content-embed-box-enabled' );
}

function wpep_get_placeholder_image() {
  return wpep_get_setting( 'wpep-placeholder-image' );
}

function wpep_feature_has_settings_panel( $feature_alias ) {
  $response = true;

  if( in_array( $feature_alias, [
    'discussion_direct_url',
    'discussion_reactions',
    'discussion_mentions'
  ] ) ) {
    if( $feature_alias === 'discussion_direct_url' )
      $response = ( defined( 'WPEP_PRE_RELEASE_DIRECT_URLS' ) && WPEP_PRE_RELEASE_DIRECT_URLS );

    if( $feature_alias === 'discussion_reactions' )
      $response = ( defined( 'WPEP_PRE_RELEASE_REACTIONS' ) && WPEP_PRE_RELEASE_REACTIONS );

    if( $feature_alias === 'discussion_mentions' )
      $response = ( defined( 'WPEP_PRE_RELEASE_MENTIONS' ) && WPEP_PRE_RELEASE_MENTIONS );
  }

  return apply_filters( 'wpep_feature_has_settings_panel', $response, $feature_alias );
}

function wpep_report_start_date() {
  $date = wpep_get_setting( 'reporting-start-date' );

  if( $date === '' || strtotime( $date ) === false ) {

    $time = strtotime( wpep_wpdb()->get_var( 'SELECT user_registered FROM `' . wpep_wpdb()->users . '` ORDER BY user_registered ASC LIMIT 1' ) );

    if( $time === false )
      $time = time() - YEAR_IN_SECONDS;

    $date = date( "Y-m-d", $time );

    wpep_set_setting( 'reporting-start-date', $date, true );
  }

  return $date;
}

/**
 * @param $tracking_code
 * @param null|int $tracking_id
 * @return string
 */
function wpep_prepare_tracking_code( $tracking_code, $tracking_id = null ) {
  if( !is_string( $tracking_code ) || $tracking_code == '' )
    return '';

  $tracking_id = ( $tracking_id === null ? rand( 100, 1000 ) : $tracking_id );

  return '<div id="wpep-tracking-code-' . $tracking_id . '" class="wpep-tracking-code">' . $tracking_code . '</div>';
}

/**
 * @param array $data
 * @return string
 */
function wpep_breadcrumb( $data = [] ) {
  $response = '';

  if( empty( $data ) )
    return $response;

  $response .= '<div class="wpep-breadcrumb-container">';
  $response .=    '<ul class="wpep-breadcrumb">';

  foreach( $data as $link => $label ) {
    $response .= '<li>';

    if( $link == false )
      $response .=  '<span>' . $label . '</span>';
    else
      $response .=  '<a href="' . $link . '">' . $label . '</a>';
    $response .= '</li>';
  }

  $response .=    '</ul>';
  $response .=    '<div class="clear"></div>';
  $response .= '</div>';

  return $response;
}

function wpep_frontend_button( $link, $attributes = [] ) {
  $text            = isset($attributes['text']) ? $attributes['text'] : $link;
  $position        = isset($attributes['position']) ? $attributes['position'] : 'left';
  $force_download  = isset($attributes['force_download']) ? $attributes['force_download'] : wpep_string_ends_with( $link, [ 'mp3', 'pdf', 'doc', 'mov', 'mp4', 'xlsx' ] );
  $download_name   = isset($attributes['download_name'])  ? $attributes['download_name'] : basename( $link );
  $container_class = isset($attributes['container_class'])  ? $attributes['container_class'] : '';

  $link = ( $link === '#' ? '#' : ( strpos( $link, "http" ) !== 0 ? 'http://' . $link : $link ) );

  $style = '';

  if( isset( $attributes['color'] ) )
    $style .= 'color : ' . $attributes['color'] . ' !important;';

  if( isset( $attributes['background_color'] ) )
    $style .= 'background-color : ' . $attributes['background_color'] . ' !important;';

  $response = '';

  $response .= '<p class="wpep-link-container wpep-link-container-' . $position . ( $container_class === '' ? '' : ' ' . $container_class ) . '">';
  $response .=    '<a class="wpep-button wpep-button-primary" href="' . $link . '" ';
  $response .=        ( $force_download ? ' download="' . $download_name . '" ' : '' );
  $response .=        ( $style !== '' ? ' style="' . $style . '" ' : '' );

  if( isset( $attributes[ 'simple_sync' ] ) && is_array( $attributes[ 'simple_sync' ] ) )
    $response .= ' data-ec-simple-sync="' . esc_attr( json_encode( $attributes[ 'simple_sync' ] ) ) . '"';

  if( isset( $attributes[ 'simple_download' ] ) && is_array( $attributes[ 'simple_download' ] ) )
    $response .= ' data-ec-simple-download="' . esc_attr( json_encode( $attributes[ 'simple_download' ] ) ) . '"';

  $response .=     '>';
  $response .=      $text;
  $response .=    '</a>';
  $response .= '</p>';

  return $response;
}

/**
 * @param int $progress_percent
 * @param bool $display_progress_number
 * @return string
 */
function wpep_progress_bar( $progress_percent = 0, $display_progress_number = false ) {
  $response = '';
  $response .= '<div class="wpep-progress-bar" data-wpep-progress-bar-has-number="' . ( $display_progress_number ? 1 : 0 ) . '">';

  if( $display_progress_number )
    $response .= '<div class="wpep-progress-bar-number">' . wpep_get_percentage( $progress_percent ) . '</div>';

  $response .=    '<div class="wpep-progress-bar-inner"';
  $response .=        ' role="progressbar"';
  $response .=        ' aria-valuenow="' . $progress_percent . '"';
  $response .=        ' aria-valuemin="0"';
  $response .=        ' aria-valuemax="100"';
  $response .=        ' data-wpep-progress-percentage-formatted="' . wpep_get_percentage( $progress_percent ) . '"';
  $response .=        ' style="width: ' . $progress_percent . '%"></div>';
  $response .= '</div>';

  return $response;
}

function wpep_migration_backup( $version_number, $table_list = [] ) {
  if( empty( $table_list ) )
    return;

  foreach( $table_list as $table_name ) {
    $destination = wpep_get_upload_dir_migration_backup() . '/' . $version_number . '-' . $table_name. '.json';

    $results = wpep_wpdb()->get_results( 'SELECT * FROM `' . wpep_wpdb()->prefix . $table_name . '`');

    if( !empty( $results ) ) {
      $information = json_encode( $results  );

      file_put_contents( $destination, $information );
    }
  }
}

/**
 * @param $current_step
 * @param $step_count
 * @return string
 */
function wpep_navigation_bar( $current_step, $step_count ) {
  $response = '';
  $response .= '<div class="wpep-navigation-bar-container">';
  $response .=    '<ul class="wpep-navigation-bar wpep-navigation-bar-items-' . $step_count . ' wpep-navigation-bar-current-item-' . $current_step . '">';

  for( $i = 1; $i <= $step_count; $i++ ) {
    $class  = 'wpep-navigation-bar-point';
    $class .= ( $i < $current_step ? ' wpep-navigation-bar-point-passed' : '' );
    $class .= ( $i == $current_step ? ' wpep-navigation-bar-point-current' : '' );

    $response .=    '<li class="' . $class . '">' . $i . '</li>';
  }

  $response .=    '</ul>';
  $response .= '</div>';

  return $response;
}

/**
 * @param string|bool $title
 * @param string $container_class
 * @param string $container_tag
 * @return string
 */
function wpep_spinner_loader( $title = false, $container_class = 'wpep-spinner-loader-container', $container_tag = 'div' ) {
  $content = '';

  $content  .= '<' . $container_tag . ' class="' . $container_class . '">';

  if( $title != false )
    $content .=       '<h2>' . $title . '</h2>';

  $content .=       '<span class="wpep-spinner-loader wpep-loading-container">' . __( "Loading...", "wpep" ) . '</span>';

  $content .= '</' . $container_tag . '>';

  return $content;
}

function wpep_mini_spinner_loader() {
  return '<span class="wpep-mini-spinner-loader"><i></i><i></i><i></i><i></i></span>';
}

/**
 * @param $content
 * @param $id
 * @param bool $field_name
 * @param $editor_args
 * @return bool | true if it uses the wp_editor
 */
function wpep_editor( $content, $id, $field_name = false, $editor_args = [] ) {
  $editor_mode = WPEP_Settings::instance()->get_option( 'wpep-editor-setup' );
  $field_name  = ( $field_name == false ? $id : $field_name );

  if( $editor_mode == 'textarea' ) {
    echo '<textarea id="' . $id . '" 
                    class="wpep-editor wpep-editor-setup-textarea"
                    style="width:100%;max-width:960px;min-height:200px;"
                    name="' . $field_name . '">' . $content . '</textarea>';

    return false;
  }

  wpep_editor_wp( $content, $id, $field_name, $editor_args );
  
  return true;
}


/**
 * @param $content
 * @param $id
 * @param bool $field_name
 * @param $wp_editor_args
 * @return bool | true if it uses the wp_editor
 */
function wpep_editor_wp( $content, $id, $field_name = false, $wp_editor_args = [] ) {
  $field_name  = ( $field_name == false ? $id : $field_name );
  $editor_mode = WPEP_Settings::instance()->get_option( 'wpep-editor-setup' );

  if( !isset( $wp_editor_args['textarea_name'] ) )
    $wp_editor_args['textarea_name'] = $field_name;

  if( !isset( $wp_editor_args['textarea_rows'] ) )
    $wp_editor_args['textarea_rows'] = 10;

  if( $editor_mode == 'visual' ) {
    $wp_editor_args['quicktags']      = [];
    $wp_editor_args['default_editor'] = 'tinymce';
  }

  $wp_editor_args['editor_class'] = ( isset( $wp_editor_args['editor_class'] ) ? $wp_editor_args['editor_class'] . ' ' : '' ) . 'wpep-editor wpep-editor-wp-editor wpep-editor-setup-' . $editor_mode;

  wp_editor( $content, $id, $wp_editor_args );

  return true;
}

/**
 * @param $title
 * @param array $head_row_data
 * @param array $body_row_list
 * @param bool|array $download_button
 * @param null $pagination_current
 * @param null $pagination_total
 * @param string $table_class
 */
function wpep_admin_display_table(
  $title,
  $head_row_data = [],
  $body_row_list = [],
  $download_button = false,
  $pagination_current = null,
  $pagination_total   = null,
  $table_class        = 'wp-list-table widefat fixed striped wpep-statistics-table'
) {
  if( !empty( $title ) )
    wpep_admin_display_table_title( $title, ( $download_button !== false && !empty( $body_row_list ) ? $download_button : false ) );

  if( empty( $body_row_list ) ) {
    echo '<div class="wpep-notification wpep-notification-warning">';
    echo    '<p>' . __( "There are no statistics available right now, please check back later.", "wpep" ) . '</p>';
    echo '</div>';

    return;
  }

  echo '<table class="' . $table_class . '">';
  echo    '<thead>';
  echo      '<tr>';

  foreach( $head_row_data as $current_entry_column ) {
    if( !is_array( $current_entry_column ) )
      $current_entry_column = [ 'value' => $current_entry_column ];

    echo    '<th class="' . ( isset( $current_entry_column['type'] ) ? $current_entry_column['type'] : 'normal' ) . '"' .
                ( isset( $current_entry_column['style'] ) ? 'style="' . $current_entry_column['style'] . '" ' : '' ) .
                ' >' .
                $current_entry_column['value'] .
            '</th>';
  }

  echo      '</tr>';
  echo    '<tbody>';


  foreach( $body_row_list as $current_entry_row ) {
    echo '<tr' . ( isset( $current_entry_row[ 'row_highlight' ] ) ? ' class="wpep-row-highlight wpep-row-highlight-' . $current_entry_row[ 'row_highlight' ] . '"' : '' ) . '>';

    foreach( $head_row_data as $current_entry_index => $current_entry_title ) {
      if( !isset( $current_entry_row[$current_entry_index] ) ) {
        echo '<td>-</td>';

        return;
      }

      $current_entry_column = $current_entry_row[$current_entry_index];

      if( !is_array( $current_entry_column ) )
        $current_entry_column = [ 'value' => $current_entry_column ];

      echo    '<td class="' . ( isset( $current_entry_column['type'] ) ? $current_entry_column['type'] : 'normal' ) . '" ' .
                ( isset( $current_entry_column['style'] ) ? 'style="' . $current_entry_column['style'] . '" ' : '' ) .
                ' >' .
        $current_entry_column['value'] .
        '</td>';
    }

    echo '</tr>';
  }

  echo    '</tbody>';
  echo '</table>';

  if( $pagination_current === null || $pagination_total === null )
    return;

  $base = remove_query_arg( 'pagination', wpep_get_current_url() );

  echo '<div class="wpep-pagination-links">';

  echo paginate_links( [
    'base'    => $base . '%_%',
    'format'  => '&pagination=%#%',
    'current' => $pagination_current,
    'total'   => $pagination_total
  ] );

  echo '</div>';
}

function wpep_admin_display_table_title( $title, $download_button = false ) {
  echo '<div class="wpep-statistics-table-information-top' . ( is_array( $title ) ? ' wpep-with-nav-tab' : '' ) . '">';
  if( is_array( $title )  ) {
    echo '<div class="wpep-nav-links">';
    foreach( $title as $tab_link => $tab_name ) {
      if( $tab_link == false ) {
        echo '<a class="wpep-nav-link wpep-nav-link-active" href="#">' . $tab_name . '</a>';
        continue;
      }

      echo '<a class="wpep-nav-link" href="' . $tab_link . '">' . $tab_name . '</a>';
      continue;
    }
    echo '</div>';
  } else {
    echo '<h2 class="wpep-statistics-title">' . $title . '</h2>';
  }

  if( $download_button !== false ) {
    echo    '<div class="wpep-operation-container">';

    if( is_array( $download_button ) ) {
      foreach( $download_button as $download_button_link => $download_button_text )
        echo '<a class="wpep-button wpep-button-primary" href="' . $download_button_link . '">' . $download_button_text . '</a>';
    } else {
      echo '<form method="POST"><input type="submit" class="wpep-button wpep-button-primary" name="wpep_download_statistics" value="' . __( "Download", "wpep" ) . '"></form>';
    }
    echo    '</div>';
  }
  echo '</div>';
}

function wpep_get_external_page_embed_copy_button(
  $post_id,
  $permalink = false,
  $class     = 'wpep-button wpep-button-primary',
  $button_text        = '',
  $button_text_copied = ''
) {
  $permalink          = ( $permalink === false ? get_permalink( $post_id ) : $permalink );
  $button_text        = ( $button_text === ''  ? __( "Copy Embed Code", "wpep" ) : $button_text );
  $button_text_copied = ( $button_text_copied === '' ? __( "Copied !", "wpep" ) : $button_text_copied );

  return '<a class="' . $class . '"
               data-wpep-embed-id="wpep-embed-' . $post_id . '"
               data-wpep-embed-link-copy="' . add_query_arg( 'wpep-embed', 'true', $permalink ) . '"
               data-wpep-embed-link-copied="' . $button_text_copied . '">' . $button_text . '</a>';
}

/**
 * This is not by default used on the frontend, and the option Enforce Ordering is required to be checked.
 * @param $term_list
 * @return mixed
 */
function wpep_term_list_ordering( $term_list ) {
  if( !is_array( $term_list ) )
    return $term_list;

  if( empty( $term_list ) )
    return $term_list;

  foreach( $term_list as $term_index => $term )
    $term_list[$term_index]->wpep_order = intval( get_term_meta( $term->term_id, 'wpep_order', true ) );

  usort( $term_list, function( $a, $b ) {
    if ( $a->wpep_order == $b->wpep_order )
      return 0;

    return ( $a->wpep_order < $b->wpep_order ) ? -1 : 1;
  });

  return $term_list;
}

function wpep_get_theme_list_as_entity_layout_option() {
  $theme_list = WPEP_Controller::instance()->templateStyle->get_theme_list();

  $response = [
    ''  =>  sprintf( __( "Default Theme - %s", "wpep" ), $theme_list[ WPEP_Controller::instance()->templateStyle->theme ]['name'] )
  ];

  foreach( $theme_list as $theme_key => $theme_information )
    $response[ $theme_key ] = $theme_information['name'];

  return $response;
}

/**
 * @return array
 */
function wpep_get_custom_style_old_files() {
  $upload_dir = wpep_get_upload_dir();

  $files = array_diff( scandir( $upload_dir ) , [ '.', '..' ] );

  foreach( $files as $file_index => $file ) {
    if( strpos( $file, 'custom-style-' . WPEP_VERSION ) === 0 || strpos( $file, 'custom-style' ) !== 0 ) {
      unset( $files[$file_index] );
    } elseif( is_dir( $upload_dir . DIRECTORY_SEPARATOR . $file ) ) {
      unset( $files[$file_index] );
    }
  }

  if( !empty( $files ) )
    return $files;

  if( !is_dir( $upload_dir . '/custom-stylesheet' ) )
    return [];

  $files = array_diff( scandir( $upload_dir . DIRECTORY_SEPARATOR . 'custom-stylesheet' ) , [ '.', '..' ] );

  foreach( $files as $file_index => $file )
    if( strpos( $file, 'style-' . WPEP_VERSION ) === 0
      || strpos( $file, 'style' ) !== 0 )
      unset( $files[$file_index] );

  return $files;
}

/**
 * @param string $prefix
 * @return bool|string
 */
function wpep_get_custom_style_file( $prefix = 'baseurl' ) {
  if( !file_exists( wpep_get_upload_dir() . DIRECTORY_SEPARATOR . wpep_get_custom_style_file_name() ) ) {
    $file_exists = wpep_refresh_custom_style_file();

    if( !$file_exists )
      return false;
  }

  if( $prefix === 'baseurl' )
    return wpep_get_upload_dir( $prefix ) . '/custom-stylesheet/' . wpep_get_custom_style_file_name();

  return wpep_get_upload_dir( $prefix ) . DIRECTORY_SEPARATOR . 'custom-stylesheet' . DIRECTORY_SEPARATOR . wpep_get_custom_style_file_name();
}

/**
 * @param bool $wpep_theme
 * @return bool|string
 */
function wpep_refresh_custom_style_file( $wpep_theme = false ) {
  if( $wpep_theme === true ) {
    foreach( WPEP_Controller::instance()->templateStyle->get_theme_list() as $theme_key => $theme_information )
      wpep_refresh_custom_style_file( $theme_key );

    return true;
  }

  $wpep_style_dir = wpep_get_upload_dir() . '/custom-stylesheet';

  if( !is_dir( $wpep_style_dir ) )
    mkdir($wpep_style_dir, 0755, true);

  $file_path = $wpep_style_dir . '/' . wpep_get_custom_style_file_name( $wpep_theme );

  if( file_exists( $file_path ) )
    @unlink( $file_path );

  file_put_contents( $file_path, wpep_controller()->templateStyle->get_custom_style( $wpep_theme ) );

  if( !file_exists( $file_path ) )
    return false;

  return $file_path;
}

/**
 * @param string $index
 * @return string
 */
function wpep_get_upload_dir( $index = 'basedir' ) {
  $upload_dir = wp_upload_dir();

  return $upload_dir[ $index ] . '/wpep';
}

function wpep_get_upload_dir_migration_backup( $index = 'basedir' ) {
  $dir_path = wpep_get_upload_dir() . '/migration-backup';

  _wpep_http_locked_directory_check( $dir_path );

  return wpep_get_upload_dir( $index ) . '/migration-backup';
}

function _wpep_http_locked_directory_check( $dir_path ) {
  if( is_dir( $dir_path ) )
    return;

  mkdir($dir_path, 0755, true);

  file_put_contents(
    $dir_path . '/.htaccess',
    "Options -Indexes" . "\n" .
    "deny from all" . "\n" .
    "<FilesMatch '\.(jpg|jpeg|png|gif|mp3|ogg)$'>" . "\n" .
    "Order Allow,Deny" . "\n" .
    "Allow from all" . "\n" .
    "</FilesMatch>"
  );
}

/**
 * @param bool|string $wpep_theme
 * @return string
 */
function wpep_get_custom_style_file_name( $wpep_theme = false ) {
  $wpep_theme = ( $wpep_theme === false ? WPEP_Controller::instance()->templateStyle->theme : $wpep_theme );

  return 'style-' . WPEP_VERSION . '-' . $wpep_theme . '.css';
}

function wpep_flag_setting_field_options() {
  return [
    0         => __( 'Disabled', "wpep" ),
    1         => __( 'Enabled', "wpep" ),
  ];
}

function wpep_register_chart_display( $target, $args = [] ) {
  $response = '';

  $response .= '<script type="text/javascript">';
  $response .= '  jQuery( document ).ready( function() {';
  $response .= '    eLearnCommerce.Library.Chart.Init("' . $target. '", ' . json_encode( $args ). ');';
  $response .= '  } );';
  $response .= '</script>';

  return $response;
}

function wpep_register_universal_interface() {
  wp_register_style('ec-style', WPEP_BASE_URL_PATH . 'interface/style.css', [], WPEP_VERSION );
  wp_enqueue_style('ec-style' );

  wp_register_script( 'elearncommerce', WPEP_BASE_URL_PATH . 'interface/app.min.js', [ 'jquery', 'wp-util' ], WPEP_VERSION, true );
  wp_localize_script( 'elearncommerce', 'ec_configuration', [
    'is_user_logged_in'  => '' . is_user_logged_in(),
    'ajax_url'      => admin_url( 'admin-ajax.php' ),
    'library_url'   => WPEP_BASE_URL_PATH . 'interface/',
    'component_url' => WPEP_BASE_URL_PATH . 'interface/component/',
    'date_format'   => ( !wpep_get_setting_flag( 'admin-library-datetimepicker-simple-format' ) ? get_option( 'date_format' ) : 'Y-m-d' ),
    'libraries'                     => [
      'dashicons'             => includes_url( "css/dashicons.min.css" ),
      'chartjs'               => [
        WPEP_BASE_URL_PATH . 'interface/third-party-library/html5-chart/chart.min.js'
      ],
      'select2'               => [
        WPEP_BASE_URL_PATH . 'interface/third-party-library/select2/select2.min.css',
        WPEP_BASE_URL_PATH . 'interface/third-party-library/select2/select2.full.min.js'
      ],
      'datetimepicker' => [
        WPEP_BASE_URL_PATH . 'interface/third-party-library/datetimepicker/jquery.datetimepicker.min.css',
        WPEP_BASE_URL_PATH . 'interface/third-party-library/datetimepicker/jquery.datetimepicker.full.js'
      ],
      'quill' => [
        WPEP_BASE_URL_PATH . 'interface/third-party-library/quill-editor/quill-theme.css',
        WPEP_BASE_URL_PATH . 'interface/third-party-library/quill-editor/quill.min.js'
      ],
      'quill_mention' => [
        WPEP_BASE_URL_PATH . 'interface/third-party-library/quill-editor/quill-mention.min.js'
      ],
      'jspdf' => [
        WPEP_BASE_URL_PATH . 'interface/third-party-library/jspdf/jspdf.min.js'
      ],
    ],
    'file_version' => ( defined( "WP_DEBUG" ) && WP_DEBUG ? time() : WPEP_VERSION )
  ] );

  wp_localize_script( 'elearncommerce', 'ec_lang', [
    'asset_failed_fetch'        => __( "Page scripts couldn't be loaded, check your internet connection & refresh the page.", "wpep" ),
    'xhr_failed_response'       => __( "This content couldn't be loaded, check your internet connection & refresh the page.", "wpep" ),
    'xhr_no_results_response'   => __( "No Results found to display.", "wpep" ),

    'layout_compact'            => 'Compact',
    'layout_expand'             => 'Expand',

    'pagination_loaded_entries' => __( "Currently loaded %s pages, included in the file exports.", "wpep" ),
    'pagination_load_all'       => __( "Load All", "wpep" ),
    'pagination_previous'       => __( "Previous", "wpep" ),
    'pagination_next'           => __( "Next", "wpep" ),

    'analytics_view_placeholder_search'  => __( "Search", "wpep" ),
    'analytics_view_custom_sources'  => __( "My Data Views", "wpep" ),

    'analytics_view_table'      => __( "Table", "wpep" ),
    'analytics_view_chart_line' => __( "Line Chart", "wpep" ),
    'analytics_view_chart_bar'  => __( "Bar Chart", "wpep" ),
    'analytics_view_chart_pie'    => __( "Pie Chart", "wpep" ),
    'analytics_view_chart_radar'  => __( "Radar Chart", "wpep" ),
    'analytics_view_chart_bar_horizontal'  => __( "Horizontal Chart", "wpep" ),

    'search_input_placeholder'  => wpep_get_setting_translated( 'search-input-placeholder' ),
    'search_no_results'         => wpep_get_setting_translated( 'search-no-results-found' )
  ] );
  wp_enqueue_script( 'elearncommerce' );
}